%% Example file for class HiMPC
%
% In this demo we use the HiMPC toolbox to show how to build a decentralized
% hierarchical controller for linear systems subject to linear constraints 
% on input and output variables.
%
%% Description
%
% For this demo we use the following system consting of four balls and 
% seven springs.
%
% <html>
% <IMG SRC="plant.jpg", height=400>
% </html>
%
% <latex>
% \includegraphics[width=.7\hsize]{plant.jpg}
% </latex>
%
% The process is composed by four masses moving vertically, each one 
% connected by a spring to a fixed ceiling, subject to damping due to 
% viscous friction with the environment, and connected to its neighbor mass
% by another spring.
% An input force u [Nm] can be applied to each mass by the
% lower-level controller. The state of the system is the vector
% $x$ collecting the vertical positions and accelerations of the masses,
% while the output $y$ is the vector that collects the positions.
% The constrained masses-springs system is an extension of the example
% presented [1]. However in this demo both layers have a decentralized
% structure, in particular four controllers at the higher level 
% independently process the user provided references and command
% their respective actuators.
%
% The final objective of this class functions is to compute the maximum 
% reference variation, element-wise, that the higher controller can apply
% in order to guarantee the system stability while 
% fulfilling the constraints.
% Such goal is achieved by ensuring that the system state vector starting 
% in a Maximal Output Admissible Set (MOAS) will be in a new MOAS at the 
% next execution of the higher level
% controller. The MOAS is obtained by tightening the output
% constraints by a fixed value, $\Delta y$ that is a tuning knob of the
% approach. 
% The computation of maximum reference variation is an optimization
% problem, which can be conveniently casted as a Mixed Integer Linear Program 
% (MILP), and solved via suitable solvers (we strongly suggest IBM, former 
% ILOG, Cplex).
%
% The simulations are performed using at the higher layer a set of
% independent hierarchical MPC (HiMPC) which adopt a linear formulation.
% The upper-layer controllers must embed constraints
% on the generated references, to ensure stability
% and constraint satisfaction. 
% In this example we impose constraints on plant outputs, namely the 
% positions of the balls depicted in figure. 
%
% [1] D. Barcelli, A. Bemporad and G. Ripaccioli, _Hierarchical MPC
% controller design for LTI systems_, 49th Control and Decision Conference,
% Atlanta, Georgia, USA, 2010. 

%% Simulation Example
%
close all
clear variables
clear globals
clc

% Computing data is very time consiming, set the variable below to 0 in
% case you want to recompute
precomputed_data=0;
showPlot=1;

%% Plant model
%
% Sample time 
TL=.25;
%%
% Creates the closed-loop model of the plant with the lower layer controller 
[sys N]=buildModelFriction(TL);
[A B C D] = ssdata(sys);
%%
% Number of states and inputs respectively
[Nx Ny]=size(B);
%
%% Element-wise state constraints
% The output constraints have to be expressed in state coordinates in order
% to compute the MOAS. 
% To avoid numerical problems the state constraints polytope is bounded, 
% hence the speed is limited to 10 [m/s] 
Xcon.min=[];
for i=1:N,Xcon.min=[Xcon.min;-.3;-10];end
Xcon.max=[];
for i=1:N,Xcon.max=[Xcon.max;1;10];end
%
%% Coupled constraints
% mass #2 position is forbidden to go below mass #1 position more than 0.3 
coupledCons(1).H=[-1 0 1 0];
coupledCons(1).K=[0.3];
coupledCons(2).H=[];
coupledCons(2).K=[];
coupledCons(3).H=[];
coupledCons(3).K=[];
%
%% The decentralization of the model is defined as:
dec(1).x=1:4;
dec(1).y=1:2;
dec(1).u=1:2;
dec(1).applied=1:2;
dec(2).x=[5 6];
dec(2).y=3;
dec(2).u=3;
dec(2).applied=3;
dec(3).x=[7 8];
dec(3).y=4;
dec(3).u=4;
dec(3).applied=4;
%%        
% $\Delta k$ :  Tightening of the output admissible set. Tuning knob.
DeltaK{1}=[.3;2;.3;2;.3;2;.3;2;0.12];
DeltaK{2}=[.3;2;.3;2];
DeltaK{3}=[.3;2;.3;2];

%% Truncated DCgain
%
% In order to have a decentralized structure stabilizing feedback must
% retain the structure, hence we simply nulify out-of-structure entries
E=inv(dcgain(sys));
nd=length(dec);
for i=1:nd
    % for each subsystem
    for h=dec(i).u
        for j=setdiff(1:4,dec(i).u)
            E(h,j)=0;
        end
    end
end
sys.B=sys.B*E;


if precomputed_data
    %%
    % Create the object, i.e. the HiMPC calss instance
    dobj = HiMPC(sys,dec,Xcon,DeltaK,coupledCons);
    %%
    % Compute the MOAS. For this purpose each sub-model is considered
    % independent and subject to an unknown but bounded disturbance. The
    % disturbance models the influence of the unmodeled dynamics, i.e. the
    % state components that are not present in that sub-system. The uncertainty
    % polytope is computed in an exact manner by considering the range of each
    % unmodeled state component and its influence with respect to state
    % components which belong to the current sub-model.
    % It follows that the invariant set is actually a Maximal Output
    % Admissible Robust Set.
    dobj=dobj.computeMOARS();
    %%
    % Plot both the MOAS with (blue) and without (red) the influence of the
    % other sub-models for each sub-model
    dobj.plotMOARS();
    %%
    % Solve the MILP for values of sample-time ratio from 1 to 50 and,
    % accordingly to the ratio, compute the maximal allowable $\Delta r$
    dobj=dobj.computeDeltaR();
    %%
    % Plot both $\Delta r(N)$ as a function of N, that is the higher-lower
    % sample times ratio, and $\Delta r(N)/N$ for each sub-model
    dobj.plotDeltaR();
    save Data/dobj_data dobj;
else
   load Data/mac_pro_f;
end
%% Pre-simulation considerations
%
% The first and the forth sub-models MOAS are completely equivalent as well as
% the corresponding $\Delta r(N)$ and $\Delta r(N)/N$. The same consideration 
% can be extended to the second and third sub-models. This is motivated by the
% internal symmetry of both the plant and the decentralization.
%

%% Higher level controller setup
% In this section we define the higher layer controllers. 
% We propose a model predictive control (MPC) design strategy for such a
% layer. The function lincon from the hybrid toolbox is used to perform 
% that task.
warning off


%% Centralized HiMPC for comparison
cdec(1).x=1:8;
cdec(1).y=1:4;
cdec(1).u=1:4;
cdec(1).applied=1:4;
ccoupledCons(1).H=[-1 0 1 0 0 0 0 0];
ccoupledCons(1).K=[.3];
cDeltaK{1}=[.3;2;.3;2;.3;2;.3;2;.3;2;.3;2;.3;2;.3;2;0.12];
csys=ss(A,B,C,D,TL);
cdcg=dcgain(csys);
csys.B=csys.B*inv(cdcg);
if ~precomputed_data
    cobj = HiMPC(csys,cdec,Xcon,cDeltaK,ccoupledCons);
    cobj=cobj.computeMOARS();
    cobj=cobj.computeDeltaR();
    save Data/cobj_data cobj;
else
    load Data/c_mac_pro_f;
end
[sup ind] = max(cobj.DrN{1});
cdrOpt=cobj.Dr{1}(ind);
cTH = ind*TL;
cc=[csys.c;zeros(17,8)];
dd=[csys.d;cobj.Hr{1}*cdcg];
cmodel=ss(A,B,cc,dd,TL);

nDec=length(dec);
dcg={};
for i=1:nDec
    %%
    % High level sample time is chosen guaranteeing the maximal allowable
    % slope for the references, as the maximum of the ratio $\Delta r(N)/N$
    [sup ind] = max(dobj.DrN{i});
    drOpt=dobj.Dr{i}(ind);
    %%
    % compute the Higher level sample-time
    TH(i) = ind*TL;
    %%
    % Sub-models definition
    Ai=A(dec(i).x,dec(i).x);
    Bi=B(dec(i).x,dec(i).u);
    Ci=C(dec(i).y,dec(i).x);
    Di=D(dec(i).y,dec(i).u);
    
    %[Ai Bi Ci Di]=ssdata(d2d(ss(Ai,Bi,Ci,Di,TL),TH(i)));
    %Ai=round(Ai*1e3)/1e3;
    %Bi=round(Bi*1e3)/1e3;
    
    nx=length(dec(i).x);
    nu=length(dec(i).u);
    ny=length(dec(i).y);
    %%
    % State and reference polytopes (MOAS) to be imposed in order to
    % guarantee stability
    Ko=dobj.Ko;
    Ho=dobj.Ho;
    Hx=dobj.Hx;
    K=dobj.K;
    Hr=dobj.Hr;
    Kr=dobj.Kr;
    nco(i)=length(Ko{i});
    ncu(i)=length(Kr{i});
    ncx(i)=length(K{i});
    
    % Ci=[Ci;Ho{i};zeros(ncu(i),nx)];
    % Di=[Di;-Ho{i}*Ci(1:ny,:)'*0;Hy{i}];
    Ci=[Ci;zeros(ncu(i),nx)];
    Di=[Di;Hr{i}];
    
    %%
    % Controller tuning
    sysOld=ss(Ai,Bi,Ci,Di,TL);
    model{i}=d2d(sysOld,TH(i));
    dcg{i}=dcgain(model{i});
    %keyboard
    model{i}.d=[D(dec(i).y,dec(i).u);Hr{i}*dcg{i}(nu,nu)];
    type='track';
    %%
    % MPC setup, 'help lincon' for more details
    cost.S=blkdiag(1e2*eye(ny),1e-3*eye(ncu(i)));
    cost.T=1e0*eye(nu);
    cost.rho=1e6;
    interval.N=10;
    interval.Nu=5;
    % Compute output constraints (only mass positions)
    toOut=[1 0 0 0;0 0 0 0;0 1 0 0;0 0 0 0;0 0 1 0;0 0 0 0;0 0 0 1;0 0 0 0]';
    Ycon.min=toOut*Xcon.min;
    Ycon.max=toOut*Xcon.max;
    limits.ymin=[Ycon.min(dec(i).y);-inf*ones(ncu(i),1)];
    limits.ymax=[Ycon.max(dec(i).y);Kr{i}];
    limits.dumin=-drOpt*ones(nu,1);
    limits.dumax=drOpt*ones(nu,1);
    
    
    %%
    % Controller object creation
    L{i}=lincon(model{i},type,cost,interval,limits,'cplex',1);
end

% Centralized Controller
cmodel=d2d(cmodel,cTH);
type='track';
cost.S=blkdiag(1e2*eye(4),1e-3*eye(17));
cost.T=1e0*eye(4);
cost.rho=1e6;
interval.N=10;
interval.Nu=5;
limits.ymin=[Ycon.min;-inf*ones(17,1)];
limits.ymax=[Ycon.max;cobj.Kr{1}];
limits.dumin=-cdrOpt*ones(4,1);
limits.dumax=cdrOpt*ones(4,1);
cL=lincon(cmodel,type,cost,interval,limits,'cplex',1);



%% Simulation
% Simulation time
Tsim = 3*60;

t=0:TL:Tsim-TL;
%%
% Number of iteration of the discrete time loop
TT=Tsim/TL;
%%
% Define references
r=[ .65*ones(1,TT/2-15) .2*ones(1,TT/2+15);
    .8*ones(1,TT/4+10) .7*ones(1,TT*2/4+10) .4*ones(1,TT*1/4-20);
    0.0*ones(1,TT/2-120) 0.7*ones(1,TT/2+120);
    0.0*ones(1,TT)];
%%
% Initial condition 
x0=[0 0 0 0 0 0 0 0]';
%%
% Initial output
u0=zeros(4,1);
x=[x0 x0];
cx=x;
u=u0;
cu=u;
%%
% Simulation loop
for k=2:TT-1
    %%
    % For each controller
    for i=1:nDec
        %%
        % Determine which states are used by the current controller
        xx=x(dec(i).x,k);
        %yy=[xx;himpc_obj.Ho{i}*xx;himpc_obj.Hy{i}*u(dec(i).u,k-1)];
        %%
        % Determine the references that applies to the current sub-system
        rr=[r(dec(i).y,k);zeros(ncu(i),1)];
        %%
        % If it is not time to change the reference (input given by the
        % current controller) then use the previous one
        u(dec(i).u,k) = u(dec(i).u,k-1);
        %%
        % Check if it is time for the higher level controller to execute
        if mod(k,TH(i)/TL)==2
            %%
            % The controller type is 'track' the MPC will return the input
            % increment
                
            u(dec(i).u,k)=u(dec(i).u,k-1)+eval(L{i},xx,rr,u(dec(i).u,k-1));
        end
    end
    %%
    % Update the state   
    x(:,k+1)=A*x(:,k)+B*u(:,k);
    %%
    % Assume the next input to be the same as current one. That is done for
    % plot reasons which request x and u to have the same length
    u(:,k+1)=u(:,k);
    
    
    %% Centralized
    if mod(k,cTH/TL)==2
        cu(:,k)=cu(:,k-1)+eval(cL,cx(:,k),[r(:,k);zeros(17,1)],cu(:,k-1));
    end
    cx(:,k+1)=A*cx(:,k)+B*cu(:,k);
    cu(:,k+1)=cu(:,k);
    
end

if showPlot
    %%
    % Plot the first (continuous blue) and second (dashed blue) mass positions
    % against their references (continuous red and dash red), respectively
    figure(4);
    plot(t,x(1,:)','b',t,x(3,:)','--b',t,r(1,:)','r',t,r(2,:)','--r',t,cx(1,:)','k',t,cx(3,:)','--k');
    title('First and second mass positions')
    %%
    % Plot first and second inputs against their references
    figure(5);
    plot(t,u(1,:)','b',t,u(2,:)','--b',t,r(1,:)','r',t,r(2,:)','--r')
    hold on
    plot(t,cu(1,:)','k',t,cu(2,:)','--k')
    title('First and second inputs')
    
    %%
    % Plot showing all the mass trajectories against their references
    figure(6);
    plot(t,x(1:2:7,:)','b',t,cx(1:2:7,:)','k',t,r,'r')
    title('All mass positions')
    %%
    % Plot showing all inputs (applied references) against their references
    figure(7);
    plot(t,u','b',t,cu','k',t,r,'r')
    title('All inputs')
end

%% Computation of performance indices
% Compute the MPC cost over the whole simulation horizon
index_d=0;
index_c=0;
for i=TT
    index_c=index_c+cx(:,i)'*C'*cost.S(1:4,1:4)*C*cx(:,i)+cu(:,i)'*cost.T*cu(:,i);
    index_d=index_d+x(:,i)'*C'*cost.S(1:4,1:4)*C*x(:,i)+u(:,i)'*cost.T*u(:,i);    
end
disp(['Comulated const with centralized feedback is ' num2str(index_c) ...
    ' and with decentralized feedback is ' num2str(index_d)]);
disp(['Performance improvement is ' num2str(100*(index_c-index_d)/index_d) '%'])


%% Considerations
%
% Figures 4 and 6 show that the hard constraint that bounds the mass upper
% position holds for both masses. Moreover the tracking is effective with
% limited overshoot. Figure 4 also show that the system is actually coupled
% since both masses 3 and 4, whose reference is stationary after time
% instant 20, have some fluctuation due to the interaction with neighbors.
%
% Figures 5 and 7 show the references actually applied to the lower level
% controller. It is interesting to see how limits imposed have the effect
% of limiting the overshoot on the mass position. Moreover, it should be
% observed that the reference on the second mass (dashed blue line) does
% not reach the user defined reference (red continuous line) which is not 
% in the admissible set for references (we recall that the admissible
% reference set is the output set tighter by the tuning know factor 
% $\Delta y$ for allowing the determination of a MOAS).
