%% Example file for class HiMPC
%
% In this demo we use the HiMPC toolbox to show how to build a decentralized
% hierarchical controller for linear systems subject to linear constraints 
% on input and output variables.
%
%% Description
%
% For this demo we use the following system consting of four balls and 
% seven springs.
%
% <html>
% <IMG SRC="plant.jpg", height=400>
% </html>
%
% <latex>
% \includegraphics[width=.7\hsize]{plant.jpg}
% </latex>
%
% The process is composed by four masses moving vertically, each one 
% connected by a spring to a fixed ceiling, subject to damping due to 
% viscous friction with the environment, and connected to its neighbor mass
% by another spring.
% An input force u [Nm] can be applied to each mass by the
% lower-level controller. The state of the system is the vector
% $x$ collecting the vertical positions and accelerations of the masses,
% while the output $y$ is the vector that collects the positions.
% The constrained masses-springs system is an extension of the example
% presented [1]. However in this demo both layers have a decentralized
% structure, in particular four controllers at the higher level 
% independently process the user provided references and command
% their respective actuators.
%
% The final objective of this class functions is to compute the maximum 
% reference variation, element-wise, that the higher controller can apply
% in order to guarantee the system stability while 
% fulfilling the constraints.
% Such goal is achieved by ensuring that the system state vector starting 
% in a Maximal Output Admissible Set (MOAS) will be in a new MOAS at the 
% next execution of the higher level
% controller. The MOAS is obtained by tightening the output
% constraints by a fixed value, $\Delta y$ that is a tuning knob of the
% approach. 
% The computation of maximum reference variation is an optimization
% problem, which can be conveniently casted as a Mixed Integer Linear Program 
% (MILP), and solved via suitable solvers (we strongly suggest IBM, former 
% ILOG, Cplex).
%
% The simulations are performed using at the higher layer a set of
% independent hierarchical MPC (HiMPC) which adopt a linear formulation.
% The upper-layer controllers must embed constraints
% on the generated references, to ensure stability
% and constraint satisfaction. 
% In this example we impose constraints on plant outputs, namely the 
% positions of the balls depicted in figure. 
%
% [1] D. Barcelli, A. Bemporad and G. Ripaccioli, _Hierarchical MPC
% controller design for LTI systems_, 49th Control and Decision Conference,
% Atlanta, Georgia, USA, 2010. 

%% Simulation Example
%
close all
clear variables
clear globals
clc


%% Plant model
%
% Sample time 
TL=.25;
%%
% Creates the closed-loop model of the plant with the lower layer controller 
[sys N]=buildModel(TL);
[A B C D] = ssdata(sys);
%%
% Number of states and inputs respectively
[Nx Ny]=size(B);
%%        
% $\Delta y$ :  Tightening of the output admissible set. Tuning knob.
DeltaY=.3;
%%
% Output constraints
Ycon.min = -0.3*ones(N,1);
Ycon.max = 1*ones(N,1);
%%
% State constraints
% The output constraints have to be expressed in state coordinates in order
% to computer the MOAS. 
% To avoid numerical problems the state constraints polytope is bounded, 
% hence the speed is limited to 10 [m/s] 
Xcon.min=[];
for i=1:N,Xcon.min=[Xcon.min;-DeltaY;-10];end
Xcon.max=[];
for i=1:N,Xcon.max=[Xcon.max;DeltaY;10];end
%%
coupledCons(1).H=[];
coupledCons(1).K=[];
coupledCons(2).H=[];
coupledCons(2).K=[];
coupledCons(3).H=[];
coupledCons(3).K=[];
coupledCons(4).H=[];
coupledCons(4).K=[];
%%
% The decentralization of the model is defined as:
dec(1).x=[1 2];
dec(1).y=1;
dec(1).u=1;
dec(1).applied=1;
dec(2).x=[3 4];
dec(2).y=2;
dec(2).u=2;
dec(2).applied=2;
dec(3).x=[5 6];
dec(3).y=3;
dec(3).u=3;
dec(3).applied=3;
dec(4).x=[7 8];
dec(4).y=4;
dec(4).u=4;
dec(4).applied=4;
%%
% Create the instance of the object
himpc_obj = HiMPC(sys,dec,Xcon,DeltaY,coupledCons);
%%
% Compute the MOAS. For this purpose each sub-model is considered
% independent and subject to an unknown but bounded disturbance. The 
% disturbance models the influence of the unmodeled dynamics, i.e. the 
% state components that are not present in that sub-system. The uncertainty
% polytope is computed in an exact manner by considering the range of each
% unmodeled state component and its influence with respect to state 
% components which belong to the current sub-model.
himpc_obj=himpc_obj.computeMOAS();
%%
% Plot both the MOAS with (blue) and without (red) the influence of the 
% other sub-models for each sub-model
himpc_obj.plotMOAS();
%%
% Solve the MILP for values of sample-time ratio from 1 to 50 and, 
% accordingly to the ratio, compute the maximal allowable $\Delta r$
himpc_obj=himpc_obj.computeDeltaR(50);
%%
% Plot both $\Delta r(N)$ as a function of N, that is the higher-lower 
% sample times ratio, and $\Delta r(N)/N$ for each sub-model
himpc_obj.plotDeltaR();

%% Pre-simulation considerations
%
% The first and the forth sub-models MOAS are completely equivalent as well as
% the corresponding $\Delta r(N)$ and $\Delta r(N)/N$. The same consideration 
% can be extended to the second and third sub-models. This is motivated by the
% internal symmetry of both the plant and the decentralization.
%

%% Higher level controller setup
% In this section we define the higher layer controllers. 
% We propose a model predictive control (MPC) design strategy for such a
% layer. The function lincon from the hybrid toolbox is used to perform 
% that task.
warning off

nDec=length(dec);
for i=1:nDec
    %%
    % High level sample time is chosen guaranteeing the maximal allowable
    % slope for the references, as the maximum of the ratio $\Delta r(N)/N$
    [sup ind] = max(himpc_obj.DrN{i});
    drOpt=himpc_obj.Dr{i}(ind);
    %%
    % compute the Higher level sample-time
    TH(i) = ind*TL;
    %%
    % Sub-models definition
    Ai=A(dec(i).x,dec(i).x);
    Bi=B(dec(i).x,dec(i).u);
    Ci=C(dec(i).y,dec(i).x);
    Di=D(dec(i).y,dec(i).u);

    [Ai Bi Ci Di]=ssdata(d2d(ss(Ai,Bi,Ci,Di,TL),TH(i)));

    nx=length(dec(i).x);
    nu=length(dec(i).u);
    ny=length(dec(i).y);
    %%
    % State and reference polytopes (MOAS) to be imposed in order to
    % guarantee stability
    Ko=himpc_obj.Ko;
    Ho=himpc_obj.Ho;
    Ky=himpc_obj.Ky;
    Hy=himpc_obj.Hy;
    ncx(i)=length(Ko{i});
    ncu(i)=length(Ky{i});

    Ci=[Ci;Ho{i};zeros(ncu(i),nx)];
    Di=[Di;-Ho{i}*Ci(1:ny,:)';Hy{i}];
    %%
    % Controller tuning
    sysOld=ss(Ai,Bi,Ci,Di,TL);
    sysNew=d2d(sysOld,TH(i));
    model=sysNew;
    type='track';
    %%
    % MPC setup, 'help lincon' for more details
    cost.Q = .01*eye(nx);
    cost.R = .01*eye(nu);
    cost.S=blkdiag(1*eye(ny),.001*eye(ncx(i)+ncu(i)));
    cost.T=.01*eye(nu);
    cost.rho=1e6;
    interval.N=2;
    interval.Nu=1;
    limits.ymin=[Ycon.min(dec(i).y);-inf*ones(ncx(i)+ncu(i),1)];
    %%
    % Tighted output constraint computation
    KK=Ky{i}-DeltaY;
    limits.ymax=[Ycon.max(dec(i).y); Ko{i} ; KK];
    limits.dumin=-drOpt*ones(nu,1);
    limits.dumax=drOpt*ones(nu,1);

    %%
    % Controller object creation
    L{i}=lincon(model,type,cost,interval,limits,'cplex',1);
end


%% Simulation
% Simulation time
Tsim = 60;

t=0:TL:Tsim-TL;
%%
% Number of iteration of the discrete time loop
TT=Tsim/TL;
%%
% Define references
r=[ .65*ones(1,TT/2-15) .2*ones(1,TT/2+15);
    .8*ones(1,TT/4+10) .7*ones(1,TT*2/4-10) .4*ones(1,TT*1/4);
    0.0*ones(1,TT/2) 0.7*ones(1,TT/2);
    0.0*ones(1,TT)];
%%
% Initial condition 
x0=[0 0 0 0 0 0 0 0]';
%%
% Initial output
u0=zeros(4,1);
x=[x0 x0];
u=u0;
%%
% Simulation loop
for k=2:TT-1
    %%
    % For each controller
    for i=1:nDec
        %%
        % Determine which states are used by the current controller
        xx=x(dec(i).x,k);
        %yy=[xx;himpc_obj.Ho{i}*xx;himpc_obj.Hy{i}*u(dec(i).u,k-1)];
        %%
        % Determine the references that applies to the current sub-system
        rr=[r(dec(i).y,k);zeros(ncx(i)+ncu(i),1)];
        %%
        % If it is not time to change the reference (input given by the
        % current controller) then use the previous one
        u(dec(i).u,k) = u(dec(i).u,k-1);
        %%
        % Check if it is time for the higher level controller to execute
        if mod(k,TH(i)/TL)==2
            %%
            % The controller type is 'track' the MPC will return the input
            % increment
            u(dec(i).u,k) = u(dec(i).u,k)+eval(L{i},xx,rr,u(dec(i).u,k-1));
        end
    end
    %%
    % Update the state
    x(:,k+1)=A*x(:,k)+B*u(:,k);
    %%
    % Assume the next input to be the same as current one. That is done for
    % plot reasons which request x and u to have the same length
    u(:,k+1)=u(:,k);
end

%%
% Plot the first (continuous blue) and second (dashed blue) mass positions 
% against their references (continuous red and dash red), respectively
figure(4);
plot(t,x(1,:)','b',t,x(3,:)','--b',t,r(1,:)','r',t,r(2,:)','--r');
title('First and second mass positions')
%%
% Plot first and second inputs against their references
figure(5);
plot(t,u(1,:)','b',t,u(2,:)','--b',t,r(1,:)','r',t,r(2,:)','--r')
title('First and second inputs')

%%
% Plot showing all the mass trajectories against their references
figure(6);
plot(t,x(1:2:7,:)','b',t,r,'r')
title('All mass positions')
%%
% Plot showing all inputs (applied references) against their references
figure(7);
plot(t,u','b',t,r,'r')
title('All inputs')

%% Considerations
%
% Figures 4 and 6 show that the hard constraint that bounds the mass upper
% position holds for both masses. Moreover the tracking is effective with
% limited overshoot. Figure 4 also show that the system is actually coupled
% since both masses 3 and 4, whose reference is stationary after time
% instant 20, have some fluctuation due to the interaction with neighbors.
%
% Figures 5 and 7 show the references actually applied to the lower level
% controller. It is interesting to see how limits imposed have the effect
% of limiting the overshoot on the mass position. Moreover, it should be
% observed that the reference on the second mass (dashed blue line) does
% not reach the user defined reference (red continuous line) which is not 
% in the admissible set for references (we recall that the admissible
% reference set is the output set tighter by the tuning know factor 
% $\Delta y$ for allowing the determination of a MOAS).
