%% LSmodel - Large Scale Model Class
% |LSmodel| is a class for representation and management of Large Scale 
% (LS) models. LS model is described as a set of submodels together with
% description of submodels mutual interconnections and submodels
% interconnections with external inputs and outputs.
%
% Signal interconnections is name based, i.e. to define that outputs and 
% intpus of certain submodels are conneted requires to give them the same 
% names. Submodels are internally represented as state-space models. 
% However any model type, which can be converted by SS command, can by 
% supplied as submodel and original model is stored in LSmodel class along 
% with its state-space conversion.
%
% LS model is created from a set of submodels with named inputs and outputs
% (at least interconnection signals should be named), a set of summator
% created for example by SUMBLK commands and string cell arrays defining
% external inputs and outputs (see |LSmodel_example.m|). The model can be 
% later modyfied by adding new submodels, removing submodels, 
% adding/removing external inputs/outputs.
% 
% LS model supports specification of different signal types for each
% signal
%
%  -------------------------------------------------------
%  | Signal Type               | Abbreviation |  Channel |
%  -------------------------------------------------------
%  | Manipulated               |     MV       |  Input   |
%  | Measured Disturbances     |     MD       |  Input   |
%  | Unmeasured Disturbances   |     UD       |  Input   |
%  | Measured Outputs          |     MO       |  Output  |
%  | Unmeasured Outputs        |     UO       |  Output  |
%  | Internal Signal           |     X        |    --    |
%  -------------------------------------------------------
%
% and specification of signal limits (min/max values, min/max slopes,...).
% This information is later used for estimator and/or controller design.
%
% |LSmodel| has methods for structured model order reduction, decomposition
% of subsystems into groups for distributed control/estimation and merging 
% of information from multiple models into single one.
%
% Model structure can be ploted by overloaded plot command and model
% analysis can be done by many standard overloaded functions (see bellow).
%
% State-space model of full |LSmodel| can be obtained by SS method.
% Parts of model can be obtained either for selected inputs and outputs
% or by extracting group of subsystems, where grouping information may be 
% a result of built-in decomposition method.
%
% *Methods*
%
%   LSmodel      - constructor, compatible with Matlab connect function
%
%   add_mod      - adds submodel to LS model
%   add_sum      - adds summator (creates new internal signal)
%   add_ext_inp  - adds external input to LS model
%   add_ext_out  - adds external output from LS model
%   rem_mod      - removes submodel from LS model
%   rem_sum      - removes summator
%   rem_ext_inp  - removes external input
%   rem_ext_out  - removes external output
%
%   set_sig_type - signal types according to the MPC toolbox
%   set_sig_lim  - set signal limits (min/max, delta min/max)
%   set_sig_data - assigns (user) data structure to selcted signal(s)
%
%   select       - extracts part of the model for selected inputs and 
%                  outputs
%   group        - extracts subsystems belonging to given group
%   squeeze      - remove unused external inputs/outputs and unconnected
%                  submodels
%
%   struct_red   - structured model order reduction
%   merge        - merging of ARX models of different structure
%   freq_uncert  - return frequency uncertainty for ARX model
%   eps          - epsilon decomposition
%   bbd          - Border Block Diagonal decomposition
%
%   display      - presents the model content and shows the numbering of
%                  subsystems, summators, inputs and outputs
%   n            - returns total order (sum of subsystems order)
%   orders       - returns subsystems orders
%   plot         - plots interaction between subsystems
%
% *Overloaded functions*
%
%   dcgain, pole, zero, impulse, step, bode, nyquist, pzmap, iopzmap, 
%   ss, ...
%
% *Internal model structure*
%
% Internal model structure can be seen in the following figure. The signal
% names are:
%
%   w ... external inputs to LS model
%   z ... external outputs from LS model
%   u ... aggregated inputs to submodels
%   y ... aggregated outputs from submodels
% 
% Submodels are stored in cell array |M| as state-space models. 
% The original models supplied by the user are stored in cell array
% |Morig|. External model inputs are stored in string cell array |Wnames|
% and outputs in |Znames|. Static interconnections matrices 
% Lzw,Lzy,Luw,Luy represent submodels mutual interconnections and submodel
% interconnections with external inputs and outputs. They are automatically 
% computed after each model modification.
%
% <<./figs/lsmodel>>
% 


%%
classdef LSmodel %< handle
  
  % === Read only properties ==============================================  
  properties (GetAccess=public, SetAccess=protected)
    M                       % cell array of submodels (converted to ss)
    Morig                   % original models supplied to LSmodel
    N                       % number of subsystems
    Ns                      % number of summators
    sum                     % cell array of summators
    nw                      % number of external inputs
    ny                      % total number of diagonalized models outputs
    nz                      % number of external outputs
    nu                      % total number of diagonalized models inputs    
    Lzw,Lzy,Luw,Luy         % static feedback matrices    
    signal                  % struct array of signal properties (name, type, limits, price, (user) data)        
    Wnames,Znames           % external I/O names    
  end %set protected properties  
  
  
  % === Public properties =================================================
  properties (Access = public)
    grouping                % group index for each subsystem    
  end %public properties
  
  
  % === Protected properties ==============================================
  properties (Access = protected)
    modified                % object structure modification flag
    xy                      % vertex positions for plot (automatically computed)
  end %protected properties  
  
  
  methods

    % =====================================================================
    function obj = LSmodel( varargin )
      %% LSmodel Constructor
      % |LSMODEL( X1,...,Xn, INPUT_NAMES, OUTPUT_NAMES )| constructs large
      % scale model by specifying a set of submodels, summators and a list
      % of external input and output names. Internal connections are
      % automatically created by matching signal names. Xi are models or
      % summators (single or in a cell array). INPUT_NAMES and OUTPUT_NAMES
      % are cell arrays of strings with external input and output names.
      % Summators can be created by command SUMBLK. Examples:
      %
      % |mod = LSmodel(M,sum,{'fuel','demand'},{'pressure','flow'});|
      %
      % |mod = LSmodel(M1,M2,M3,M4,sum1,sum2,{'fuel','demand'},{'pressure','flow'});|
      %
      % M is cell array of models (ss,tf,...) with named inputs and 
      % outputs. SUM is cell array of summators created by SUMBLK.
      %
      % *Constructor is compatible with standard CONNECT function.*
      % It is also possible to construct the model by using CONNECT 
      % compatible numeric indexing of inputs and outputs (see help on 
      % CONNECT). However, name based signal referencing will be prefered 
      % as numeric indexing may be confusing for large scale models.       
      
      % empty object 
      obj.M     = {};
      obj.Morig = {};
      obj.N     = 0;
      obj.Ns    = 0;
      obj.Lzw = [];
      obj.Lzy = [];
      obj.Luw = [];
      obj.Luy = [];
      obj.nw  = 0;  % number of external inputs
      obj.ny  = 0;  % total number of diagonalized models outputs
      obj.nz  = 0;  % number of external outputs
      obj.nu  = 0;  % total number of diagonalized models inputs
      obj.grouping = [];
      obj.Wnames = {};
      obj.Znames = {};
      obj.signal.name  = [];
      obj.signal.tag   = [];
      obj.signal.type  = []; % Note that signal type is valid only for unconnected signals.      
      obj.signal.limit = [];
      obj.signal.price = [];
      obj.signal.data =  [];
      obj.modified    =  1;
      obj.xy          =  [];
      
      if nargin>0,
        % This should be compatible with connect/append standard
        % functions. However, this function preserves structure
        % information in comparison to connect, where this
        % information is lost.
        %
        % Additionaly it should accept the first parameter as
        % a cell of models.


        % -----------------------------------------------------------------
        % --- Connecting by signal names ----------------------------------
        % -----------------------------------------------------------------        
        if (iscellstr(varargin{nargin-1}) || ischar(varargin{nargin-1}) ) && ...
           (iscellstr(varargin{nargin}) || ischar(varargin{nargin})),
          % --- External Inputs w --------------------
          obj.Wnames  = varargin{nargin-1};
          if ischar(obj.Wnames), obj.Wnames = {obj.Wnames}; end
          if size(obj.Wnames,2)>1, obj.Wnames = obj.Wnames'; end
          obj.nw = length(obj.Wnames);
          
          % --- External Outputs z -------------------
          obj.Znames = varargin{nargin};
          if ischar(obj.Znames), obj.Znames = {obj.Znames}; end
          if size(obj.Znames,2)>1, obj.Znames = obj.Znames'; end
          obj.nz = length(obj.Znames);
          obj.Znames = obj.Znames;
          
          % --- Separating Models and Summators ------
          mod_cnt = 1;
          sum_cnt = 1;
          for i=1:nargin-2,
            par = varargin{i};
            if ~iscell(par), par = {par};end
            for j=1:length(par),
              mod = ss( par{j} );
              if size(mod.A>0),       % dynamic model
                if isempty(mod.Name),
                  mod.Name = ['M' num2str(mod_cnt)];
                end
                obj.M{mod_cnt}     = mod;
                obj.Morig{mod_cnt} = par{j};
                obj = obj.add_default_io(mod); % add inputs/outputs to signal list
                mod_cnt = mod_cnt + 1;
              else                    % summator
                if isempty(mod.Name),
                  mod.Name = ['S' num2str(sum_cnt)];
                end                
                obj.sum{sum_cnt} = mod;
                sum_cnt = sum_cnt + 1;
              end
            end
          end          
          obj.N  = mod_cnt - 1;
          obj.Ns = sum_cnt - 1;
                        
          % --- Find L matrix from signal names ------
          obj = update_model(obj);

          % --- Grouping -----------------------------
          obj.grouping = zeros(1,length(obj.M));

          
        else
        % -----------------------------------------------------------------          
        % --- Connecting by indexes ---------------------------------------
        % -----------------------------------------------------------------               
          
          % --- Get SS Models  -----------------------
          obj.nu = 0;
          obj.ny = 0;
          mod_cnt = 1;
          for i=1:nargin-3,
            par = varargin{i};
            if ~iscell(par), par = {par};end
            for j=1:length(par),
              try
                mod = ss( par{j} );
              catch exception
                error('Conversion to ss model error.');
              end
              obj.M{mod_cnt}     = mod;
              obj.Morig{mod_cnt} = par{j};
              mod_cnt = mod_cnt + 1;
              % -----
              obj.nu = obj.nu + size(mod.B,2);
              obj.ny = obj.ny + size(mod.C,1);              
            end
          end          
          obj.N  = mod_cnt - 1;
          obj.Ns = 0;
          
          % --- Interconnection Matrix ---------------
          Q = varargin{nargin-2};
          
          % --- External I/O indexes -----------------
          input_indexes  = varargin{nargin-1};
          output_indexes = varargin{nargin};
          
          obj.nw = length(input_indexes);
          obj.nz = length(output_indexes);
          
          % --- Setting up interconnection matrixes --
          obj.Lzw = zeros(obj.nz,obj.nw); % always zero in this parameterization        
          obj.Lzy = zeros(obj.nz,obj.ny);
          obj.Luw = zeros(obj.nu,obj.nw);
          obj.Luy = zeros(obj.nu,obj.ny);
                              
          % --- External Inputs ----------------------
          if iscell(input_indexes), % input enters multiple subsystems inputs
            for i=1:obj.nw,
              obj.Luw( input_indexes{i} , i ) = 1;              
            end
          else % single input indexing
            for i=1:obj.nw,
              obj.Luw( input_indexes(i) , i ) = 1;
            end
          end
          
          % --- External Outputs ---------------------
          if iscell(output_indexes), % output as a sum of multiple subsystem outputs
            for i=1:obj.nz,
              obj.Lzy( i , output_indexes{i} ) = 1;              
            end
          else % simple indexing
            for i=1:obj.nz,
              obj.Lzy( i , output_indexes(i) ) = 1;
            end
          end
          
          % --- Subsystems interconnection -----------
          for i=1:size(Q,1),
            for j=2:size(Q,2),
              if Q(i,j)>0,
                obj.Luy( Q(i,1) , abs(Q(i,j)) ) = sign(Q(i,j));
              end
            end
          end
          
          % --- Grouping -----------------------------
          obj.grouping = zeros(1,length(obj.M));         
          
        end
      end
    end %LSModel

    
    
    % =====================================================================
    function obj = rem_mod(obj,par)
      %% Remove Model 
      % |REM_MOD(OBJ,PAR)| removes model from large scale system. PAR is a
      % vector of indexes or string cell array of model names to be
      % deleted. (model indexes are shown by display function)

      % --- Find model index from name ---------------
      M_IND = [];
      if ischar(par) || iscellstr(par),
        if ~iscell(par), par = {par};end
        for j=1:length(par),
          m_ind = 0;
          for i=1:obj.N,
            if strcmp(obj.M{i}.Name,par{j}),
              m_ind = i;
              break;
            end
          end
          if ~m_ind,
            error(['Model named ' par{j} ' was not found.']);
          end
          M_IND(j)=m_ind;
        end
      else
        M_IND = par;
      end
      
      % ----------------------------------------------
      M_IND = sort(M_IND,'descend');

      % ----------------------------------------------
      for i = 1:length(M_IND),
        m_ind = M_IND(i);
        
        % --- Check range ------------------------------
        if m_ind<1 || m_ind>length(obj.M),
          error('Model index is out of range.');
        end
        
        % --- Remove model -----------------------------
        obj.M        = [ obj.M(1:m_ind-1)        , obj.M(m_ind+1:end) ];
        obj.Morig    = [ obj.Morig(1:m_ind-1)    , obj.Morig(m_ind+1:end) ];
        obj.grouping = [ obj.grouping(1:m_ind-1) , obj.grouping(m_ind+1:end) ];
        obj.N = obj.N - 1;        
      end
      
      % --- Recompute interconnection matrix L -------
      obj = update_model(obj);
      obj.modified = 1;      
      
    end %remove_model
  

    
    % =====================================================================
    function obj = rem_sum(obj,par)
      %% Remove Summator
      % |REM_SUM(OBJ,PAR)| removes summator from large scale system.
      % PAR is the index or the name of the summator to be deleted. 
      % Summator indexes are shown by display function.

      % --- Find model index from name ---------------
      if ischar(par),
        s_ind = 0;
        for i=1:obj.Ns,
          if strcmp(obj.sum{i}.Name,par),
            s_ind = i;
            break;
          end
        end
        if ~s_ind,
          error(['Summator named ' par ' was not found.']);
        end
      else
        s_ind = par;
      end
      
      % --- Check range ------------------------------
      if s_ind<1 || s_ind>length(obj.Ns),
        error('Summator index is out of range.');
      end
           
      % --- Remove summator --------------------------
      obj.sum   = [ obj.sum(1:s_ind-1) , obj.sum(s_ind+1:end) ];
      obj.Ns    = obj.Ns - 1;                
        
      % --- Recompute interconnection matrix L -------
      obj = update_model(obj);
      obj.modified = 1;
      
    end %remove_summator      
      
    
        
    % =====================================================================
    function obj = add_mod(obj,newM)
      %% Add Model 
      % |ADD_MOD(OBJ,NEWM)| adds new model NEWM to large scale model class 
      % and connects it according to the input / output names. NEWM can be 
      % a cell array of models.
            
      mod_cnt = obj.N+1;
      if ~iscell(newM), newM = {newM};end
      for j=1:length(newM),
        try
          mod = ss( newM{j} );
        catch exception
          error('Conversion to ss model error.');
        end
        
        if isempty(mod.Name),
          mod.Name = ['M' num2str(mod_cnt)];
        end
        
        obj.M{mod_cnt}        = mod;
        obj.Morig{mod_cnt}    = newM{j};
        obj = obj.add_default_io(mod); % add inputs/outputs to signal list
        obj.grouping(mod_cnt) = 0;
        mod_cnt = mod_cnt + 1;
        
        if isempty(mod.A),
          warning('LSmodel:mod_err','Model is static?');
        end        
      end
      
      obj.N = mod_cnt - 1;
      
      % --- Update L matrix from signal names --------
      obj = update_model(obj);
      obj.modified = 1;      
            
    end %add_model
    
    
    
    % =====================================================================
    function obj = add_sum(obj,newS)
      %% Add Summator
      % |ADD_SUM(OBJ,NEWS)| adds new summator to the model and connects
      % it according the input/output names. Numeric based interconnection 
      % is not possible. NEWS can be an array of summators.
      % 
      % Summator can be created by SUMBLK function.
            
      % --- Separating Models and Summators ------
      sum_cnt = obj.Ns+1;
      if ~iscell(newS), newS = {newS};end
      for j=1:length(newS),
        try
          mod = ss( newS{j} );
        catch exception
          error('Conversion to ss model error.');
        end
        if ~isempty(mod.A),
          error('Summator must be static system.');          
        end
        if size(mod.C,1)~=1,
           error('Summator can have only one output.');
        end
                
        if isempty(mod.Name),
          mod.Name = ['S' num2str(sum_cnt)];
        end          
        obj.sum{sum_cnt} = mod;
        sum_cnt = sum_cnt + 1;
      end     

      obj.Ns = sum_cnt - 1;
           
      % --- Update L matrix from signal names --------
      obj = update_model(obj);
      obj.modified = 1;      
      
    end %add_summator

    
    
    % =====================================================================
    function obj = rem_ext_inp(obj,par)     
      %% Remove External Input
      % |REM_EXT_INP(OBJ,PAR)| Removes external input. PAR can be input 
      % index or its NAME.
      
      % --- Find input index ------------------------
      if ischar(par)
        ind = find( strcmp(obj.Wnames,par), 1);
      else
        ind = par;
      end
      
      % --- Check range ------------------------------
      if ind<1 || ind>length(obj.Wnames),
        error('External input index is out of range.');
      end
      
      % --- Remove output ----------------------------
      if ~isempty(ind),
        obj.Wnames = [ obj.Wnames(1:ind-1) , obj.Wnames(ind+1:end) ];
        % --- Find L matrix from signal names ----
        obj = update_model(obj);
      else
        error(['External input named ' name ' was not found.']);
      end      
    end %remove_input
    

    
    % =====================================================================
    function obj = add_ext_inp(obj,name)      
      %% Add External Input
      % |ADD_EXT_INP(OBJ,NAME)| adds external input named NAME.
      
      obj.Wnames{end+1,1} = name;
      
      % --- Find L matrix from signal names ----
      obj = update_model(obj);
    end %add_input     
      
        

    % =====================================================================
    function obj = rem_ext_out(obj,par)
      %% Remove External Output 
      % |REM_EXT_OUT(OBJ,PAR)| Removes external output. PAR can be output 
      % index or its NAME.
      
      % --- Find output index ------------------------
      if ischar(par)
        ind = find( strcmp(obj.Znames,par), 1);
      else
        ind = par;
      end
      
      % --- Check range ------------------------------
      if ind<1 || ind>length(obj.Znames),
        error('External output index is out of range.');
      end
      
      % --- Remove Output ----------------------------
      if ~isempty(ind),
        obj.Znames = [ obj.Znames(1:ind-1) ; obj.Znames(ind+1:end) ];
        % --- Find L matrix from signal names ----
        obj = update_model(obj);
      else
        error(['External output named ' name ' was not found.']);
      end      
    end %remove_output
    
    
    
    % =====================================================================
    function obj = add_ext_out(obj,name)
      %% Add External Output
      % |ADD_EXT_OUT(OBJ,NAME)| adds external output named NAME.
      
      obj.Znames{end+1,1} = name;
      
      % --- Find L matrix from signal names ----
      obj = update_model(obj);
    end %add_input    

    
    
    % =====================================================================
    function display(obj)
      %% Display Object
      % |DISPLAY| Displays basic information about LSmodel.
      if isempty(obj.M) && isempty(obj.sum),
        disp('Empty large scale model.');
      else
        order = zeros(1,obj.N);
        for i=1:obj.N
          order(i) = size(obj.M{i}.A,1);
        end       
        disp(['Large scale model (total order=' num2str(sum(order)) ', ' num2str(obj.N) ' subsystem(s), ' num2str(obj.Ns) ' summator(s)):']);
        disp('  --- Subsystems -----------');
        for i=1:obj.N,
          n_in   = size( obj.M{i}.B , 2 );
          n_out  = size( obj.M{i}.C , 1 );
          n_n    = size( obj.M{i}.A , 1 );
          n_name = obj.M{i}.Name;
          disp(['    M' num2str(i) ': ' num2str(n_in) ' inputs, ' num2str(n_out) ' outputs, order=' num2str(n_n) ', name="' num2str(n_name) '"']);
        end
        if obj.Ns>0,
          disp('  --- Summators -----------------');
          for i=1:obj.Ns,
            n_in   = size( obj.sum{i}.B , 2 );
            n_name = obj.sum{i}.Name;
            disp(['    Sum' num2str(i) ': ' num2str(n_in) ' inputs, name = "' num2str(n_name) '"']);
          end
        end
        disp('  --- External Inputs -----------');
        for i=1:length(obj.Wnames),
          sig_ind = find( strcmp({obj.signal.name},obj.Wnames{i}) ,1);                    
          if isempty(sig_ind), 
            fprintf('    IN%02d: %-16s\n',i,obj.Wnames{i});  
            continue;
          end
          tmp=obj.signal(sig_ind).limit;
          flds = fields(tmp);
          ss = '';
          for j=1:length(flds)
            if ~isinf( tmp.(flds{j}) ),
              ss = [ss flds{j} '=' num2str(tmp.(flds{j})) ','];
            end            
          end
          if ~isempty(ss), ss = [' (' ss(1:end-1) ')']; end;
          %disp([ '    In' num2str(i) ':  ' obj.Wnames{i}]);
          fprintf('    IN%02d: %-16s %s %s\n',i,obj.signal(sig_ind).name,obj.signal(sig_ind).type,ss);  
        end
        disp('  --- External Outputs ----------');
        for i=1:length(obj.Znames),          
          sig_ind = find( strcmp({obj.signal.name},obj.Znames{i}) ,1);                    
          if isempty(sig_ind), 
            fprintf('    OUT%02d: %-15s\n',i,obj.Znames{i});           
            continue;
          end          
          tmp=obj.signal(sig_ind).limit;
          flds = fields(tmp);
          ss = '';
          for j=1:length(flds)
            if ~isinf( tmp.(flds{j}) ),
              ss = [ss flds{j} '=' num2str(tmp.(flds{j})) ','];
            end            
          end
          if ~isempty(ss), ss = [' (' ss(1:end-1) ')']; end;          
          %disp(['    Out' num2str(i) ': ' obj.Znames{i}]);
          fprintf('    OUT%02d: %-15s %s %s\n',i,obj.signal(sig_ind).name,obj.signal(sig_ind).type,ss);           
        end
      end
      disp(' ');
    end %display

    
    
    % =====================================================================
    function n = n(obj)
      %% Return Global Model Order
      % |N=N(OBJ)| returns global model order as a sum of subsystem
      % orders.
      n = sum(obj.orders);
    end %orders
    
    
    
    % =====================================================================
    function n = orders(obj)
      %% Return Orders of Subsystems
      % |N=ORDERS(OBJ)| returns orders of subsystems in array
      for i=1:obj.N,
        n(i) = size( obj.M{i}.A , 1);
      end
    end %orders
    
    
    
    % =====================================================================
    function mod = ss(obj)
      %% Get Global SS Model
      % |MOD=SS(obj)| returns state space model of the whole model.
      for i=1:obj.N,
        if i==1,
          MM = ss(obj.M{i});
        else
          MM = append( MM , ss(obj.M{i}) );
        end
      end
      
      L = [obj.Lzw,obj.Lzy;obj.Luw,obj.Luy];
      
      Kw  = [eye(obj.nw)      ; zeros(obj.ny,obj.nw)];
      Ky  = [zeros(obj.nw,obj.ny) ; eye(obj.ny) ];
      Kz  = [eye(obj.nz)      , zeros(obj.nz,obj.nu)];
      Ku  = [zeros(obj.nu,obj.nz) , eye(obj.nu) ];
      
      mod = Kz * feedback(L,Ky*MM*Ku,+1) * Kw;
      
      mod.InputName  = obj.Wnames;
      mod.OutputName = obj.Znames;
    end %ss
    
  
    
    % =====================================================================
    function obj = eps(obj,n_target)
      %% Epsilon Decomposition
      % |EPS(OBJ,N_TARGET)| finds \epsilon decomposition of water network
      % model after leafs condensation. Decomposes the network to N_TARGET
      % subnetworks.
      
      for i = 1:obj.N,
        ssM = ss(obj.M{i});
        nu(i) = size(ssM.B,2);
        ny(i) = size(ssM.C,1);
      end
      
      w = zeros(1,obj.N);
      Lcell = mat2cell([obj.Luw obj.Luy],nu,obj.nw+obj.ny);
      for i=1:obj.N,
        w(i) = sum( any(Lcell{i},1) );
      end
           
      Lcell = mat2cell(obj.Luy,nu,ny);
      C = zeros(obj.N);
      for i = 1:size(Lcell,1),
        for j = 1:size(Lcell,2),
          %C(i,j) = norm(Lcell{i,j},2);
          C(i,j) = (w(i)+w(j)) * any(any(Lcell{i,j}));
        end
      end
           
      [~,perm,block,~] = epsdec_groups(C,n_target);  
            
      %cond_gr = block(perm);  % this was WRONG !!!
      
      tmp = zeros(size(perm));  
      for i=1:length(perm),
        tmp(perm(i)) = block(i);
      end
      obj.grouping = tmp;      
      
    end %eps
    
    
    
    % =====================================================================
    function obj = set_sig_type(obj,names,type)
    %% Set Signal Type   
    % |SET_SIG_TYPE(OBJ,NAMES,TYPE)| - set type of signals for MPC
    % design. Signal types are compatible with MPC toolbox:    
    %
    %  -------------------------------------------------------
    %  | Signal Type               | Abbreviation |  Channel |
    %  -------------------------------------------------------
    %  | Manipulated               |     MV       |  Input   |
    %  | Measured Disturbances     |     MD       |  Input   |
    %  | Unmeasured Disturbances   |     UD       |  Input   |
    %  | Measured Outputs          |     MO       |  Output  |
    %  | Unmeasured Outputs        |     UO       |  Output  |
    %  | Internal Signal           |     X        |    --    |
    %  -------------------------------------------------------
    %
    % MODEL is name or index of submodel. SIGNALS is a cell of input names
    % or their indexes and TYPE is abbreviation of signal type (MV,MD,UD).
    
    
      % --- Check signal type ------------------------
      allowed_types = {'MV','MD','UD','MO','UO','X'};
      if ~any(strcmp(type,allowed_types)),
        error(['Signal type "' type '" is not valid.']);
      end    
    
      if ischar(names), names = {names}; end
    
      % --- Modify signal type(s) --------------------
      for i=1:length(names),
        ind = find( strcmp({obj.signal.name},names{i}) , 1 );
        if isempty(ind),
          error(['Signal "' names{i} '" was not found.']);          
        end
        obj.signal(ind).type = type;
      end
      
    end %set_sig_type
    
    
    
    % =====================================================================
    function obj = set_sig_lim(obj,names,varargin)
      %% Set Signal Limits
      % |SET_SIG_LIM(OBJ,NAMES,MIN,MAX,DMIN,DMAX,SMIN,SMAX)| sets MIN / MAX
      % limits on absolute values, sets DMIN / DMAX limits on speed of 
      % change and sets SMIN / SMAX soft limits on absolute values. Limits
      % are applied to signal(s) specified by name in NAMES
      %
      % |SET_SIG_LIM(OBJ,NAMES,MIN,MAX,DMIN,DMAX)| sets MIN / MAX limits on
      % absolute values and sets DMIN / DMAX limits on speed of change.
      %
      % |SET_SIG_LIM(OBJ,NAMES,MIN,MAX)| sets MIN / MAX limits on absolute
      % values.
      %
      % |SET_SIG_LIM(OBJ,NAMES)| removes all limits (sets them to inf).
      %
      
      % --- Get Limits -------------------------------
      lim.min  = -inf;
      lim.max  = inf;
      lim.dmin = -inf;
      lim.dmax = inf;
      lim.smin = -inf;
      lim.smax = inf;        
      
      if nargin==2,             % remove limits        
      elseif nargin==4,         % min max limits
        lim.min  = varargin{1};
        lim.max  = varargin{2};                
      elseif nargin==6,         % min max dmin dmax limits
        lim.min  = varargin{1};
        lim.max  = varargin{2};
        lim.dmin = varargin{3};
        lim.dmax = varargin{4};
      elseif nargin==8,         % min max dmin dmax smin smax limits
        lim.min  = varargin{1};
        lim.max  = varargin{2};
        lim.dmin = varargin{3};
        lim.dmax = varargin{4};
        lim.smin = varargin{5};
        lim.smax = varargin{6};
      else
        error('Number of parameters error.');
      end
      
      if isempty(lim.min),  lim.min  = -inf; end
      if isempty(lim.max),  lim.max  =  inf; end
      if isempty(lim.dmin), lim.dmin = -inf; end
      if isempty(lim.dmax), lim.dmax =  inf; end
      if isempty(lim.smin), lim.smin = -inf; end
      if isempty(lim.smax), lim.smax =  inf; end
      
      
      % --- Modify signal type(s) --------------------
      if ischar(names), names = {names}; end
      for i=1:length(names),
        ind = find( strcmp({obj.signal.name},names{i}) , 1 );
        if isempty(ind),
          error(['Signal "' names{i} '" was not found.']);          
        end
        obj.signal(ind).limit = lim;
      end      
      
    end %set_sig_lim
    
        
    
    % =====================================================================
    function obj = set_sig_data(obj,names,data)
      %% Set Signal Data
      % |SET_SIG_DATA(OBJ,NAMES,DATA)| assigns user data structure to
      % signal(s) specified in NAMES.
      
      if ischar(names), names = {names}; end
      for i=1:length(names),
        ind = find( strcmp({obj.signal.name},names{i}) , 1 );
        if isempty(ind),
          error(['Signal "' names{i} '" was not found.']);          
        end
        obj.signal(ind).data = data;
      end      
      
    end %set_sig_data
    
    
    % =====================================================================
    function [Mzw,Mzy,Muw,Muy] = masterL(obj)
      %% Master Connection Matrix
      % |[MZW,MZY,MUW,MUY] = MASTERL(OBJ)| returns aggregated matrices
      % indicating connection between external inputs, submodels and external
      % outputs.
                   
      for i = 1:obj.N,
        NU(i) = size(obj.M{i}.B,2);
        NY(i) = size(obj.M{i}.C,1);
      end
      
      NW = length(obj.Wnames);
      NZ = length(obj.Znames);           
      
      Mzw = any(any(obj.Lzw));
      
      Lcell = mat2cell(obj.Lzy,NZ,NY);
      for j = 1:size(Lcell,2),
        Mzy(1,j) = any(any(Lcell{1,j}));
      end
      
      Lcell = mat2cell(obj.Luw,NU,NW);
      for i = 1:size(Lcell,1),
        Muw(i,1) = any(any(Lcell{i,1}));
      end
      
      Lcell = mat2cell(obj.Luy,NU,NY);
      Muy = zeros(obj.N);
      for i = 1:size(Lcell,1),
        for j = 1:size(Lcell,2),
          Muy(i,j) = any(any(Lcell{i,j}));
        end
      end            
            
    end %masterL
    
   
    
    % =====================================================================
    function Nobj = select(obj,inputs,outputs,opt)      
    %% Extract model part for given external inputs and outputs
    % |NOBJ = SELECT(OBJ,INPUTS,OUTPUTS)| extracts part of the model for 
    % selected inputs and outputs. Submodels, which are not conotrllable or 
    % observable from selected inputs and outputs are not included in 
    % extracted model. INPUTS and OUTPUTS can be cell array of
    % inputs/outputs signal names or their numerical index (see DISP).
    %
    % |NOBJ = SELECT(OBJ,INPUTS,OUTPUTS,OPT)| parameter OPT controls, which
    % submodels are kept and which are removed from final model:
    %
    %  'io' ........ eliminates submodels, which are not controllable and/or
    %                observable
    %  'full' ...... keep all submodels
    %  'ss' ........ (default) eliminates submodels which are not 
    %                controllable and observable are removed
    %
    
      % --- Indexes of selected inputs/outputs -------
      if isnumeric(inputs),
        inp_ind = inputs;
      else
        if ~iscell(inputs),  inputs  = {inputs}; end
        inp_ind = zeros(1,obj.nw);
         for i=1:length(obj.Wnames),
           inp_ind(i) = any(strcmp(obj.Wnames{i},inputs));          
         end
        inp_ind = (inp_ind==1);
      end
      
      if isnumeric(outputs),
        out_ind = outputs;
      else
        if ~iscell(outputs), outputs = {outputs};end
        out_ind = zeros(obj.nz,1);
        for i=1:length(obj.Znames),
          out_ind(i) = any(strcmp(obj.Znames{i},outputs));
        end
        out_ind = (out_ind==1);
      end
      
      % --- Controlability / Observability -----------
      ssM = obj.ss;
      [A,B,C,D] = ssdata(ssM);
      B = B(:,inp_ind);
      C = C(out_ind,:);
      D = D(out_ind,inp_ind);
      ssM = ss(A,B,C,D,ssM.Ts);
      
      Co = ctrb(ssM);
      Ob = obsv(ssM);
      
      % --- Selection of submodels -------------------      
      if nargin<4,
        opt = 'ss';
      end
      
      if strcmpi(opt,'io'),
        tot = sum(Co~=0,2)>0 & (sum(Ob~=0,1)>0)';
      elseif strcmpi(opt,'full'),
        tot = ones(size(Co,1),1);
      else
        tot = sum(Co~=0,2)>0 | (sum(Ob~=0,1)>0)';
      end
      
      order = zeros(1,obj.N);
      for i=1:obj.N
        order(i) = size(obj.M{i}.A,1);
      end
                  
      tot = mat2cell(tot,order);
      sel = zeros(1,length(tot));
      for i=1:length(tot),
        sel(i) = any(tot{i});
      end
      
      % --- Select model part ------------------------      
%       Nobj = obj;
%       Nobj.M = Nobj.M(sel==1);
%       Nobj.Morig = Nobj.Morig(sel==1);
%       Nobj.grouping = Nobj.grouping(sel==1);
%       Nobj.Wnames = Nobj.Wnames(inp_ind);
%       Nobj.Znames = Nobj.Znames(out_ind);
%       Nobj = Nobj.update_model;
%       Nobj.modified = 1;
      
      Nobj = obj.rem_mod( find( ~sel ) );
      
    end %select

    
    
    % =====================================================================
    function Nobj = group(obj,gr_ind)
      %% Extract submodels belonging to given group
      % |NOBJ = GROUP(OBJ,GR_IND)| extracts parts of LS model, where
      % submodels belong to the group(s) specified in vector GR_IND.
      %
      
      ind = ismember( obj.grouping , gr_ind );    
      
      Nobj = obj.rem_mod( find( ~ind ) );
      
      
    end %group
    

    % =====================================================================
    function obj = squeeze(obj)
      %% Remove unused external inputs/outputs and submodels
      % |NOBJ = SQUEEZE(OBJ)| removes external inputs and outputs, which
      % are not connected to any submodels and also removes submodels which
      % are not connected anything.
      %
      
      % --- Inputs ----------------------------------------
      inp2keep  = sum( [obj.Lzw ; obj.Luw] , 1)' ~= 0;      
      
      % --- Outputs ---------------------------------------
      out2keep  = sum( [obj.Lzy , obj.Lzw] , 2) ~= 0;
            
      % --- Submodels -------------------------------------      
      for i = 1:obj.N,
        ssM = ss(obj.M{i});
        nu(i) = size(ssM.B,2);
        ny(i) = size(ssM.C,1);
      end
      
      Lcell = mat2cell( [obj.Luw , obj.Luy] , nu , length(obj.Wnames)+sum(ny) );      
      has_input = zeros(1,obj.N);
      for i = 1:size(Lcell,1),
        has_input(i) = any(any(Lcell{i,1}~=0));
      end
      
      Lcell = mat2cell( [obj.Lzy ; obj.Luy] , length(obj.Znames)+sum(nu) , ny );      
      has_output = zeros(1,obj.N);
      for j = 1:size(Lcell,2),
        has_output(i) = any(any(Lcell{1,j}~=0));
      end                  
      
      mod2remove = find( ~(has_input | has_output) );
      
      
      % --- Apply changes ---------------------------------
      obj.Wnames = obj.Wnames(inp2keep);
      obj.Znames = obj.Znames(out2keep);      
      obj = obj.rem_mod( mod2remove );      
      
    end %group

    
    
    % =====================================================================
    function obj = struct_red(obj,n_target,method)
    %% Structured Model Order Reduction
    % |OBJ = STRUCT_RED(OBJ,N_TARGET)| reduces total model order to 
    % N_TARGET while preserving model structure.
    %
    % *Algorithm:*
    %   Henrik Sandberg, Richard M. Murray: "Model reduction of
    %   interconnected linear systems".   Optimal Control, Applications and
    %   Methods, Special Issue on Directions, Applications, and Methods in
    %   Robust Control,  30:3, pp. 225--245, May/June 2009.
      
      % --- Compute full grammians -----------------------
      ssM = obj.ss;
      P = gram(ssM,'c');
      Q = gram(ssM,'o');
            
      % --- Separate Grammians -----------------------
      order = zeros(1,obj.N);
      for i=1:obj.N
        order(i) = size(obj.M{i}.A,1);
      end
      
      for i=1:obj.N,
        if i==1,
          i1 = 1;
        else
          i1 = sum(order(1:i-1))+1;
        end        
        i2 = sum(order(1:i));
        PP{i} = P( i1:i2 , i1:i2 );
        QQ{i} = Q( i1:i2 , i1:i2 );
        
        % --- Individual balancing transformations ---
        R = chol(QQ{i});
        [U,Sigma2,V] = svd(R*PP{i}*R');
        Sigma    = diag(sqrt(diag(Sigma2)));
        sig{i,1} = diag(Sigma);
        T{i}     = inv(R) * U * sqrt(Sigma);        
        
        % --- Submodels transformation ---------------
        obj.M{i}.A = T{i} \ (obj.M{i}.A * T{i});
        obj.M{i}.B = T{i} \ obj.M{i}.B;
        obj.M{i}.C = obj.M{i}.C * T{i};        
      end
      
      % --- Find target orders of individual submodels ----
      tag = [];
      for i=1:obj.N,
        tag = [ tag ; i*ones(order(i),1) ];
      end
      
      [sig_sorted,ind] = sort( cell2mat(sig) ,'descend' );
      tag = tag(ind); % indicates singular value corresponding model      
      
      % --- Plot singular numbers --------------------
      col = jet(obj.N)/1.1;
      leg = {};
      for i = 1:obj.N,
        semilogy( find(tag==i), sig_sorted(tag==i), 'o', ...
                  'Color', col(i,:), 'LineWidth', 2, ...
                  'MarkerFaceColor', col(i,:) , 'MarkerSize', 8 ); 
        %leg = [ leg ; ['submodel ' num2str(i)] ];
        leg = [ leg ; ['M_{' num2str(i) '} - "' obj.M{i}.Name] '"' ];
        hold on;
      end
      y_cut = mean([ sig_sorted(n_target) sig_sorted(n_target+1)]);
      plot([1 sum(order)], y_cut*ones(1,2), 'r', 'LineWidth', 2);
      title('\bfSorted Singular Numbers');
      legend([ leg ; 'threshold'],'Location','BestOutside');
      grid on;
      xlim([1 length(sig_sorted)]);
                  
      tag = tag(1:n_target);
      
      clear nn
      for i=1:obj.N,
        nn(i) = sum(tag==i);
      end
      
      % --- Reduce submodels -------------------------
      for i = 1:obj.N,
        elim = ones(1,order(i));
        elim(1:nn(i)) = 0;
        name = obj.M{i}.Name;
        obj.M{i} = modred(obj.M{i},elim==1);
        obj.M{i}.Name = name;
      end
     
    end %struct_red
    
    
    
    % =====================================================================
    function obj = merge(obj,mod,new_mod, target_nb, target_na)
    %% Merging of uncertain models
    % |MERGE(OBJ,MOD,NEW_MOD,TARGET_NB,TARGET_NA)| updates uncertain 
    % submodel by new model. Updated submodel is determined by its index or 
    % its name in variable MOD and new model is specified by NEW_MOD.
    % This function supports so far only ARX models (IDARX or IDPOLY
    % classes of submodels).
    %
    % *Algorithm:*
    %
    % P. Trnka and V. Havlena, Overlapping models merging and 
    % interconnection for large-scale model management. Proceedings of 
    % IEEE Multi-Conference on Systems and Control. 2010.
    %
        
      % --- Find model index from name ---------------
      if ischar(mod),
        m_ind = 0;
        for i=1:obj.N,
          if strcmp(obj.M{i}.Name,mod),
            m_ind = i;
            break;
          end
        end
        if ~m_ind,
          error(['Model named ' mod ' was not found.']);          
        end
      else
        m_ind = mod;
      end
      
      % --- Check model types ------------------------
      if ~isa(new_mod,'idarx') && ~isa(new_mod,'idpoly'),
        error('New model must be IDARX or IDPOLY model.');
      end
      
      if ~isa(obj.Morig{m_ind},'idarx') && ~isa(obj.Morig{m_ind},'idpoly'),
        error('Original model has not IDARX or IDPOLY type.');
      end
      
      % --- Merging II -------------------------------      
      theta1 = obj.Morig{m_ind}.ParameterVector;  theta1 = theta1(end:-1:1);
      P1     = obj.Morig{m_ind}.CovarianceMatrix; P1     = P1(end:-1:1,end:-1:1);
      mod1.theta = theta1;
      mod1.P     = P1;
      mod1.nb    = obj.Morig{m_ind}.nb;
      mod1.na    = obj.Morig{m_ind}.na;
      mod1.s     = obj.Morig{m_ind}.NoiseVariance;
      
      theta2 = new_mod.ParameterVector;  theta2 = theta2(end:-1:1);
      P2     = new_mod.CovarianceMatrix; P2     = P2(end:-1:1,end:-1:1);
      mod2.theta = theta2;
      mod2.P     = P2;
      mod2.nb    = new_mod.nb;
      mod2.na    = new_mod.na;
      mod2.s     = new_mod.NoiseVariance;
      
      mod12 = MergeARXs(mod1,mod2,target_nb,target_na);
      
      obj.Morig{m_ind}.nb               = mod12.nb;      
      obj.Morig{m_ind}.na               = mod12.na;
      obj.Morig{m_ind}.ParameterVector  = mod12.theta(end:-1:1);
      obj.Morig{m_ind}.CovarianceMatrix = mod12.P(end:-1:1,end:-1:1);      
      obj.Morig{m_ind}.NoiseVariance    = mod12.s;
      
      name = obj.M{m_ind};
      obj.M{m_ind}                      = ss( obj.Morig{m_ind} );
      obj.M{m_ind} = name;
      
    end %merge
    
    
    
    % =====================================================================
    function [mean,var] = freq_uncert(obj,mod,w)
    %% Frequency uncertainty of ARX model
    % |[MEAN,VAR] = FREQ_UNCERT(OBJ,MOD,W)| returns mean value and
    % variation for model MOD on frequencies W.
    % This function supports so far only ARX models (IDARX or IDPOLY
    % classes of submodels).
        
      % --- Find model index from name ---------------
      if ischar(mod),
        m_ind = 0;
        for i=1:obj.N,
          if strcmp(obj.M{i}.Name,mod),
            m_ind = i;
            break;
          end
        end
        if ~m_ind,
          error(['Model named ' mod ' was not found.']);          
        end
      else
        m_ind = mod;
      end
      
      % --- Frequency uncertainty -------------------------
      m = obj.Morig{m_ind};
      
      nb    = m.nb;
      na    = m.na;
      nk    = m.nk;
      Ts    = m.Ts;
      T = [nb:-1:1  na+nb:-1:nb+1];
      
      theta = m.ParameterVector;
      theta = theta(end:-1:1);
      theta = theta(T);
      
      P     = m.NoiseVariance * m.CovarianceMatrix;
      P     = P(end:-1:1,end:-1:1);
      P     = P(T,T);
      
      [mean,var] = ARXfreqUncertainty(theta,P,nb,na,nk,w,Ts);
      
    end %freq_uncertainty
          
      
    

  end %methods ============================================================
  
  
  
  methods (Access=protected)
    
    % =====================================================================
    function obj = add_default_io(obj,mod)      
      % Add default signal types. Note that signal type is valid only for 
      % unconnected signals.      
      
      mod_StateName = mod.StateName;
      if isempty(mod_StateName{1}),
        mod_StateName = {};
      end
      
      IXO    = [ mod.InputName ; mod_StateName ; mod.OutputName ];
      n_inp    = length(mod.InputName);      
      n_states = length(mod_StateName);      
      if isempty(obj.signal(1).name),
        n_sig = 0;
      else
        n_sig = length({obj.signal.name});
      end

      % --- Default limits ---------------------------      
      lim.min  = -inf;
      lim.max  = inf;
      lim.dmin = -inf;
      lim.dmax = inf;
      lim.smin = -inf;
      lim.smax = inf;        
      
      for i=1:length(IXO),
        if any(strcmp({obj.signal.name},IXO{i})), % check for duplicity
          continue;
        end
        sig.name = IXO{i};
        sig.tag  = [];
        sig.type = 'X';
%         if i<=n_inp,
%           sig.type = 'MV';
%         else
%           sig.type = 'MO';
%         end
        sig.limit = lim;
        sig.price = [];
        sig.data  = [];
        obj.signal(n_sig+1) = sig;
        n_sig = n_sig + 1;
      end
      
    end %add_default_io
    
    
    
    % =====================================================================
    function obj = update_model(obj)
      % Updates interconnection matrices Lzw,Lzy,Luw,Luy by connecting
      % signals with the same name. 
      obj.nw = length(obj.Wnames);
      obj.nz = length(obj.Znames);
      obj.N  = length(obj.M);
      
      % --- Model I/O Names ----------------------
      Unames = {};
      Ynames = {};
      for i=1:length(obj.M),
        Unames = [ Unames ; obj.M{i}.InputName ];
        Ynames = [ Ynames ; obj.M{i}.OutputName ];
      end
      obj.nu = length(Unames);
      obj.ny = length(Ynames);
      
      % --- Summator Output Names ----------------
      obj.Ns = length(obj.sum);
      Snames = {};
      for i=1:obj.Ns,
        Snames = [ Snames ; obj.sum{i}.OutputName ];
      end
      
      % --- Stacking Names -----------------------
      WYSnames = [ obj.Wnames ; Ynames ; Snames ];
      ZUnames =  [ obj.Znames ; Unames ];
      
      % --- Interconnection [A11 A12] ------------
      A1 = zeros(length(ZUnames),length(WYSnames)); %[A11 A12]
      for i=1:length(ZUnames),
        for j=1:length(WYSnames),
          if strcmp(ZUnames{i},WYSnames{j}),
            A1(i,j) = 1;
          end
        end
      end
      
      % --- Interconnection [A21 A22] ------------
      A2 = zeros(obj.Ns,length(WYSnames)); %[A21 A22]
      for i=1:obj.Ns,
        for j=1:length(obj.sum{i}.InputName)
          match = strcmp( obj.sum{i}.InputName{j} , WYSnames );
          %if ~any(match) || sum(match)>1, error('!'); end
          A2(i,match) = obj.sum{i}.D(1,j);
        end
      end
      
      % --- Separate A1,A2 to A11,A12,A21,A22 ----
      n1  = obj.nw+obj.ny;
      A11 = A1(:,1:n1);
      A12 = A1(:,n1+1:end);
      A21 = A2(:,1:n1);
      A22 = A2(:,n1+1:end);
      
      % --- Eliminate Summators ------------------
      if isempty(obj.sum),
        L = A11;
      else
        if rank(eye(size(A22))-A22)==0,
          error('Models cannot be combined.');
        end
        
        %L = A11 + A12*inv(eye(size(A22))-A22)*A21;
        L = A11 + A12*( (eye(size(A22))-A22) \ A21 );
      end
      
      % --- Separate L ---------------------------
      obj.Lzw = L( 1:obj.nz , 1:obj.nw );
      obj.Lzy = L( 1:obj.nz , obj.nw+1:end );
      obj.Luw = L( obj.nz+1:end , 1:obj.nw );
      obj.Luy = L( obj.nz+1:end , obj.nw+1:end );
      
      % --- Update Signals -----------------------
      % <TODO>
      
    end %update_model      

  end %protected methods ==================================================
  
  
end %classdef


%% Plot LS model structure
% PLOT(MOD) plots structure of large scale model defined in MOD (LSmodel
% class object). The structure is plotted as a graph, where vertexes are 
% subsystems and edges indicate interaction between subsystems. Small
% triangles at vertex edges indicate which subsystems have direct
% connection to external inputs/outputs.
%
% PLOT(MOD,'IO') plots inputs and outputs as additional vertexes.
%
% It is useful to call PLOT command as
%
%   MOD = PLOT(MOD)
%
% as plot automatically computes graph vertex position and stores it for
% later use. The computation can be time consuming for larger models (>50
% vertexes).
%






