% PLOT(MOD) plots structure of large scale model defined in MOD (LSmodel
% class object). The structure is plotted as a graph, where vertexes are 
% subsystems and edges indicate interaction between subsystems.
%
% PLOT(MOD,'IO') plots inputs and outputs as additional vertex showing.
% Otherwiese small triangles are plotted to indicate which subsystems have
% direct connection to external inputs/outputs.
%
% It is useful to call PLOT command as
%
%   MOD = PLOT(MOD)
%
% as plot automatically computes graph vertex position and stores it for
% later use. The computation can be time consuming for larger models (>50
% vertexes).
%

function obj = plot(obj,opt)

% --- Default options ---------------------------
show_io = 0;          % show inputs/outputs as vertexes
show_triangles = 1;   % show input/output triangles
io_aside = 0;         % place inputs and outputs aside

% --- Options processing ------------------------
if nargin>=2,
  if strcmpi(opt,'io'),
    show_io = 1;
    show_triangles = 0;
  end
end


% -----------------------------------------------
M = obj.M;
n = obj.N;

for i = 1:n,
    ssM = ss(M{i});
    nu(i) = size(ssM.B,2);
    ny(i) = size(ssM.C,1);    
end

% --- Systems interactions ----------------------
Lcell = mat2cell(obj.Luy,nu,ny);
C = zeros(n);
vertex_name = {};
vertex_type = [];
for i = 1:size(Lcell,1),
    for j = 1:size(Lcell,2),
        C(i,j) = any(any(Lcell{i,j}));        
        %norm(Lcell{i,j},inf)
        %svd(Lcell{i,j})        
    end
    vertex_name{i} = [ 'M' int2str(i) ];
    vertex_type(i) = 1;
end

if show_io,
  % --- Inputs ------------------------------------
  Lcell = mat2cell(obj.Luw,nu,obj.nw);
  for i = 1:size(Lcell,1),
    Muw(i,:) = any(Lcell{i,1},1);
  end
  for i = 1:obj.nw,
    vertex_name{end+1} = [ 'u' int2str(i) ];
    vertex_type(end+1) = 2;
  end
  
  % --- Outputs -----------------------------------
  Lcell = mat2cell(obj.Lzy,obj.nz,ny);
  for j = 1:size(Lcell,2),
    Mzy(:,j) = any(Lcell{1,j},2);
  end
  for i = 1:obj.nz,
    vertex_name{end+1} = [ 'y' int2str(i) ];
    vertex_type(end+1) = 3;
  end
  C = [C Muw Mzy'; [Muw' ; Mzy]  zeros(obj.nw+obj.nz)];
end

% -----------------------------------------------
C = min(1,C+C');

% ---------------------------------------------------------

if obj.modified || ... % --- recompute vertex positions ---------
   size(obj.xy,1)~=size(C,1),
  
  fprintf('Computing vertex positions... ');
  
  if io_aside,
    xy = distxy(C(1:obj.N,1:obj.N));
  else
    xy = distxy(C);
  end
  
  % --- Normalize graph size -------------------------
  % By finding mean distance of neighbouring vertexes
  min_dst = inf;
  clear dst; cnt = 0;
  for i=1:n,
    for j=i+1:n,
      if C(i,j),
        cnt = cnt + 1;
        dst(cnt) = min( min_dst , sqrt( (xy(i,1)-xy(j,1))^2 + (xy(i,2) - xy(j,2))^2 ) );
      end
    end
  end
  
  % --- Adjust size
  if exist('dst','var'),
    xy = xy./mean(dst);
  end
  % --- Center
  xy = xy - repmat(mean(xy,1),size(xy,1),1);
  
  fprintf('Done.\n');
  obj.xy = xy;
  obj.modified = 0;
  
  if io_aside,
    min(xy(:,1)) 
    max(xy(:,1))
    xy = [xy ; [(min(xy(:,1))-1)*ones(obj.nw,1) linspace(min(xy(:,2)),max(xy(:,2)),obj.nw)']];
    xy = [xy ; [(max(xy(:,1))+1)*ones(obj.nz,1) linspace(min(xy(:,2)),max(xy(:,2)),obj.nz)']];
  end
  
else
  xy = obj.xy;
end

% ---------------------------------------------------------

n = obj.N;

has_input = zeros(n,1);
Lcell = mat2cell(obj.Luw,nu,size(obj.Luw,2));
for i=1:n,
    has_input(i) = any(any(Lcell{i,1}));
end

% ---------------------------------------------------------

has_output = zeros(n,1);
Lcell = mat2cell(obj.Lzy,size(obj.Lzy,1),ny);
for i=1:n,
    has_output(i) = any(any(Lcell{1,i}));
end

% ---------------------------------------------------------

for i = 1:size(C,1),
    for j = i+1:size(C,2),
        if C(i,j)
            if i>obj.N || j>obj.N,
              edge_color = 'k'; %0.5*ones(1,3);
            else
              edge_color = 'b';
            end
            plot( [xy(i,1) xy(j,1)] , [xy(i,2) xy(j,2)] , ...
                  'Color', edge_color, 'LineWidth' , 3); 
            hold on;
        end
    end
end

% ---------------------------------------------------------

% find group numbers and set group colors
tmp = zeros(1,max(obj.grouping));
if sum(obj.grouping)~=0,
  tmp(obj.grouping)=1; 
  indGroups=find(tmp); 
  nGroups=length(indGroups);
else
  indGroups = [0]; 
  nGroups   = 1;  
end

group_colors = 0.5 + 0.5*hsv(nGroups);

r = 0.3;
for i = 1 : size(xy,1),
    x = xy(i,1);
    y = xy(i,2);
    switch vertex_type(i),
      case 1    % subsystem
        vertex_curv  = .1;
        vertex_color = group_colors((indGroups==obj.grouping(i)),:);
        vertex_textc = 'k';
      case 2
        vertex_curv  = 1;
        vertex_color = 'w';       
        vertex_textc = 'k';       
      case 3
        vertex_curv  = 1;
        vertex_color = 'w';       
        vertex_textc = 'k';
    end
    rectangle('Position', [x-r/2, y-r/2, r, r],...
        'Curvature', vertex_curv*ones(1,2), ...  
        'EdgeColor', 'k', ...
        'FaceColor', vertex_color );
    text(x,y,vertex_name{i},'HorizontalAlignment','center','VerticalAlignment','middle','Fontsize',8,'Color',vertex_textc);
    if show_triangles,
      if i<=obj.N
        if has_input(i),
          plot(x-r/2+.1*r,y,'k>','MarkerFaceColor','k');
        end
        if has_output(i),
          plot(x+r/2-.1*r,y,'k>','MarkerFaceColor','k');
        end
      end
    end
end

axis off;
daspect([1 1 1]);

% ---------------------------------------------------------
clear c leg
for i=1:nGroups,
    c(i) = area([0 0],[0 0],'FaceColor',group_colors(i,:));
    leg{i} = ['Group ' num2str( indGroups(i))];
end

if nGroups>1,
  legend(c,leg,'Location','BestOutside');
end

set(gcf,'Color',0.95*ones(1,3));

%title(obj.name);

% =========================================================================
  function xy = distxy(A)
    % XY = distxy(A) -- give g a distance based embedding
    % (Modified version of distxy, which does not need original graph toolbox.)
    % we attempt to embed g in the plane so that the graph-theoretic distance
    % between vertices matches the eucliden distance.
    %
    % REQUIRES THE OPTIMIZATION TOOLBOX
    %
    % This function is from Matlab BGL Toolbox
    
    n = size(A,1);
    d = allpaths(A);    % finds shorthest distances
    
    [i,j] = find(d==inf);
    ni = length(i);
    for k=1:ni
      d(i(k),j(k)) = n/2;
    end
    
    fi = linspace(0,2*pi,n+1)';
    fi = fi(1:end-1);
    xy0 = 5*[cos(fi) , sin(fi)];
    
    %xy0 = 5*randn(n,2);
    
    opts = optimset('MaxIter', 10*n,'Display', 'off');
    [xy,e] = lsqnonlin(@dist_discrep, xy0, [], [], opts);
    
    
    %--------------------------------------------------------    
    function dd = dist_discrep(xy)      
      %divisor = d + eye(n);
      dxy = dist_pair(xy);
      divisor = real((d+eye(n)));
      % divisor = exp(d/4)-0.8;
      dd = d - dxy;
      dd = dd./divisor;
      dd = dd(:);      
    end
        
    %--------------------------------------------------------
    function D = dist_pair(xy)
      % find Euclidean distances between vertices in xy embedding
      n = size(xy,1); % get # of rows
      c = sum(xy.*xy,2); % norm^2 of rows
      Y = c * ones(1,n);
      D = Y + Y' - 2*xy*xy';
      D = real(sqrt(D));                 
    end % end of dist_pair
    
    %--------------------------------------------------------        
    
  end % end of distxy


% =========================================================================
end % end of plot




function B = allpaths(A)
% Rapidly returns the shortest node-to-node distance along the edges of a
% graph, for all nodes in the graph.
%
% USAGE: B = allspath(A)
%
% A = input distance matrix between nodes
% B = shortest path distance matrix between all nodes

B=full(A);
B(B==0)=Inf;
C=ones(size(B));
iter=0;
while any(C(:))
  C=B;
  B=min(B,squeeze(min(repmat(B,[1 1 length(B)])+...
    repmat(permute(B,[1 3 2]),[1 length(B) 1]),[],1)));
  C=B-C;
end
B(logical(eye(length(B))))=0;
end %end of allspath