%% WNmodel - Water Network Model Class
% WNmodel class is a child of large scale model LSmodel used for modeling
% of water (distribution) networks.
%
% *Methods:*
%
%   WNmodel       - constructor creating empty model or importing model from
%                   definition file
%   import_scheme - imports water network model from file
%   plot          - overloaded plot function
%
%   eps           - overloaded epsilon decomposition for water networks
%


%%
classdef WNmodel < LSmodel
  
  
 % === Read only properties ===============================================
  properties (GetAccess=public, SetAccess=protected)
    demand                  % cell array of demands for every tank/submodel
    sources                 % definition of water sources
  end %set protected properties   
  
  
  % === Public properties =================================================
  properties (Access = public)
  end %public properties
  
  
  methods  
    % =====================================================================
    function obj = WNmodel( varargin )
      %% WNmodel Constructor
      % |WNMODEL( FILENAME )| constructs empty water network model or 
      % imports model from given definition file (see |IMPORT_SCHEME| for
      % description of file formats).
      %
           
      if nargin==1,
        filename = varargin{1};
        if ischar(filename),          
          LSmodel_obj = obj.import_scheme(filename);
          % copy properties
          prop = properties(LSmodel_obj);
          for i=1:length(prop)
            obj.(prop{i}) = LSmodel_obj.(prop{i});
          end          
        else
          error('First paramter of WNmodel constructor should be file name.');
        end
      end
    end %WNmodel constructor
    
    
    
    % =====================================================================
    function Nobj = import_scheme(obj,filename)
      %% Import water network scheme description from file.
      % |NOBJ = IMPORT_SCHEME(OBJ,FILENAME)| import water netwerk scheme from
      % files.
      % Water network is defined by two files [filename].net and [filename].mat,
      % where '.net' defines network structure and '.net' defines variables
      % limits, prices, etc.
      %
      % *Structure definition file* is a simple text file describing individual
      % tanks and their interconnections by specifying signal names and
      % directions.
      %
      % The first line of each tank definition block specifies tank number and
      % its name:
      %
      %   Tank##,<tank name>
      %
      % or it can specify node with it number
      %
      %   Node##
      %
      % Following lines are same for tanks and nodes:
      %
      %   d,<demand name>
      %   s,<source name>
      %   +,<outlet pump/valve name>,<destination tank name>
      %   -,<inlet pump/valve name>,<source tank name>
      %
      % Tank/node definition blocks can be separated by empty line(s). Empty
      % lines and Matlab type comments are ignored.
      %
      % *Variable definition file* is standard Matlab MAT file with struct for
      % every signal defining limits by fields
      %
      %  .min  ... min value
      %  .max  ... max value
      %  .dmin ... min slope value
      %  .dmax ... max slope value
      %  .smin ... soft limit min value
      %  .smax ... soft limit max value
      %  .type ... signal type ('MV','MD','UD','MO','MV','X')
      %
      % *Example (part of definition file):*
      %
      %  Tank01,d450BEG
      %  d,c450BEG
      %  -,iBegues4,d369BEG
      %
      %  Tank02,d369BEG
      %  d,c369BEG
      %  +,iBegues4,d450BEG
      %  -,iBegues3,d255BEG
      %
      %  Node1
      %  s,AportA
      %  d,c82PAL
      %  +,vPalleja70,Node2
      %  +,vPapiolATLL,d110PAP
      %  +,iPapiol2AGBAR,d110PAP
      %  +,vFontSanta,d54REL
      %
      %  Node2
      %  d,c70PAL
      %  +,iPalleja4,d125PAL
      %  -,vPalleja70,Node1
      %
      %  <End of example>
      
      % Strip file extension ----------------------------------------------
      ind = findstr(filename,'.');
      if ~isempty(ind),
        filename = filename(1:ind-1);
      end
      
      netw_filename = [filename '.net'];
      data_filename = [filename '.mat'];
      
      sc = 3600;                    % scale flows from m3/s to m3/hrs
      recompute_graph = 1;          % recompute vertex positions for graph plot
      
      
      % Read Data File ----------------------------------------------------
      fid = fopen(netw_filename,'r');
      tline = fgetl(fid);
      i = 1;
      while ischar(tline)
        if ~isempty(tline),
          tline = tline(tline~=' ');      % trim out spaces
          if tline(1)~='%',               % ignore comments
            lines{i} = tline;
            i = i + 1;
          end
        end
        tline = fgetl(fid);
      end
      fclose(fid);
      
      
      % Read tank descriptions --------------------------------------------
      % and number them together as (interconnection) networks
      
      A = 0; B = []; C = 1; D = []; % empty tank model
      input_names = {}; state_names = {}; output_names = {};
      
      ext_inputs = {};  % together with disturbances (demands)
      ext_outputs = {};
      
      iTank = 0;
      for iline=1:length(lines),
        tline = lines{iline};
        
        
        % --- Valves / Pumps ---------------------------
        if tline(1)=='+' || tline(1)=='-',
          i1=3;
          i2=find(tline(3:end)==',',1)+1;
          if isempty(i2), error('Parsing Error');end
          pump_valve_name = tline(i1:i2);
          input_names = [ input_names , pump_valve_name ];
          if ~any(strcmp(ext_inputs,pump_valve_name)), % check for duplicity
            ext_inputs  = [ ext_inputs , pump_valve_name ];
          end
          if tline(1)=='+',
            B = [B -1];
          else
            B = [B +1];
          end
          D = [D 0];
        end
        
        
        % --- Sources ----------------------------------
        if tline(1)=='s',
          source_name = tline(3:end);
          B = [B +1];
          D = [D 0];
          input_names = [ input_names , source_name ];
          if ~any(strcmp(ext_inputs,source_name)), % check for duplicity
            ext_inputs  = [ ext_inputs  , source_name ];
          end
        end
        
        
        % --- Demands ----------------------------------
        if tline(1)=='d',
          demand_name = tline(3:end);
          B = [B -1];
          D = [D 0];
          input_names = [ input_names , demand_name ];
          if ~any(strcmp(ext_inputs,demand_name)), % check for duplicity
            ext_inputs  = [ ext_inputs , demand_name ];
          end
          
        end
        
        
        % --- First row of tank description ------------
        if strcmpi(tline(1:4),'Tank') || strcmpi(tline(1:4),'Node') || iline==length(lines),
          % --- Save new model ---------------
          if iTank>0 || iline==length(lines), % save new model
            mod = ss(A,B,C,D); %,-1);
            mod.Name = model_name;
            mod.InputName  = input_names;
            mod.StateName  = state_names;
            mod.OutputName = output_names;
            if ~any(strcmp(ext_outputs,model_name)), % check for duplicity
              ext_outputs  = [ ext_outputs  , model_name ];
            end
            %mod.UserData   =
            M{iTank} = mod;
            
            A = 0; B = []; C = 1; D = []; % empty tank model
            input_names = {}; state_names = {}; output_names = {};
          end
          iTank = iTank + 1;
          
          
          % --- Read model name --------------
          if strcmpi(tline(1:4),'Tank'),
            i1=find(tline==',',1)+1;
            if isempty(i1), error('Parsing Error');end
            model_name = tline(i1:end);
          else
            model_name = tline;
          end
          
          output_names = model_name;
          state_names  = model_name;
          
        end
        
      end %iline
      
      
      % Create LSmodel ---------------------------------------------------------
      Nobj = LSmodel( M , ext_inputs , ext_outputs );
            
      
      % Signal Types and Limits ------------------------------------------------
      eval(['load ' data_filename]);
      
      
      % Valves Combination (MANUAL!) --------------------------------------
      %combines duplicite valves connecting two same tanks in the same direction.
      if strcmpi(filename(1:3),'bcn'),
        vMix1.data     = vTerCarmel.data     + vTrinitat130.data;
        vMix1.flow_min = vTerCarmel.flow_min + vTrinitat130.flow_min;
        vMix1.flow_max = vTerCarmel.flow_max + vTrinitat130.flow_max;
        
        vMix2.data     = vTrinitat100.data     + vSifoTer.data;
        vMix2.flow_min = vTrinitat100.flow_min + vSifoTer.flow_min;
        vMix2.flow_max = vTrinitat100.flow_max + vSifoTer.flow_max;
        
        vMix3.data     = vSarriaCS.data;
        vMix3.flow_min = vSarriaCS.flow_min - vSarriaSC.flow_max;
        vMix3.flow_max = vSarriaCS.flow_max - vSarriaSC.flow_min;
      end
      % -------------------------------------------------------------------
      
      
      for i=1:length(Nobj.signal),
        sig_name = Nobj.signal(i).name;
        switch lower(sig_name(1))
          
          case 'd'            % --- TANK -------------------
            Nobj = Nobj.set_sig_lim(sig_name, ...
              eval([sig_name '.vol_min']), ...
              eval([sig_name '.vol_max']) );
            Nobj = Nobj.set_sig_type(sig_name,'MO'); % state / measured output
            
          case {'i','v'}      % --- PUMP/VALVE -------------
            Nobj = Nobj.set_sig_lim(sig_name, ...
              eval([sig_name '.flow_min']), ...
              eval([sig_name '.flow_max']) );
            Nobj = Nobj.set_sig_type(sig_name,'MV'); % manipulated variable
            if isfield(eval(sig_name),'data'),
              Nobj = Nobj.set_sig_data(sig_name,eval([sig_name '.data']));
            else
              disp(['No data for signal ' sig_name]);
              %Nobj = Nobj.set_sig_data(sig_name,0);
              Nobj = Nobj.set_sig_data(sig_name,1);
            end
            
          case 'c'            % --- DEMAND -----------------
            Nobj = Nobj.set_sig_type(sig_name,'MD'); % measured disturbance
            Nobj = Nobj.set_sig_data(sig_name,eval([sig_name '.data']));
            
          case 'a'            % --- WATER SOURCE -----------
            Nobj = Nobj.set_sig_type(sig_name,'MV'); % manipulated variable
            
          case 'n'            % --- NODE -------------------
            Nobj = Nobj.set_sig_type(sig_name,'X');  % internal signal
            
          otherwise
            error(['Unknown signal "' sig_name '"']);
            
        end
      end
    end %import
      
    
    
    % =====================================================================
    function display(obj)
      %% Display Object
      % |DISPLAY| shows basic information about water network model.
      if isempty(obj.M) && isempty(obj.sum),
        disp('Empty water network model.');
      else
        order = zeros(1,obj.N);
        for i=1:obj.N
          order(i) = size(obj.M{i}.A,1);
        end       
        disp(['Water network model (' num2str(obj.N) ' subsystem(s), ' num2str(obj.Ns) ' summator(s)):']);
        disp('  --- Tanks & Nodes ----------------');
        for i=1:obj.N,
          n_in   = size( obj.M{i}.B , 2 );
          n_out  = size( obj.M{i}.C , 1 );
          n_n    = size( obj.M{i}.A , 1 );
          n_name = obj.M{i}.Name;
          if strcmpi(n_name(1:4),'node'),
            %disp(['    M' num2str(i), ': "' n_name '" with ' num2str(n_in) ' inputs']);
            fprintf('    M%02d: %-17s, %d inputs\n',i,n_name,n_in');            
          else
            %disp(['    M' num2str(i), ': Tank "' n_name '" with ' num2str(n_in) ' inputs']);
            fprintf('    M%02d: Tank %-12s, %d inputs\n',i,['"' n_name '"'],n_in');            
          end          
        end
        
        if obj.Ns>0,
          disp('  --- Summators -----------------');
          for i=1:obj.Ns,
            n_in   = size( obj.sum{i}.B , 2 );
            n_name = obj.sum{i}.Name;
            disp(['    Sum' num2str(i) ': ' num2str(n_in) ' inputs, name = "' num2str(n_name) '"']);
          end
        end
        disp('  --- External Inputs -----------');
        for i=1:length(obj.Wnames),
          sig_ind = find( strcmp({obj.signal.name},obj.Wnames{i}) ,1);                    
          if isempty(sig_ind), error('???');end
          tmp=obj.signal(sig_ind).limit;
          flds = fields(tmp);
          ss = '';
          for j=1:length(flds)
            if ~isinf( tmp.(flds{j}) ),
              ss = [ss flds{j} '=' num2str(tmp.(flds{j})) ','];
            end            
          end
          if ~isempty(ss), ss = [' (' ss(1:end-1) ')']; end;
          %disp([ '    In' num2str(i) ':  ' obj.Wnames{i}]);
          fprintf('    IN%02d: %-15s %s %s\n',i,obj.signal(sig_ind).name,obj.signal(sig_ind).type,ss);  
        end
        disp('  --- External Outputs ----------');
        for i=1:length(obj.Znames),          
          sig_ind = find( strcmp({obj.signal.name},obj.Znames{i}) ,1);                    
          if isempty(sig_ind), error('???');end          
          tmp=obj.signal(sig_ind).limit;
          flds = fields(tmp);
          ss = '';
          for j=1:length(flds)
            if ~isinf( tmp.(flds{j}) ),
              ss = [ss flds{j} '=' num2str(tmp.(flds{j})) ','];
            end            
          end
          if ~isempty(ss), ss = [' (' ss(1:end-1) ')']; end;          
          %disp(['    Out' num2str(i) ': ' obj.Znames{i}]);
          fprintf('    OUT%02d: %-15s %s %s\n',i,obj.signal(sig_ind).name,obj.signal(sig_ind).type,ss);           
        end
      end
      disp(' ');
    end %display
    
    
    % =====================================================================
    function obj = eps(obj,n_target)
      %% Epsilon Decomposition
      % |EPS(OBJ,N_TARGET)| finds \epsilon decomposition of water network
      % model after leafs condensation. Decomposes the network to N_TARGET
      % subnetworks.
      
      I = obj.incidence_matrix;
      
           
      % --- Leafs Condensation ------------------
      
      A = I>0;
      A = A .* (1-diag(ones(size(A,1),1)));
                 
      Acond = A;
      Fcond = I;
      Grouping = diag(1:obj.N);
      
      iTank = 1;
      no_action = 1;
      untouched = 1:obj.N;
      while true,
        if sum(Acond(iTank,:))==1, % condensate if conected only by single edge
          connected2 = find(Acond(iTank,:),1);
          sel = [ 1:iTank-1 , iTank+1:size(Acond,1) ];
          Acond = Acond(sel,sel);
          Fcond = Fcond(sel,sel);
          Grouping(connected2,:) = Grouping(connected2,:) + Grouping(iTank,:);
          Grouping = Grouping(sel,:);
          untouched = untouched(sel);
          no_action = 0;
        else
          iTank = iTank + 1;
        end
        if iTank>size(Acond,1),
          if no_action, break; end
          no_action = 1;
          iTank = 1;
        end
      end
      
      
      % --- Epsilon Decomposition ---------------
      I = Fcond;
               
      [~,perm,block,~] = epsdec_groups(I,n_target,'inv');  
      
      %cond_gr = block(perm);  % this was WRONG !!!
      
      tmp = zeros(size(perm)); 
      for i=1:length(perm),
        tmp(perm(i)) = block(i);
      end
      cond_gr = tmp;
      
      % --- Regroup according to eps-dec ---      
      for i=1:max(cond_gr),
        GroupingNEW(i,:) = sum( Grouping(cond_gr==i,:) , 1);
      end
      Grouping = GroupingNEW;
      
      
      % Assign group numbers to vertexes ---------------------------------------
      
      % aggregate groups with less then given group size
      gr_size_limit = 1;
      group = zeros(obj.N,1);
      group_sizes = sum(Grouping>0, 2);
      sel = Grouping(group_sizes<=gr_size_limit,:);
      group( sel(sel>0) ) = 1; % master group (index 1)
      % it should be checked whether master group is continous graph!
      sel = Grouping(group_sizes>gr_size_limit,:);
      for i=1:size(sel,1), % other groups
        group( sel(i,sel(i,:)>0) ) = i+1;
      end
      
      if min(group)>1,
        group = group - min(group) + 1;
      end
      
      
      % -------------------
      
      obj.grouping = group';   

    end %eps
    

    % =====================================================================
    function I = incidence_matrix(obj)
      %% Compute Tanks Incidence Matrix
      % Incidence matrix I determines if there is direct connection between
      % tanks i and j by "1" on position (i,j).
      %
      
      Luy = zeros(size(obj.Luy));
      
      M = obj.M;
      n = obj.N;
      
      for i = 1:n,            % submodels I/O dimensions
        nu(i) = size(M{i}.B,2);
        ny(i) = size(M{i}.C,1);
      end
      
      valves_pumps = sum(obj.Luw,1)==2;
      X = obj.Luw(:,valves_pumps);
      orig_ind = find(valves_pumps);
      
      clear C
      for i=1:size(X,2),
        Xcell = mat2cell( X(:,i) , nu , 1);
        for j=1:size(Xcell,1),
          C(j) = any(any(Xcell{j,1}));
        end
        ind = find(C);
        if length(ind)~=2, error('???'); end        
                
        sig_ind = find( strcmp({obj.signal.name},obj.Wnames{orig_ind(i)}) ,1);
        if isempty( obj.signal(sig_ind).data )
          tot_flow = 0;          
          disp([ 'No DATA for input signal ' obj.signal(sig_ind).name ])
        else
          tot_flow = sum( abs(obj.signal(sig_ind).data) );
        end
        
        tot_flow = max(1e-3,tot_flow);          % minimum flow adjustment
        Luy(:,ind(1)) = Luy(:,ind(1)) + tot_flow*X(:,i);
        Luy(:,ind(2)) = Luy(:,ind(2)) + tot_flow*X(:,i);
      end
      
      % ----
      
      Lcell = mat2cell(Luy,nu,ny);
      I = zeros(n);
      for i = 1:size(Lcell,1),
        for j = 1:size(Lcell,2),
          I(i,j) = sum(sum(Lcell{i,j}));
          %norm(Lcell{i,j},inf)
          %svd(Lcell{i,j})
        end
      end
      
      I = I+I';
      
    end %interactions    
    
    
  end %methods ============================================================
  
    
  methods (Access=protected) % ============================================
  end %protected methods ==================================================    
  
  
end %WNmodel


%% Plot Water Network Model
% PLOT(WNmodel) plots structure of water network. It is useful to call PLOT 
% command as
%
%   MOD = PLOT(MOD)
%
% as plot automatically computes graph vertex position and stores it for
% later use. The computation can be time consuming for larger models (>50
% vertexes).
%