function [meanG,varG] = ARXfreqUncertainty(theta,P,nb,na,nk,w,Ts)
% =========================================================================
% [meanG,varG] = ARXfreqUncertainty(theta,P,nb,na,nk,w,Ts)
%
% For given ARX model with parameters described by mean value THETA and
% covariance matrix P computes mean value and covariance of magnitude in
% frequency points specified by W.
%
% Computation is based on second order Taylor approximation of G(z,theta)
% expanded in theta.
%
% Note that mean(G(z,theta)) is not equal to G(z,mean(theta)) when using
% second order Taylor approximation.
%
% Paramters format:
% -----------------
% theta = [ b(nk) ... b(nb) , a(1) ... a(na) ]
%
%          b(nk)*d^nk + ... + b(nb)*d^nb 
% G(d) = ---------------------------------
%          1 + a(1)*d + ... + a(na)*d^na 
%
% w ... [rad/s]
%
% Author: Pavel Trnka, 2009
% =========================================================================


% -------------------------------------------------------------------------
% Parameters check
% -------------------------------------------------------------------------
n = length(theta);  % number of parameters

if size(theta,2)~=1 || size(P,1)~=n || size(P,2)~=n || nb-nk+1+na~=n,
  error('Parameters dimension error.');
end


% -------------------------------------------------------------------------
% 2nd order Taylor series approximation of TF
% -------------------------------------------------------------------------
b = theta(1:nb-nk+1)';
a = [1 theta(nb-nk+2:end)'];

clear j
for iii = 1:length(w),
  % Numerator and denumerator z powers -----------
  z = exp(1i*w(iii)*Ts);
  for i=1:nb-nk+1,
    znum(i) = z^(na-nk-i+1);
  end
  for i=1:na+1,
    zden(i) = z^(na-i+1);
  end  
  
  % Gradient g -----------------------------------
  ind = 1;
  for i=1:nb-nk+1,
    g(ind,1) = znum(i) / sum(zden.*a);
    ind = ind + 1;
  end
  for i=1:na,
    g(ind,1) = - sum(znum.*b) / (sum(zden.*a)^2) * zden(i+1);
    ind = ind + 1;    
  end
  
  % Hessian H ------------------------------------
  % <<<<<<<<<<< Needs to be CHECKED >>>>>>>>>>>>>>
  indi = 1;
  for ii=1:nb-nk+1,
    
    indj = 1;
    for jj=1:nb-nk+1,
      H(indi,indj) = 0;
      indj = indj + 1;
    end    
    for jj=1:na,
      H(indi,indj) = - znum(ii) / sum(zden.*a)^2 * zden(jj+1);      
      indj = indj + 1;    
    end        
    
    indi = indi + 1;
  end
  
  for ii=1:na,
    
    indj = 1;
    for jj=1:nb-nk+1,
      H(indi,indj) = - znum(jj) / (sum(zden.*a)^2) * zden(ii+1);
      indj = indj + 1;
    end
    
    for jj=1:na,
      H(indi,indj) = 2* sum(znum.*b) / (sum(zden.*a)^3) * zden(ii+1) * zden(jj+1);
      indj = indj + 1;    
    end                
    
    indi = indi + 1;    
  end  
    
  
  % Mean value and Variance -----------------------------------------------
  % <<<<<<<<<<< 1st ORDER APPROXIMATION >>>>>>>>>>>>>>>>>
%  meanG(iii) = abs( sum(znum.*b) / sum(zden.*a)  );
%  varG(iii)  = abs( g'*P*g  );
  % <<<<<<<<<<< 2nd ORDER APPROXIMATION >>>>>>>>>>>>>>>>>  
  meanG(iii) = abs( sum(znum.*b) / sum(zden.*a)  +  1/2*trace(H*P) );
  varG(iii)  = abs( g'*P*g  +  1/2*trace(H*P*H*P) );
  
end

