function [x,err] = BSEsolver(Q,k,x0)
%--------------------------------------------------------------------------
% [x,err] = BSEsolver(Q,k)
% [x,err] = BSEsolver(Q,k,x0)
%-------------------------------------
% Bilinear Set of Equations Solver - more correctly set of quadratic matrix
% functions.
%-------------------------------------
% Numerical solver based on non-linear least squares solving following set
% of equations:
%
%  x' * Q(:,:,i) * x = k(i)    for i=1,..,neq
%
% The number of equations neq can be arbitrary.
% Input:
%   Q    ... 3D matrix (n,n,neq)
%   k    ... vector (n,1)
% Output:
%   x    ... solution
%   err  ... quadratic error of solution 
%   [x0] ... optional initial solution
%
% Author: Pavel Trnka, 2009
%--------------------------------------------------------------------------

% Default parameters ---------
maxiter = 10; tol     = 1e-10;
% ----------------------------

%maxiter = 20; tol     = 1e-30;

%--------------------------------------------------------------------------
% Parameters Check
%--------------------------------------------------------------------------
n   = size(Q,1);   % variable dimension
neq = size(Q,3); % number of equations

if size(k,2)~=1, k=k';end

if size(Q,2)~=n || size(k,1)~=neq || size(k,2)~=1,
  error('Input parameters dimensions error.');
end


%--------------------------------------------------------------------------
% Solver Parameterizaiton
%--------------------------------------------------------------------------
opt = optimset('lsqnonlin');
opt.MaxIter     = 1e1*opt.MaxIter;
opt.TolFun      = 1e-5*opt.TolFun; %important param
opt.TolX        = 1e-2*opt.TolX;

opt.MaxFunEvals = 2e3;       % may be sifnificantly less then default value
opt.Jacobian    = 'on';
opt.Display     = 'off';  % off / on / iter
opt.LargeScale  = 'off';  % Questionable....
opt.Algorithm   = 'levenberg-marquardt';


%--------------------------------------------------------------------------
% Iterations for random initial solutions
%--------------------------------------------------------------------------
min_err = inf;
best_x = [];
for iter = 1:maxiter,
  
  % Initial value -----------------------
  if nargin==2,
    xx0 = randn(n,1);  
  elseif iter==1,
    xx0 = x0;    
  else
    xx0 = x0 + norm(x0)*randn(n,1);
  end
    
  % Non-linear Least Squares ------------
  x = lsqnonlin(@myfun,xx0,[],[],opt);

  err=0;
  for i=1:neq,
    err = err + (k(i) - x' * Q(:,:,i) * x)^2;
  end
  
  if err<min_err,
    min_err = err;
    best_x  = x;
  end

  %disp(['Current normalized iteration error = ' num2str(err/norm(k))]);
  %if err/norm(k)<tol, break; end
  disp(['Current iteration error = ' num2str(err)]);
  if err<tol, break;end
end

if iter==maxiter,
  disp('Maximum number of iterations exceeded.');
end

x   = best_x;
err = min_err;


%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------

  function [F,J] = myfun(x)
    for ii=1:neq,
      F(ii) = k(ii) - x' * Q(:,:,ii) * x;
    end
    if nargout > 1   % Jacobian of the function evaluated at x      
      for ii=1:neq,
        J(ii,:) = -(Q(:,:,ii)+Q(:,:,ii)') * x;
      end
    end
  end

%--------------------------------------------------------------------------
end
