function [Z,b,u,y] = FictiveZb(theta,P,nb,na,N)
%==========================================================================
% [Z,b,u,y] = FictiveZb(theta,P,nb,na,N)
% computes fictive regressor with ARX like structure, such that 
% fictive data completely represents given parameters vector \theta and
% covariance matrix P as it complies with
%   Z * \theta = b,
%   Z' * Z = inv(P).
% Number of regressor rows can be selected as N.
% Z has structure of two Hankel matrices with width nb and na.
%
% Inputs:
% theta ... parameters vector
% P ....... (normalized) parameters covariance
% nb ...... inputs window length
% na ...... outputs window length
% N ....... desired regressor rows
%
% Outputs:
% Z ....... fictive I/O data arranged to ARX like regressor
% b ....... RHS vector for Z
% u ....... fictive inputs
% y ....... fictive outputs
%
% Author: Pavel Trnka, 2009
%==========================================================================

% -------------------------------------------------------------------------
% Parameters check 
% -------------------------------------------------------------------------
n = na+nb;                             % length of parameters vector \theta

if size(theta,1)~=n || size(theta,2)~=1 || size(P,1)~=n || size(P,2)~=n,
  error('Parameters dimension error.');  
end

% Minimum number of DoFs to completely represent model (theta + chol fact.)
nparmin = n + n*(n+1)/2;                  
% Number of independent params in Z and b
npar = N+nb-1 + N+na-1 + 1; 

if npar<nparmin,  
  if exist('cprintf','file'),
    cprintf([0.7 0 0],'Warning: Selected regressor size is insufficient for given model.\n');
  else
    warning('Selected regressor size is insufficient for given model.'); %#ok<WNTAG>
  end  
end

F = pinv(P); % precision matrix


% -------------------------------------------------------------------------
% Structure representing matrices
% -------------------------------
% Matrices with repeated entries can be represented by vectorization
%   vec(Z) = Nz * z,
% where z is vector with unique parameterization. Similarly
%   vac(b) = Nb * b.
% -------------------------------------------------------------------------
[Nz,Nb] = CreateNzNb(nb,na,N);


% -------------------------------------------------------------------------
% Compute subspace of correct solutions to structured Z*theta=b
% -------------------------------------------------------------------------
M = kron(theta',eye(N)) * Nz - Nb;

% z must lie in the null space of M
[U,S,V] = svd(M);
% z can be any linear combination of B columns
B = V(:,rank(M)+1:end); 
%B = V(:,N+1:end); 


% -------------------------------------------------------------------------
% Find linear combination of B columns to meet required covariance
% Z'*Z=F   ->   r'*B'*Ni'*Nj*B*r = Fij
% -------------------------------------------------------------------------

% Condition on precision matrix
cit = 1;
for i=1:size(F,1),
  for j=i:size(F,2),
    Q(:,:,cit) = B' * Nz(1+N*(i-1):N*i,:)' * Nz(1+N*(j-1):N*j,:)* B;
    k(cit,1) = F(i,j);
    cit = cit + 1;
  end
end

% Condition on residual variance 
% Not working as B*r is always exact solution of Z*theta=b !
% M = Nb*B - kron(theta',eye(N))*Nz*B;
% Q(:,:,cit) = (1/N) * M'*M;
% k(cit,1)   = var_e0;


% -------------------------------------------------------------------------
% Estimating initial value for numerical solver
% -------------------------------------------------------------------------
% [U,S,V] = svd(F);
% Z0 = diag(sqrt(diag(S))) * U';
% %b0 = diag(1./sqrt(diag(S))) * U' * F0 * theta;
% x0 = (kron(theta',eye(N)) * Nz) \ reshape(Z0,[],1);


% -------------------------------------------------------------------------
% Numerical solution of a set of bilinear equations of type r'*Qi*r=ki.
% The solver BSEsolver starts with several different random initial 
% solutions -> there is non-zero probability that all initial solutions
% would lead to defective solutions.
% -------------------------------------------------------------------------
[r,err] = BSEsolver(Q,k);
z = B*r;
%disp(['Precission matrix quadratic error = ' num2str(err)]);
disp(['Precission matrix quadratic error (normalized)= ' num2str(err/norm(k))]);
if err>mean(mean(F)),
  disp(' **********  Warning: Fictive data quality may be low.  **********');
end

Z = reshape(Nz*z,N,[]);
b = Nb*z;
u = z(1:N+nb-1);
y = z(N+nb:end);
