%% Demonstration of Large Scale Model Class |LSmodel|
% This script demonstrates the use of |LSmodel| class for modeling and 
% model management of Large Scale (LS) systems.
% The first part shows definition of |LSmodel| for multiple boilers
% connected to a single header. The second part is demonstration of
% |LSmodel| methods on randomly generated model with 15 submodels.
% 


%% Submodels Definition 
% This section defines models of individual boilers and header. These
% models will be used to create LS model in the next section. The structure
% of the model is in following figure.
%
% <<./figs/boilers_header>>
%

% --- Boiler parameters ------------------------------
n_boilers = 5;               % number of boilers (1-5)
T = [50 100 150 200 250];    % fuel -> steam time constants [s]
Ks = 10;                     % amount of steam [t/hrs] from unit of 
                             % fuel [t/hrs]
V = [300 500 700 800 600];   % boiler volumes [m3] (1/V ~ pressure 
                             % integration constant)
K = [100 130 150 130 140];   % boiler->header pipe "conductivity", 
                             % =flow/(pressure difference) [t/hrs/bar]
Kh = 1;                      % "conductivity" to turbine 
Vh = 2e3;                    % header volume [m3]

% --- Boiler models ----------------------------------
% Inputs:
%   FF ... fuel flow - same value flows to all boilers [t/hrs]
%   ph ... header pressure [MPa]
% ----------------------------------------------------
% States:
%   GS ... generated steam [t/hrs]
%   pb ... boiler pressure [MPa]
% ----------------------------------------------------
% Outputs:
%   SF ... overall steam flow from all boilers [t/hrs]
%   pb ... boiler pressure [MPa]
% ----------------------------------------------------
for i=1:n_boilers,
  A = [ -1/T(i) , 0 
        1/V(i)  , -K(i)/V(i) ]; 
  B = [ Ks/T(i) , 0
        0       , K(i)/V(i) ];
  C = [ 0 , K(i)
        0 , 1    ];  
  D = [ 0 , -K(i)
        0 , 0    ];  
  M{i} = ss(A,B,C,D);           % models of individual boilers    
  M{i}.InputName  = { 'FF' , 'ph' };
  M{i}.StateName  = { ['GS' num2str(i)] , ['pb' num2str(i)] };
  M{i}.OutputName = { ['SF' num2str(i)] , ['pb' num2str(i)] };
  %M{i}.Name = ['B' num2str(i) ];
  M{i}.Name = ['Boiler ' num2str(i) ];
end


% --- Header model -----------------------------------
% Inputs:
%   SF ... total steam flow from boilers [t/hrs]
%   SD ... steam demand from (from turbine) [t/hrs]
% ----------------------------------------------------
% Outputs:
%   ph ... header pressure [MPa]
% ----------------------------------------------------
A = -Kh/Vh;                 
B = [1/Vh -1/Vh];        
C = 1;
D = 0;         
M{n_boilers+1} = ss(A,B,C,D); 
M{n_boilers+1}.InputName  = { 'SF' , 'SD' };
M{n_boilers+1}.StateName  = { 'ph' };
M{n_boilers+1}.OutputName = { 'ph' };
%M{n_boilers+1}.Name       = 'H';
M{n_boilers+1}.Name       = 'Header';

% --- Steam flow summing block -----------------------
% sums steam flows from boilers to header
sum1 = sumblk('SF','SF1','SF2','SF3','SF4','SF5');
%sum1.Name = 'SFsum';
sum1.Name = 'Steam Flow Summator';



%%  Construction of LSmodel 

%%
% LS model is contructed by specifying submodels, summators and a list of
% external inputs and outputs. Internal connections are automatically
% created by matching input/output names
mod = LSmodel(M,sum1,{'FF','SD'},{'ph','SF'})

%%
% Constructor function is compatible with standard CONNECT function, where 
% the same model would be defined as (connect does not accept cell array
% of models)
mod_connect = connect(M{1},M{2},M{3},M{4},M{5},M{6},sum1, ...
                      {'FF','SD'}, ...
                      {'ph','SF'});

%%
% It is also possible to construct the model by using numeric indexing of
% inputs and outputs. However, name based signal referencing will be
% prefered as numeric indexing may be confusing for LS systems.
inputs = { [1 3 5] , 8 };   % extended notation of standard connect
Q = [ 2 7 0 0
      4 7 0 0 
      6 7 0 0
      7 1 3 5 ];
outputs = { 7 , [1 3 5] };

inputs = { [1 3 5 7 9], 12 };
Q = [2  11 0  0  0  0
     4  11 0  0  0  0
     6  11 0  0  0  0
     8  11 0  0  0  0
     10 11 0  0  0  0
     11 1  3  5  7  9 ];
outputs = { 11 , [1 3 5 7 9] };   

mod2 = LSmodel(M{1},M{2},M{3},M{4},M{5},M{6},Q,inputs,outputs);



%% Plot model structure
% Plots structure of large scale model as a graph, where vertexes are 
% subsystems and edges indicate interaction between subsystems.
figure(10); clf; subfigure(10,3,3,[1 5]); % publish
subplot(121);
plot(mod);
title('\bfModel Structure');
subplot(122);
plot(mod,'io');
title('\bfModel Structure with I/O');




%% Random LS model with 15 sub-models
% Previous model is rather small. Larger model will be randomly generated
% by using command |RLS| parameterized by number of sub-models, maximum
% sub-model order, number of external outputs and number of external
% inputs.
seed=963;rand('seed',seed);randn('seed',seed); % to get stable CL model
mod = rls(15,3,3,3)
clf;
plot(mod,'io');
title('\bfModel Structure');



%% Structure preserving model order reduction
% Standard model order reduction destroys model internal structure.
% Structure preserving model order reduction reduces order of individual
% sub-models to achieve minimum mismatch with original model from external
% input/output point of view. Structured singular numbers are plotted to
% show which sub-models are reduced in order to achieve target global model 
% order.
figure(2);clf;
mod_r20=mod.struct_red(20);       % model reduced to 20th order
mod_r3=mod.struct_red(3);         % model reduced to 3rd order 
% Compare step responses of original and reduced order models
figure(1); 
clf;step(mod,30);hold all; 
step(mod_r20,'r--',30);
step(mod_r3,'g--',30);
legend(['Original n=' num2str(mod.n)],...
       ['Reduced n=' num2str(mod_r20.n)],...
       ['Reduced n=' num2str(mod_r3.n)]);


%%  "On-line" Model changes 


%%
% *Removing submodel(s)*
%
% Remove subsystems, which have zero order after structured reduction (just
% for demonstration of large scale model manipulations).
ind = find(mod_r20.orders==0);
removed_models = mod.M(ind);
modX = mod.rem_mod(ind);
figure(10); clf; 
subplot(121);plot(mod,'io');   
title('\bfOriginal Model');
subplot(122);plot(modX,'io');  
title('\bfReduced Model without zero order subsystems');


%% 
% *Adding submodel(s)*
modX = modX.add_mod( removed_models );
cla; plot(modX,'io');
title('\bfRestored Model (different numbering)');


%%
% *Removing external input*
modX = modX.rem_ext_inp('s24')
cla; plot(modX,'io'); 
title('\bfExternal Input Removed');

%%
% *Adding external input*
modX = modX.add_ext_inp('s24')
cla; plot(modX,'io'); 
title('\bfExternal Input Added');

%%
% *Removing external output*
modX = modX.rem_ext_out('s8')
cla; plot(modX,'io');
title('\bfExternal Output Removed');

%%
% *Adding external output*
modX = modX.add_ext_out('s8')
cla; plot(modX,'io');
title('\bfExternal Output Added');

%%
% *Extraction of model part* - eliminates submodels which are not
% controlable and observable for selected inputs and outputs.
ex_mod = mod.select('s24','s7');
clf;step(ex_mod);
title('\bfStep Response for Selected Model Part');



%% Submodels grouping
% Each sub-model can be assigned to a group. Each group has its own number
% and sub-models are assigned to given groups in property GROUPING.
% Following command combines sub-models to groups according to their order.
mod.grouping = mod.orders;
clf; plot(mod,'io');



%% Time domain models merging
% Merging of multiple models describing the same system. The merging is
% done in time domain and allows to combine information from uncertain
% ARX models with different orders (|idarx| or |idpoly|).

% Load 3 models of the same system identified from different data
% sets. The models have different structure.
[id_mod,sys] = demo_models; 

for i=1:3,
  LS{i} = LSmodel(id_mod{i},'u1','y1');
end

LS12 = LS{1}.merge('M1',id_mod{2},4,4);
LS123 = LS12.merge('M1',id_mod{3},4,4);


% --- Plot the result ---------------------------
sc = 3; % variance scale
figure(10);clf;
w = logspace(-1,log10(pi/id_mod{1}.Ts),300);
X = [w w(end:-1:1)]; 
axx = [w(1) w(end) 0 2.5];

for i=1:3,
  [meanG{i}  ,varG{i}  ] = LS{i}.freq_uncert('M1',w);
end
[meanG123,varG123] = LS123.freq_uncert('M1',w);

line_c = { 'b','g','m' };
fill_c = { [.8 .8 1],[.8 1 .8],[1 .8 .8]};
for i=1:3,
  subplot(2,2,i);
  h(i) = semilogx(w,meanG{i},line_c{i},'LineWidth',3); hold on;
  Y = meanG{i}+sc*sqrt(varG{i}); 
  Y = [meanG{i}-sc*sqrt(varG{i}) Y(end:-1:1)];
  hf(i) = fill(X,Y,fill_c{i},'EdgeAlpha',0,...
               'FaceAlpha',0.5,'LineStyle','none');
  semilogx(w,[meanG{i}-sc*sqrt(varG{i}) ; ...
              meanG{i}+sc*sqrt(varG{i})]',...
              [line_c{i} '--'],'LineWidth',1);
  switch i,
    case 1, title('\bf Model 1 (1st resonance peak)');
    case 2, title('\bf Model 2 (2nd resonance peak)');
    case 3, title('\bf Model 3 (low frequencies)');
  end
end

subplot(2,2,4);
h(4) = semilogx(w,meanG123,'r','LineWidth',3); hold on;
Y = meanG123+sc*sqrt(varG123);
Y = [meanG123-sc*sqrt(varG123) Y(end:-1:1)];
hf(4) = fill(X,Y,[1 .8 .8],'EdgeAlpha',0,...
             'FaceAlpha',0.5,'LineStyle','none');
semilogx(w,[meanG123-sc*sqrt(varG123) ; ...
            meanG123+sc*sqrt(varG123)]','r--','LineWidth',1);
title('\bf Merged Models 1, 2, 3');

[mag,phase] = bode(sys,w);
for i=1:4,
  subplot(2,2,i);
  horig(i) = semilogx(w,squeeze(mag),'k--','LineWidth',3);
  axis(axx); grid on; 
  xlabel('Frequency (rad/sec)'); ylabel('Magnitude');
  legend([h(i) hf(i) horig(i)] , ...
         'mean',['+/- ',num2str(sc),'\sigma'],'orig.');
end





%%  Overloaded functions 
% Many standard functions are overloaded for LSmodel class.

%% 
dcgain(mod)

%%
pole(mod)

%%
zero(mod)


%% Impulse response
figure(10);clf;
impulse(mod);

%% Step response
step(mod);

%% Bode frequency response
bode(mod);

%% Nyquist frequency response
nyquist(mod);

%% Pole-zero map
pzmap(mod);

%% Pole-zero map for I/O pairs
iopzmap(mod);



