function [mod,u1,y1,u2,y2] = MergeARXs(mod1,mod2,nb,na)
%==========================================================================
% [mod] = MergeARXs(mod1,mod2,nb,na)
% ARX models merging in TIME domain.
% Merges two ARX model of arbitrary structure by converting them to time
% domain i.e. by converting the models to time sequences giving same
% estimates of theta and Z in the LS sense.
% Merged model is obtained by reorganizing fictive input/output data into
% correct regressor and erforming weighted LS.
%
% Inputs ----------------------------------
% mod1 ... model 1
% mod2 ... model 2
% nb   ... required inputs window lengths
% na   ... required ouptputs window lengths
%
% Outputs ---------------------------------
% mod  ... merged model

% MOD structure elements ------------------
% mod.theta
% mod.Z .... normalized covariance
% mod.s .... -> real covariance s*Z
% mod.nb
% mod.na
%
% Author: Pavel Trnka, 2009
%==========================================================================

n = na+nb;                             % length of parameters vector \theta
nparmin = n + n*(n+1)/2;                           % minimum number of DoFs 
Nfict = 2*nparmin;                                              % heuristic

Nfict = 50;                              % number of fictive regressor rows


%--------------------------------------------------------------------------
% Generate fictive data from both models
%--------------------------------------------------------------------------
[Z1,b1,u1,y1] = FictiveZb(mod1.theta, mod1.P, mod1.nb, mod1.na, Nfict);
[Z2,b2,u2,y2] = FictiveZb(mod2.theta, mod2.P, mod2.nb, mod2.na, Nfict);

% [Z1,b1,u1,y1] = FictiveZb2(mod1.theta, mod1.P, mod1.nb, mod1.na, Nfict);
% [Z2,b2,u2,y2] = FictiveZb2(mod2.theta, mod2.P, mod2.nb, mod2.na, Nfict);


%--------------------------------------------------------------------------
% Assemble new regression with requested nb and na
%--------------------------------------------------------------------------

% maximum allowable size of joint regressor
N = min( [ length(u1)-(nb-1) ...
           length(u2)-(nb-1) ...
           length(y1)-na     ...
           length(y2)-na     ]);

c  = u1(1:N);
r  = u1(N:N+nb-1);
U1 = hankel(c,r);

c = u2(1:N);
r = u2(N:N+nb-1);
U2 = hankel(c,r);

c  = y1(1:N);
r  = y1(N:N+na-1);
Y1 = hankel(c,r);
b1 = y1(1+na:N+na);

c  = y2(1:N);
r  = y2(N:N+na-1);
Y2 = hankel(c,r);
b2 = y2(1+na:N+na);


%--------------------------------------------------------------------------
% Merge regressors and find new model parameters
%--------------------------------------------------------------------------
Z = [ U1 , -Y1 ; U2 , -Y2 ];
b = [ b1 ; b2 ];

% Weights according to variance inversion
s = [mod1.s mod2.s];
w = 1./s;
%w = w./sum(w);
w = w./max(w);
W = blkdiag( w(1)*eye(size(U1,1)) , w(2)*eye(size(U2,1)) );

% mod.theta = Z \ b;
% mod.P     = pinv( Z'*Z );

%mod.theta = (W*Z) \ (W*b);         % Weighted LS
mod.theta = pinv(Z'*W*Z)*Z'*W*b;
mod.P     = pinv( Z'*W*Z );

%%mod.s     = inv( inv(s(1)) + inv(s(2)) );

%mod.s     = inv( w(1)*inv(s(1)) + w(2)*inv(s(2)) ); %previous version

% May be correct!!!
%e = b-Z*mod.theta;
%mod.s     = (1/N) * sum(e.^2);

% New computation of s^2 -------------
e = b-Z*mod.theta;
mod.s = 1/trace(W) * e'*W*e;

mod.nb    = nb;
mod.na    = na;

