%% Demonstration of Water Deistribution Network Model Class WNmodel
% This script demosntrates the use of WNmodel class for modeling and model
% management of Water Distribution Networks (WN).
% The demonstration is based on Barcelona water network.
%
% WNmodel modle is child class of general LSmodel class.
%


%% Load water distribution network model from file
% Water network is defined by two files [filename].net and [filename].mat,
% where '.net' defines network structure and '.net' defines variables
% limits, prices, etc.
%
% *Structure definition file* is a simple text file describing individual
% tanks and their interconnections by specifying signal names and 
% directions.
%
% The first line of each tank definition block specifies tank number and 
% its name:
%
%   Tank##,<tank name>
%
% or it can specify node with it number 
%
%   Node##
%
% Following lines are same for tanks and nodes:
%
%   d,<demand name>
%   s,<source name>
%   +,<outlet pump/valve name>,<destination tank name>
%   -,<inlet pump/valve name>,<source tank name>
%
% Tank/node definition blocks can be separated by empty line(s). Empty 
% lines and Matlab type comments are ignored.
%
% *Variable definition file* is standard Matlab MAT file with struct for
% every signal defining limits by fields 
%
%  .min  ... min value
%  .max  ... max value
%  .dmin ... min slope value
%  .dmax ... max slope value
%  .smin ... soft limit min value
%  .smax ... soft limit max value
%  .type ... signal type ('MV','MD','UD','MO','MV','X')
%
% *Example (part of definition file):*
%
%  Tank01,d450BEG
%  d,c450BEG
%  -,iBegues4,d369BEG
% 
%  Tank02,d369BEG
%  d,c369BEG
%  +,iBegues4,d450BEG
%  -,iBegues3,d255BEG
%   
%  Node1
%  s,AportA
%  d,c82PAL
%  +,vPalleja70,Node2
%  +,vPapiolATLL,d110PAP
%  +,iPapiol2AGBAR,d110PAP
%  +,vFontSanta,d54REL
% 
%  Node2
%  d,c70PAL
%  +,iPalleja4,d125PAL
%  -,vPalleja70,Node1
% 
%  <End of example>


mod = WNmodel('BCN_network')


%% Plot Water Network Structure
figure(1);clf;subfigure(1,1,1,1); %maximize figure
mod = plot(mod);


%% Network Decomposition for Control
% Decomposition of water network to a set of tanks for distributed control.
% EPS can be used to find given number of sets by applying graph algorithms 
% of leaf condensation and \epsilon decomposition.
%
% *Decomposition to 6 groups*
figure(2);clf;
subfigure(2,1,1,1);
mod6 = mod.eps(6);
mod6 = plot(mod6);
title('\bfDecomposition to 6 groups');


%%
% *Decomposition to 12 groups*
figure(3);clf;
subfigure(3,1,1,1);
mod12 = mod.eps(12);
mod12 = plot(mod12);
title('\bfDecomposition to 12 groups');


%% Networks separation
% LS model of any group can be obtained by command GROUP.
figure(4);clf;
subfigure(4,1,1,1);
for i=1:12,
  subplot(3,4,i);
  plot( mod12.group(i) );
  title(['\bfGroup ' num2str(i)]);
end


%% Compress model
% Model modifications using GROUPS, REM_MOD, SELECT, ... keeps unused
% inputs/outputs and submodels in LS model. SQUEEZE can be used to remove
% them.

mod12_gr9 = mod12.group(9);
mod12_gr9 = mod12_gr9.squeeze;

figure(5);clf;
subfigure(5,1,1,1);
step(mod12_gr9);
