 function [dA,perm,block,mask] = epsdec_groups(A,n_target,opt)
% [dA,perm,block,mask] = EPSDEC(A,EPSILON)
% Decomposes graph (system) into N_TARGET weakly coupled sub-graphs 
% (sub-systems).
%
% INPUTS:
% ---------
% A          - graph adjacent matrix (with edges weights)
% N_TARGET   - target number of groups
% OPT        - 'inv' removes strongest edges instead of weakest
%
% OUTPUTS:
% ----------
% dA         - decomposed graph adjacent matrix
% perm       - vector of vertex permutations
% block      - block membership number for each vertex
% mask       - 0/1 mask for blocks
%
% Author: Pavel Trnka, 2009
%

origA = A;
weights = sort(reshape(A,[],1));
weights = weights(weights>0);

inverted = 0;
if exist('opt','var'),
  if strcmpi(opt,'inv'),
    inverted = 1;
    weights = weights(end:-1:1);
  end
end

% Find graph components ---------------------------------------------------
% (uses Matlab BGL toolbox!)
if exist('components','file'),  
  for epsilon=weights',
    A = origA;
    
    if inverted,
      A = (A < epsilon) .* A;
    else
      A = (A > epsilon) .* A;
    end
    
    % Make all edges BIdirectional --------
    A = triu(A) + triu(A)';
    A = A.*(1-eye(size(A,1))) + eye(size(A,1)); % not necessary...
    
    [comp, sizes] = components(sparse(A));
      
    if length(sizes)>=n_target,
      break;
    end    
  end
  
  if length(sizes)~=n_target,
    disp(['Warning: eps-decomposition returned ' num2str(length(sizes)) ' groups (' num2str(n_target) ' groups cannot be achieved)']);
  end  
  
else
  error('Matlab BGL toolbox must be instaled for epsilon decomposition.');
end

% Find vertexes permutation -----------------------------------------------
% vertex are renumbered according to components membership
[block,perm] = sort(comp);
dA = origA(perm,perm);

% Create blocks mask ------------------------------------------------------
mask = [];
for i=1:length(sizes),
  mask = blkdiag(mask,ones(sizes(i)));
end
