function subfigure(varargin)
% Position figure within a grid-based layout. It's like subplot for figures.
%
% subfigure(h,m,n,p), or subfigure(mnp), divides the screen into an m-by-n grid of
% tiles and creates a figure within the pth tile. Tiles are counted along the
% top row of the screen, then the second row, etc.
%
% h is figure handle.
%
% If p is a vector, the figure is sized to cover all the subfigure positions
% listed in p.
%
% Example 1: Four non-overlapping figures.
%
%     subfigure(h1,2,2,1)
%     subfigure(h2,2,2,2)
%     subfigure(h3,2,2,3)
%     subfigure(h4,2,2,4)
%
% Example 2: Three non-overlapping figures of various sizes.
%
%     subfigure(h1,4,4,[1 13])
%     subfigure(h2,4,4,[2 4])
%     subfigure(h3,4,4,[6 16])
%
% Example 3: Show the grid for a 3 x 5 layout.
%
%     subfigure(3,5)

% Modification of script created by Steve Hoelzer


% Padding to allow room for figure borders and menus
hpad = 20-10;
vpad = 90-10;

% Process input arguments
switch nargin
    case 4
        h = varargin{1};
        m = varargin{2};
        n = varargin{3};
        p = varargin{4};
    otherwise
        error('Wrong number of arguments.')
end

% Error checking
if ~isscalar(m) || ~isscalar(n)
    error('Gird dimensions must be scalar values.')
end
if m < 0 || n < 0
    error('Grid dimensions must be greator than zero.')
end
if any(p > m*n)
    error('Position value exceeds grid size.')
end

% Calculate tile size and spacing
scrsz = get(0,'ScreenSize');
hstep = floor( (scrsz(3)-hpad) / n );
vstep = floor( (scrsz(4)-vpad) / m );
vsz = vstep - vpad;
hsz = hstep - hpad;

% Row and column positions of each subfigure in p
r = ceil(p/n);
c = rem(p,n);
c(c==0) = n; % Special case

% Position of each subfigure in p in pixels (left, right, bottom, top)
le = hpad + (c-1)*hstep;
ri = le + hsz;
bo = vpad + (m-r)*vstep;
to = bo + vsz;

% Position of a subfigure that covers all subfigures in p
le = min(le); % Leftmost left
ri = max(ri); % Rightmost right
bo = min(bo); % Lowest bottom
to = max(to); % Highest top

% Calculate figure position
pos = [le, bo, ri-le, to-bo]; % [left, bottom, width, height]

% Position figure
set(h,'Position',pos);

