function [GNBoutput] = GNB(GNBinput)
% -------------------------------------------------------------------------
% Author:  S.J.L.M. van Loon
% Version: X 
% Date:    03-05-2011
% Project: Stability Analysis of Networked and Quantized Control Systems: a
%          Switched Linear Systems Approach. Master project.
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%   * No quantization errors included
%     x{k+1} = (\sum_{l=1}^{L}\alpha^l * Ab + Bb * \Delta * Cb)x{k}
%
%   * Quantization errors included
%     x{k+1} = (\sum_{l=1}^{L}\alpha^l * Ab + Bb * \Delta * Cb)x{k} +
%              (\sum_{l=1}^{L}\alpha^l * Eb + Bb * \Delta * Gb)epsilon{k}
% -------------------------------------------------------------------------
% Most of the symbols that are used in this file are based on the symbols 
% of the following TAC paper: "Stability Analysis of Networked Control 
% Systems Using a Switched Linear System Approach", by Donkers et al. 2011
% -------------------------------------------------------------------------
% The input parameters are:
%  * Bb         : matrix containing transformation matrices (without the 
%                 errors)
%  * Cb         : transformation matrix
%  * JJ         : cell with real Jordan blocks
%  * gridpnt0   : initial set with grid points
%  * gridpntM   : maximum number of gridpoints
%  * epsilon_u  : user defined tightness of the overapproximation
%  * NrNodes    : number of nodes
%  * setG       : string that contains information about the case under
%                 investigation, i.e., 
%                   # constant delay case
%                   # constant transmission interval case
%                   # the case that both delays as transmission intervals 
%                     are time varying.
%  * const      : constant, either tau, or h or empty
%  * plotflag   : if plotflag == true, the partitioning of Theta into 
%                 trianges is shown in a plot. The triangle with the 
%                 largest error appears in blue, after which it is divided 
%                 into new triangles
% NOTE: this function tries to achieve the epsilon_u given the constraint 
%       on the maximum number of grid points that are allowed by the user,
%       and the maximum number of iterations, set as 2X the maximum number
%       of grid points (to prevent infinite interations)
% -------------------------------------------------------------------------
% The output parameters are:
%  * Bbd     : matrix Bb * delta
%  * gridpnt : final number of gridpoints
% -------------------------------------------------------------------------

%% Extract input data
Bb        = GNBinput.Bb;
Cb        = GNBinput.Cb;
JJ        = GNBinput.JJ;
gridpnt0  = GNBinput.gridpnt0;
gridpntM  = GNBinput.gridpntM;
epsilon_u = GNBinput.epsilon_u;
NrNodes   = GNBinput.NrNodes;
setG      = GNBinput.setG;
const     = GNBinput.const;
plotflag  = GNBinput.plotflag;

v=version;  % check matlab version, fmincon changed depending on version
if str2double(v(3:4))>=10    %10 = r2010b
    options1 = optimset('algorithm','sqp','display','off','TolFun',1e-10,'TolX',1e-12,'LargeScale','off');
    options2 = optimset('algorithm','sqp','display','off','TolFun',1e-6,'TolX',1e-8,'LargeScale','off');
elseif str2double(v(3:4))>=6  %6 = r2008a
    options1 = optimset('algorithm','active-set','display','off','TolFun',1e-10,'TolX',1e-12,'LargeScale','off');
    options2 = optimset('algorithm','active-set','display','off','TolFun',1e-6,'TolX',1e-8,'LargeScale','off');   
elseif str2double(v(3:4))>=4  %4 = r2007a
    options1 = optimset('display','off','TolFun',1e-10,'TolX',1e-12,'LargeScale','off');
    options2 = optimset('display','off','TolFun',1e-6,'TolX',1e-8,'LargeScale','off');
else
    error('Must use MATLAB 2007a or newer. Versions older than 2007a do not have ''fmincon'' ')
end    

% Determine case
switch lower(setG)
    case {'constht'}
        disp('  Total amount of gridpoints   = 1')
        disp('  The maximum value of epsilon = 0')
        
        K  = max(size(JJ));       % total number of Jordan blocks
        
        %-- Worst case approximation error is zero
        deltaM = cell(0);
        for ii = 1:K
            deltaM{ii}     = zeros(max(size(JJ{ii})));
            deltaM{ii+K}   = zeros(max(size(JJ{ii})));
            deltaM{ii+2*K} = zeros(max(size(JJ{ii})));
        end
        
        Bbd = Bb*blkdiag(deltaM{:});
        
        gridpnt = 0;
        
    case {'vart'}
        [Bbd,gridpnt] = ...
            GNB_t(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,const,plotflag,options1,options2);
    case {'varh'}
        [Bbd,gridpnt] = ...
            GNB_h(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,const,plotflag,options1,options2);    
    case {'varht'}
        [Bbd,gridpnt] = ...
            GNB_ht(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,plotflag,options1,options2);
end

% Combine output data
GNBoutput.Bbd     = Bbd;
GNBoutput.gridpnt = gridpnt;

end

% -------------------------------------------------------------------------
% Underlying functions
% -------------------------------------------------------------------------

%% ---------------- Case: varying delays -----------------------------------
function [Bbd,gridpnt,DELTA] = GNB_t(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,const,plotflag,options1,options2)
    % -------------------------------------------------------------------------
    % Overapproximation based on gridding & norm-bounding
    %   * constant transmission interval
    %   * variable transmission delays
    %   * x{k+1} = (\sum_{l=1}^{L}\alpha^l * Ab + Bb * \Delta * Cb)x{k}
    % -------------------------------------------------------------------------
    hC = const;

    K  = max(size(JJ));       % total number of Jordan blocks
    
    % Search for the maximum size of the Jordan Blocks
    Msize = zeros(K,1);
    for ii = 1:K
        Msize(ii,1) = max(size(JJ{ii}));
    end

    if max(Msize)== 1
        JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
    else
        JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
    end

    % Gridding and Norm Bounding Procedure
    conv        = 0;
    maxiter     = 2*gridpntM;            % maximum number of allowed iterations
    gridpnt_up  = gridpnt0;              
    iter        = 0;
    MAX_epsilon = [];

    if plotflag == true
        pic = figure;
        plot(gridpnt_up,1,'bo')
        set(gca,'YTick',[])
        xlabel('\tau \in [\tau_{min}, \tau_{max}]')
        hold on
    end

    while (~conv)

    iter = iter+1;
    gridpnt = gridpnt_up;

    if plotflag == true
        figure(pic)
        plot(gridpnt,1,'bo')
        set(gca,'YTick',[])
        hold on
    end  

    switch JordanB
            %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            %-- Real and disctinct eigenvalues
            case {'RD'} 
                if iter == 1
                    %-- Computation of worst case approximation error to create an initial grid
                    deltilde = cell(0); x = cell(0); alpha12 = cell(0); alpha12up = cell(0);
                    for m = 1:length(gridpnt)-1     % for all vertices            
                        for ii = 1:K                % for each Jordan block
                            if abs(JJ{ii}) > 1e-14
                            deltilde{ii} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*(hC-gridpnt(m,1))+alpha(2)*(hC-gridpnt(m+1,1)))) - ...
                                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*(hC-gridpnt(m,1))) - ...
                                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*(hC-gridpnt(m+1,1))));
                            else
                            deltilde{ii} = @(alpha) 0;
                            end
                        end
                        for ii = 1:length(deltilde)
                            %-- Parameters fmincon
                            X0 = [0.5 0.5];
                            Afc = [];     bfc = [];
                            Aeq = [1 1];  beq = 1;
                            lb = [0;0];   ub = [1;1];
                            nonlon = [];
                            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
                            DELTA(m,ii) = -deltilde{ii}(x{ii});
                            alpha12{m,ii}=x{ii};
                        end
                    end
                    else
                    %-- Computation of worst case approximation error for the two new
                    %   segments
                    for m = 1:2     % for two line segments
                        M = [rd rd+1 rd+2];
                        for ii = 1:K                % for each Jordan block
                            if abs(JJ{ii}) > 1e-14
                            deltilde{ii} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*(hC-gridpnt(M(m),1))+alpha(2)*(hC-gridpnt(M(m+1),1)))) - ...
                                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*(hC-gridpnt(M(m),1))) - ...
                                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*(hC-gridpnt(M(m+1),1))));
                            else
                            deltilde{ii} = @(alpha) 0;
                            end
                        end
                        for ii = 1:length(deltilde)
                            %-- Parameters fmincon
                            X0 = [0.5 0.5];
                            Afc = [];     bfc = [];
                            Aeq = [1 1];  beq = 1;
                            lb = [0;0];   ub = [1;1];
                            nonlon = [];
                            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
                            DELTAup(m,ii) = -deltilde{ii}(x{ii});
                            alpha12up{m,ii}=x{ii};
                        end
                    end % end for
                end % end if: first calculation or subsequent calculations

            %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            %-- Real and non-disctinct or complex eigenvalues
            case {'RnDC'}
                syms s
                if iter == 1
                    %-- Computation of worst case approximation error to create an
                    %   initial grid
                    for m = 1:length(gridpnt)-1     % for all vertices            
                        for ii = 1:K                % for each Jordan block
                            deltilde{ii} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(hC-gridpnt(m,1)),alpha(1)*(hC-gridpnt(m,1))+alpha(2)*(hC-gridpnt(m+1,1))))+...
                                                       double(alpha(2)*int(expm(JJ{ii}*s),s,(hC-gridpnt(m+1,1)),alpha(1)*(hC-gridpnt(m,1))+alpha(2)*(hC-gridpnt(m+1,1)))));
                        end
                        for ii = 1:length(deltilde)
                            %-- Parameters fmincon
                            X0 = [0.5 0.5];
                            Afc = [];     bfc = [];
                            Aeq = [1 1];  beq = 1;
                            lb = [0;0];   ub = [1;1];
                            nonlon = [];
                            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
                            DELTA(m,ii) = -deltilde{ii}(x{ii});
                            alpha12{m,ii}=x{ii};
                        end
                    end
                    else
                    %-- Computation of worst case approximation error for the two new
                    %   segments
                    for m = 1:2     % for two line segments
                        M = [rd rd+1 rd+2];
                        for ii = 1:K                % for each Jordan block
                            deltilde{ii} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(hC-gridpnt(M(m),1)),alpha(1)*(hC-gridpnt(M(m),1))+alpha(2)*(hC-gridpnt(M(m+1),1))))+...
                                                       double(alpha(2)*int(expm(JJ{ii}*s),s,(hC-gridpnt(M(m+1),1)),alpha(1)*(hC-gridpnt(M(m),1))+alpha(2)*(hC-gridpnt(M(m+1),1)))));
                        end
                        for ii = 1:length(deltilde)
                            %-- Parameters fmincon
                            X0 = [0.5 0.5];
                            Afc = [];     bfc = [];
                            Aeq = [1 1];  beq = 1;
                            lb = [0;0];   ub = [1;1];
                            nonlon = [];
                            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
                            DELTAup(m,ii) = -deltilde{ii}(x{ii});
                            alpha12up{m,ii}=x{ii};
                        end
                    end % end for
                end % end if: calculate error for two line segments    
    end % end switch

    % -- Place DELTAup and alpha12up at the right location
    if iter > 1
        ndel = size(DELTA,1);
        alpha12N = cell(0);
        
        if rd == 1 && rd == ndel            % after first calculation
            DELTA = [DELTAup];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12up{i,j};
                end
            end
        elseif rd == 1 && rd ~= ndel        % replace first row
            DELTA = [DELTAup
                     DELTA(rd+1:end,:)];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12up{i,j};
                end
            end
            for i = rd+1:ndel
                for j = 1:length(deltilde)
                    alpha12N{i+1,j} = alpha12{i,j};
                end
            end
        elseif rd == ndel && rd ~= 1        % replace last row
            DELTA = [DELTA(1:rd-1,:)
                     DELTAup];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{rd-1+i,j} = alpha12up{i,j};
                end
            end
            for i = 1:rd-1
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12{i,j};
                end
            end
        else                                % replace intermediate row
            DELTA = [DELTA(1:rd-1,:)
                     DELTAup
                     DELTA(rd+1:end,:)];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{rd-1+i,j} = alpha12up{i,j};
                end
            end
            for i = 1:rd-1
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12{i,j};
                end
            end
            for i = rd+1:ndel
                for j = 1:length(deltilde)
                    alpha12N{i+1,j} = alpha12{i,j};
                end
            end
        end
        alpha12 = alpha12N;
    end    
        
    %-- Worst case approximation error of all GRID POINTS for each real Jordan block
    deltaMgrid = cell(0);
    for ii = 1:K
        for jj = 1:length(gridpnt)
            if jj == 1
                deltaMgrid{ii,jj} = DELTA(1,ii);
            elseif jj == length(gridpnt)
                deltaMgrid{ii,jj} = DELTA(end,ii);
            else
                deltaMgrid{ii,jj} = max(DELTA(jj-1,ii),DELTA(jj,ii));
            end
        end
    end

    
    % Worst case approximation error evaluated for all gridpoints for each real Jordan block
    deltaM = cell(0);
    for ii = 1:K
        deltaM{ii}     = 0*eye(max(size(JJ{ii})));     % no uncertainty on \delta^{A_h}
        deltaM{ii+K}   = 0*eye(max(size(JJ{ii})));     % no uncertainty on \delta^{E_h}
        deltaM{ii+2*K} = max(DELTA(:,ii))*eye(max(size(JJ{ii})));
    end

    % Determine Bbd
    Bbd = Bb*blkdiag(deltaM{:});
    deltaM{:};
    % Calculate epsilon
    epsilon = zeros(NrNodes,1);
    for ii = 1:NrNodes
        epsilon(ii,1) = norm(Bbd,2)*norm(Cb{ii},2);
    end

    % Store maximum value of epsilon
    MAX_eps = max(epsilon);
    MAX_epsilon = [MAX_epsilon;MAX_eps];

    NrGridpnt = max(size(gridpnt));

    % Determine if convergence is reached
    conv = (max(epsilon) < epsilon_u) | (iter == maxiter) | (NrGridpnt == gridpntM);

    if conv == 0    % No convergence, epsilon > epsilon_u
        % -------------------------
        % NEW GRIDPOINT CALCULATION

    [rd,cd] = find(DELTA==max(max(DELTA)));
    rd = rd(1);  % if nr of rd>1, take the first one
    cd = cd(1);  % if nr of cd>1, take the first one

    % search for matching alpha
    alphagrid = alpha12{rd,cd};

    new_node = alphagrid(1,1)*gridpnt(rd)+alphagrid(1,2)*gridpnt(rd+1);

    gridpnt_update = [gridpnt;new_node];
    gridpnt_up     = sort(gridpnt_update);
    else 
        break       % epsilon < epsilon_u  or maximum iterations reached
    end
    if plotflag == true
        figure(pic)
        plot(new_node,1,'rx')
        set(gca,'YTick',[])
        hold on
    end        
end

if (iter == maxiter)
    warning('Maximum number of iterations reached \n')
end
if (NrGridpnt == gridpntM)
    warning('Maximum number of grid points reached \n')
end

% Plot information on command window
N = max(size(gridpnt));
disp(['  Total amount of gridpoints   = ' num2str(N)])
epsilonM = max(epsilon);
fprintf('  The maximum value of epsilon = %1.3e \n',epsilonM)
end

%% ---------------- Case: varying transmission intervals -------------------
function [Bbd,gridpnt] = GNB_h(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,const,plotflag,options1,options2)
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%   * variable transmission intervals
%   * constant transmission delay, denoted by tauC
%
% Date: 24-04-2011
% -------------------------------------------------------------------------

tauC = const;

K  = max(size(JJ));       % total number of Jordan blocks
    
% Search for the maximum size of the Jordan Blocks
Msize = zeros(K,1);
for ii = 1:K
    Msize(ii,1) = max(size(JJ{ii}));
end

if max(Msize)== 1
    JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
else
    JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
end

%-- Gridding and Norm Bounding Procedure
conv        = 0;
maxiter     = 2*gridpntM;         % maximum number of allowed iterations
gridpnt_up  = gridpnt0;               
iter        = 0;

if plotflag == true
    pic = figure;
    plot(gridpnt_up,1,'bo')
    set(gca,'YTick',[])
    xlabel('h \in [h_{min}, h_{max}]')
    hold on
end


% Define empty cells
alpha12N = cell(0); deltilde = cell(0); x = cell(0); alpha12 = cell(0); 
alpha12up = cell(0);


while (~conv)

    iter = iter+1;
    gridpnt = gridpnt_up;

    if plotflag == true
        figure(pic)
        plot(gridpnt,1,'bo')
        set(gca,'YTick',[])
        hold on
    end

    %-- Computation of the approximation error
    %   we distinct:
    %       * real and distinct eigenvalues;
    %       * real and nondistinct or complex eigenvalues
    %         (longer computational time due to the matrix exponential)
    %   the approximation error and convex combination only changes at
    %   the location where a new grid point is added. Instead of
    %   calculating the DELTA and alpha12 for all line segments again,
    %   we only calculate them for the two new line segments in order
    %   to reduce the computational costs/time. 
    

    switch JordanB
    %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    %-- Real and disctinct eigenvalues
    case {'RD'} 

    if iter == 1
    
    %-- Computation of worst case approximation error to create an initial grid
    for m = 1:length(gridpnt)-1     % for all vertices            
        for ii = 1:K                % for each Jordan block
            if abs(JJ{ii}) > 1e-14
            deltilde{ii} = @(alpha) -abs(exp(JJ{ii}*(alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1))) - ...
                                        alpha(1)*exp(JJ{ii}*gridpnt(m,1)) - alpha(2)*exp(JJ{ii}*gridpnt(m+1,1)));

            deltilde{ii+K} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1))) - ...
                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*gridpnt(m,1)) - ...
                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*gridpnt(m+1,1)));

            deltilde{ii+2*K} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*(gridpnt(m,1)-tauC)+alpha(2)*(gridpnt(m+1,1)-tauC))) - ...
                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*(gridpnt(m,1)-tauC)) - ...
                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*(gridpnt(m+1,1)-tauC)));
            else
            deltilde{ii} = @(alpha) 0;
            deltilde{ii+K} = @(alpha) 0;
            deltilde{ii+2*K} = @(alpha) 0;
            end
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [0.5 0.5];
            Afc = [];     bfc = [];
            Aeq = [1 1];  beq = 1;
            lb = [0;0];   ub = [1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
            DELTA(m,ii) = -deltilde{ii}(x{ii});
            alpha12{m,ii}=x{ii};
        end
    end
    else
    %-- Computation of worst case approximation error for the two new
    %   segments
    for m = 1:2     % for two line segments
        M = [rd rd+1 rd+2];
        for ii = 1:K                % for each Jordan block
            if abs(JJ{ii}) > 1e-14
            deltilde{ii} = @(alpha) -abs(exp(JJ{ii}*(alpha(1)*gridpnt(M(m),1)+alpha(2)*gridpnt(M(m+1),1))) - ...
                                        alpha(1)*exp(JJ{ii}*gridpnt(M(m),1)) - alpha(2)*exp(JJ{ii}*gridpnt(M(m+1),1)));

            deltilde{ii+K} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*gridpnt(M(m),1)+alpha(2)*gridpnt(M(m+1),1))) - ...
                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*gridpnt(M(m),1)) - ...
                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*gridpnt(M(m+1),1)));

            deltilde{ii+2*K} = @(alpha) -abs((1/JJ{ii})*exp(JJ{ii}*(alpha(1)*(gridpnt(M(m),1)-tauC)+alpha(2)*(gridpnt(M(m+1),1)-tauC))) - ...
                                          (alpha(1)/JJ{ii})*exp(JJ{ii}*(gridpnt(M(m),1)-tauC)) - ...
                                          (alpha(2)/JJ{ii})*exp(JJ{ii}*(gridpnt(M(m+1),1)-tauC)));
            else
            deltilde{ii} = @(alpha) 0;
            deltilde{ii} = @(alpha) 0;
            deltilde{ii} = @(alpha) 0;
            end
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [0.5 0.5];
            Afc = [];     bfc = [];
            Aeq = [1 1];  beq = 1;
            lb = [0;0];   ub = [1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
            DELTAup(m,ii) = -deltilde{ii}(x{ii});
            alpha12up{m,ii}=x{ii};
        end
    end % end for
    end % end if: first calculation or subsequent calculations


    %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    %-- Real and non-disctinct or complex eigenvalues
    case {'RnDC'}
    syms s
    if iter == 1
    %-- Computation of worst case approximation error to create an
    %   initial grid
    for m = 1:length(gridpnt)-1     % for all vertices            
        for ii = 1:K                % for each Jordan block
            deltilde{ii} = @(alpha) -norm(double(expm(JJ{ii}*(alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1)))) - ...
                                     double(alpha(1)*expm(JJ{ii}*gridpnt(m,1))) - double(alpha(2)*expm(JJ{ii}*gridpnt(m+1,1))));

            deltilde{ii+K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,gridpnt(m,1),alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1)))+...
                                       double(alpha(2)*int(expm(JJ{ii}*s),s,gridpnt(m+1,1),alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1))));

            deltilde{ii+2*K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpnt(m,1)-tauC),alpha(1)*(gridpnt(m,1)-tauC)+alpha(2)*(gridpnt(m+1,1)-tauC)))+...
                                       double(alpha(2)*int(expm(JJ{ii}*s),s,(gridpnt(m+1,1)-tauC),alpha(1)*(gridpnt(m,1)-tauC)+alpha(2)*(gridpnt(m+1,1)-tauC))));
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [0.5 0.5];
            Afc = [];     bfc = [];
            Aeq = [1 1];  beq = 1;
            lb = [0;0];   ub = [1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
            DELTA(m,ii) = -deltilde{ii}(x{ii});
            alpha12{m,ii}=x{ii};
        end
    end
    else
    %-- Computation of worst case approximation error for the two new
    %   segments
    for m = 1:2     % for two line segments
        M = [rd rd+1 rd+2];
        for ii = 1:K                % for each Jordan block
            deltilde{ii} = @(alpha) -norm(double(expm(JJ{ii}*(alpha(1)*gridpnt(M(m),1)+alpha(2)*gridpnt(M(m+1),1)))) - ...
                                     double(alpha(1)*expm(JJ{ii}*gridpnt(M(m),1))) - double(alpha(2)*expm(JJ{ii}*gridpnt(M(m+1),1))));

            deltilde{ii+K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,gridpnt(m,1),alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1)))+...
                                       double(alpha(2)*int(expm(JJ{ii}*s),s,gridpnt(m+1,1),alpha(1)*gridpnt(m,1)+alpha(2)*gridpnt(m+1,1))));

            deltilde{ii+2*K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpnt(m,1)-tauC),alpha(1)*(gridpnt(m,1)-tauC)+alpha(2)*(gridpnt(m+1,1)-tauC)))+...
                                       double(alpha(2)*int(expm(JJ{ii}*s),s,(gridpnt(m+1,1)-tauC),alpha(1)*(gridpnt(m,1)-tauC)+alpha(2)*(gridpnt(m+1,1)-tauC))));
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [0.5 0.5];
            Afc = [];     bfc = [];
            Aeq = [1 1];  beq = 1;
            lb = [0;0];   ub = [1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
            DELTAup(m,ii) = -deltilde{ii}(x{ii});
            alpha12up{m,ii}=x{ii};
        end
    end % end for
    end % end if: calculate error for two line segments    
    end % end switch

    % -- Place DELTAup and alpha12up at the right location
    if iter > 1
        ndel = size(DELTA,1);
                
        if rd == 1 && rd == ndel            % after first calculation
            DELTA = [DELTAup];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12up{i,j};
                end
            end
        elseif rd == 1 && rd ~= ndel        % replace first row
            DELTA = [DELTAup
                     DELTA(rd+1:end,:)];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12up{i,j};
                end
            end
            for i = rd+1:ndel
                for j = 1:length(deltilde)
                    alpha12N{i+1,j} = alpha12{i,j};
                end
            end
        elseif rd == ndel && rd ~= 1        % replace last row
            DELTA = [DELTA(1:rd-1,:)
                     DELTAup];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{rd-1+i,j} = alpha12up{i,j};
                end
            end
            for i = 1:rd-1
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12{i,j};
                end
            end
        else                                % replace intermediate row
            DELTA = [DELTA(1:rd-1,:)
                     DELTAup
                     DELTA(rd+1:end,:)];
            for i = 1:2
                for j = 1:length(deltilde)
                    alpha12N{rd-1+i,j} = alpha12up{i,j};
                end
            end
            for i = 1:rd-1
                for j = 1:length(deltilde)
                    alpha12N{i,j} = alpha12{i,j};
                end
            end
            for i = rd+1:ndel
                for j = 1:length(deltilde)
                    alpha12N{i+1,j} = alpha12{i,j};
                end
            end
        end
        alpha12 = alpha12N;
    end

    %-- Worst case approximation error of all GRID POINTS for each real Jordan block
    deltaMgrid = cell(0);
    for ii = 1:K
        for jj = 1:length(gridpnt)
            if jj == 1
                deltaMgrid{ii,jj}     = DELTA(1,ii);
                deltaMgrid{ii+K,jj}   = DELTA(1,ii+K);
                deltaMgrid{ii+2*K,jj} = DELTA(1,ii+2*K);
            elseif jj == length(gridpnt)
                deltaMgrid{ii,jj}     = DELTA(end,ii);
                deltaMgrid{ii+K,jj}   = DELTA(end,ii+K);
                deltaMgrid{ii+2*K,jj} = DELTA(end,ii+2*K);
            else
                deltaMgrid{ii,jj}     = max(DELTA(jj-1,ii),DELTA(jj,ii));
                deltaMgrid{ii+K,jj}   = max(DELTA(jj-1,ii+K),DELTA(jj,ii+K));
                deltaMgrid{ii+2*K,jj} = max(DELTA(jj-1,ii+K),DELTA(jj,ii+2*K));
            end
        end
    end

    % Worst case approximation error of all LINE SEGMENTS for each real Jordan block
    deltaM = cell(0);
    for ii = 1:K
        deltaM{ii}     = max(DELTA(:,ii))*eye(max(size(JJ{ii})));
        deltaM{ii+K}   = max(DELTA(:,ii+K))*eye(max(size(JJ{ii})));
        deltaM{ii+2*K} = max(DELTA(:,ii+2*K))*eye(max(size(JJ{ii})));
    end

    % Determine Bbd
    Bbd = Bb*blkdiag(deltaM{:});

    % Calculate epsilon
    epsilon = zeros(NrNodes,1);
    for ii = 1:NrNodes
        epsilon(ii,1) = norm(Bbd)*norm(Cb{ii});
    end

    % Determine if convergence is reached
    NrGridpnt = max(size(gridpnt));
    conv = (max(epsilon) < epsilon_u) | (iter == maxiter) | (NrGridpnt == gridpntM);

    if conv == 0    % no convergence, epsilon > epsilon_u

    %% NEW GRIDPOINT CALCULATION

    [rd,cd] = find(DELTA==max(max(DELTA)));
    rd = rd(1);  % if nr of rd>1, take the first one
    cd = cd(1);  % if nr of cd>1, take the first one

    % search for matching alpha
    alphagrid = alpha12{rd,cd};

    new_node = alphagrid(1,1)*gridpnt(rd)+alphagrid(1,2)*gridpnt(rd+1);

    gridpnt_update = [gridpnt;new_node];
    gridpnt_up     = sort(gridpnt_update);

    else 
        break       % epsilon < epsilon_u  or maximum iterations reached
    end
    if plotflag == true
        figure(pic)
        plot(new_node,1,'rx')
        set(gca,'YTick',[])
        hold on
    end        
end % end while

if (iter == maxiter)
    warning('Maximum number of iterations reached \n')
end
if (NrGridpnt == gridpntM)
    warning('Maximum number of grid points reached \n')
end

% Display information
N = max(size(gridpnt));
disp(['  Total amount of gridpoints   = ' num2str(N)])
epsilonM = max(epsilon);
fprintf('  The maximum value of epsilonb = %1.3e \n',epsilonM)
 
end

%% ----------------Case: varying transmission intervals and delays ---------
function [Bbd,gridpnt] = GNB_ht(Bb,Cb,JJ,gridpnt0,gridpntM,epsilon_u,NrNodes,plotflag,options1,options2)
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%   * variable transmission intervals
%   * variable transmission delays
%   * x{k+1} = \sum_{l=1}^{L}\alpha^l * Ab + Bb * \Delta * Cb
% -------------------------------------------------------------------------
% Date: 24-04-2011
% -------------------------------------------------------------------------

%-- Divide Theta into triangles
TRI = delaunay(gridpnt0(:,1),gridpnt0(:,2));

if plotflag == true
    pic = figure;
    subplot(1,2,1)
    triplot(TRI,gridpnt0(:,1),gridpnt0(:,2),'bo-')
    xlabel('Transmission Interval h')
    ylabel('Transmission Delay \tau')
    hold on
end

K  = max(size(JJ));       % total number of Jordan blocks
    
% Search for the maximum size of the Jordan Blocks
Msize = zeros(K,1);
for ii = 1:K
    Msize(ii,1) = max(size(JJ{ii}));
end

if max(Msize)== 1
    JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
else
    JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
end
% -------------------------------------------------------------------------
conv        = 0;
iter        = 0;
maxiter     = 2*gridpntM;      % maximum number of allowed iterations 
TRI_up      = TRI;
NoNewTRI_up = 0;
gridpnt_up  = gridpnt0;
MAX_epsilon = [];
deltaAut    = [];
alphaAut    = [];
TRIAut      = [];
gridpntAut  = [];

% Define empty cells
deltilde = cell(0); alpha123 = cell(0); alpha123up = cell(0); x = cell(0);
alpha123N = cell(0);

while (~conv)

if maxiter <= 0
    error('minimum iterations must be larger than zero, increase "gridpntM" ')
    break
end

iter = iter+1;

TRI        = TRI_up;
NoNewT     = NoNewTRI_up;
TRIAut     = [TRIAut;{iter,TRI}];
gridpnt    = gridpnt_up;
gridpntAut = [gridpntAut;{iter,gridpnt}];

if plotflag == true
    figure(pic)
    subplot(1,2,1)
    triplot(TRI,gridpnt(:,1),gridpnt(:,2),'bo-')
    xlabel('Transmission Interval h')
    ylabel('Transmission Delay \tau')
    hold on
    pause(2)
end

%-- Total number of triangles := nTRI
nTRI = size(TRI,1);

switch JordanB
    %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    %-- Real and disctinct eigenvalues
    case {'RD'}
       
    if iter == 1
    % Computation of approximation error per triangle for each Jordan block to create an initial grid
    for m = 1:nTRI                            % for each triangle
        triangle = TRI(m,:);                  % isolate triangle
        gridpntTRI = [gridpnt(triangle(1),:)  % search for corresponding gridpoints
                      gridpnt(triangle(2),:)
                      gridpnt(triangle(3),:)];  
        for ii = 1:K                          % for each Jordan block
            if abs(JJ{ii}) > 1e-14
            deltilde{ii} =   @(alpha) -abs(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1) + alpha(2)*gridpntTRI(2,1) + alpha(3)*gridpntTRI(3,1)))...
                                         -alpha(1)*exp(JJ{ii}*gridpntTRI(1,1)) - alpha(2)*exp(JJ{ii}*gridpntTRI(2,1))...
                                         -alpha(3)*exp(JJ{ii}*gridpntTRI(3,1)));         

            deltilde{ii+K} = @(alpha) -abs((alpha(1)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(1,1)))+...
                                           (alpha(2)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(2,1)))+...
                                           (alpha(3)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(3,1))));

            deltilde{ii+2*K} = @(alpha) -abs((alpha(1)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(1,1)-gridpntTRI(1,2))))+...
                                             (alpha(2)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(2,1)-gridpntTRI(2,2))))+...
                                             (alpha(3)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(3,1)-gridpntTRI(3,2)))));

            else
            deltilde{ii}     = @(alpha) 0;
            deltilde{ii+K}   = @(alpha) 0;
            deltilde{ii+2*K} = @(alpha) 0;
            end
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [1/3 1/3 1/3];
            Afc = [];       bfc = [];
            Aeq = [1 1 1];  beq = 1;
            lb = [0;0;0];   ub = [1;1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
            DELTA(m,ii) = -deltilde{ii}(x{ii});
            alpha123{m,ii}=x{ii};
        end
    end % end for (initial routine)    
    else
    %-- Computation of worst case approximation error for the NEW triangles
    for m = 1:NoNewT                          % for each new triangle
        triangle = TRI((nTRI-NoNewT+m),:);    % isolate correct triangle
        gridpntTRI = [gridpnt(triangle(1),:)  % search for corresponding gridpoints
                      gridpnt(triangle(2),:)
                      gridpnt(triangle(3),:)];  
        for ii = 1:K                          % for each Jordan block
            if abs(JJ{ii}) > 1e-14
            deltilde{ii} =   @(alpha) -abs(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1) + alpha(2)*gridpntTRI(2,1) + alpha(3)*gridpntTRI(3,1)))...
                                         -alpha(1)*exp(JJ{ii}*gridpntTRI(1,1)) - alpha(2)*exp(JJ{ii}*gridpntTRI(2,1))...
                                         -alpha(3)*exp(JJ{ii}*gridpntTRI(3,1)));         

            deltilde{ii+K} = @(alpha) -abs((alpha(1)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(1,1)))+...
                                           (alpha(2)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(2,1)))+...
                                           (alpha(3)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))-exp(JJ{ii}*gridpntTRI(3,1))));

            deltilde{ii+2*K} = @(alpha) -abs((alpha(1)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(1,1)-gridpntTRI(1,2))))+...
                                             (alpha(2)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(2,1)-gridpntTRI(2,2))))+...
                                             (alpha(3)/JJ{ii})*(exp(JJ{ii}*(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))-exp(JJ{ii}*(gridpntTRI(3,1)-gridpntTRI(3,2)))));

            else
            deltilde{ii}     = @(alpha) 0;
            deltilde{ii+K}   = @(alpha) 0;
            deltilde{ii+2*K} = @(alpha) 0;
            end
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [1/3 1/3 1/3];
            Afc = [];       bfc = [];
            Aeq = [1 1 1];  beq = 1;
            lb = [0;0;0];   ub = [1;1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options1);
            DELTAup(m,ii) = -deltilde{ii}(x{ii});
            alpha123up{m,ii}=x{ii};
        end
    end % end for (routine new triangles)
    end % end if

    case {'RnDC'}
    if iter == 1
    % Computation of approximation error per triangle for each Jordan block to create an initial grid
    for m = 1:nTRI                            % for each triangle
        triangle = TRI(m,:);                  % isolate triangle
        gridpntTRI = [gridpnt(triangle(1),:)  % search for corresponding gridpoints
                      gridpnt(triangle(2),:)
                      gridpnt(triangle(3),:)];  
        for ii = 1:K                          % for each Jordan block
        deltilde{ii} = @(alpha) -norm(double(expm(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))) - ...
                                     double(alpha(1)*expm(JJ{ii}*gridpntTRI(1,1))) - double(alpha(2)*expm(JJ{ii}*gridpntTRI(2,1))) - ...
                                     double(alpha(3)*expm(JJ{ii}*gridpntTRI(3,1))));

        deltilde{ii+K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(1,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(2,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(3,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1)))));

        deltilde{ii+2*K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(1,1)-gridpntTRI(1,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2)))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(2,1)-gridpntTRI(2,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2)))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(3,1)-gridpntTRI(3,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))));    
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [1/3 1/3 1/3];
            Afc = [];       bfc = [];
            Aeq = [1 1 1];  beq = 1;
            lb = [0;0;0];   ub = [1;1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
            DELTA(m,ii) = -deltilde{ii}(x{ii});
            alpha123{m,ii}=x{ii};
        end
    end % end for (initial routine)    
    else
    %-- Computation of worst case approximation error for the NEW triangles
    for m = 1:NoNewT                          % for each new triangle
        triangle = TRI((nTRI-NoNewT+m),:);    % isolate correct triangle
        gridpntTRI = [gridpnt(triangle(1),:)  % search for corresponding gridpoints
                      gridpnt(triangle(2),:)
                      gridpnt(triangle(3),:)];  
        for ii = 1:K                          % for each Jordan block
        deltilde{ii} = @(alpha) -norm(double(expm(JJ{ii}*(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+alpha(3)*gridpntTRI(3,1)))) - ...
                                     double(alpha(1)*expm(JJ{ii}*gridpntTRI(1,1))) - double(alpha(2)*expm(JJ{ii}*gridpntTRI(2,1))) - ...
                                     double(alpha(3)*expm(JJ{ii}*gridpntTRI(3,1))));

        deltilde{ii+K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(1,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(2,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,gridpntTRI(3,1),(alpha(1)*gridpntTRI(1,1)+alpha(2)*gridpntTRI(2,1)+...
                                        alpha(3)*gridpntTRI(3,1)))));

        deltilde{ii+2*K} = @(alpha) -norm(double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(1,1)-gridpntTRI(1,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2)))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(2,1)-gridpntTRI(2,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2)))))+ ...
                                        double(alpha(1)*int(expm(JJ{ii}*s),s,(gridpntTRI(3,1)-gridpntTRI(3,2)),(alpha(1)*(gridpntTRI(1,1)-gridpntTRI(1,2))+alpha(2)*(gridpntTRI(2,1)-gridpntTRI(2,2))+...
                                        alpha(3)*(gridpntTRI(3,1)-gridpntTRI(3,2))))));     
        end
        for ii = 1:length(deltilde)
            %-- Parameters fmincon
            X0 = [1/3 1/3 1/3];
            Afc = [];       bfc = [];
            Aeq = [1 1 1];  beq = 1;
            lb = [0;0;0];   ub = [1;1;1];
            nonlon = [];
            x{ii} = fmincon(deltilde{ii},X0,Afc,bfc,Aeq,beq,lb,ub,nonlon,options2);
            DELTAup(m,ii) = -deltilde{ii}(x{ii});
            alpha123up{m,ii}=x{ii};
        end
    end % end for (routine new triangles)
    end % end iter   
end % end switch

% -- Place DELTAup and alpha12up at the right location
if iter > 1
   ndel = size(DELTA,1);
   
   % update DELTA
   DELTA(rd,:) = [];
   DELTA = [DELTA
            DELTAup];
   % update alpha123
   if rd == 1   % delete first row
       for i = 1:ndel-1
          for j = 1:length(deltilde)
              alpha123N{i,j} = alpha123{i+1,j};
          end
       end
       for i = 1:NoNewT
          for j = 1:length(deltilde)
              alpha123N{i+ndel-1,j} = alpha123up{i,j};
          end
       end 
   elseif rd == ndel % delete last row
       for i = 1:ndel-1
          for j = 1:length(deltilde)
              alpha123N{i,j} = alpha123{i,j};
          end
       end
       for i = 1:NoNewT
          for j = 1:length(deltilde)
              alpha123N{i+ndel-1,j} = alpha123up{i,j};
          end
       end 
   else
       for i = 1:rd-1
          for j = 1:length(deltilde)
              alpha123N{i,j} = alpha123{i,j};
          end
       end
       for i = rd:ndel-1
          for j = 1:length(deltilde)
              alpha123N{i,j} = alpha123{i+1,j};
          end
       end
       for i = 1:NoNewT
          for j = 1:length(deltilde)
              alpha123N{i+ndel-1,j} = alpha123up{i,j};
          end
       end 
   end
   alpha123 = alpha123N;    
   clear DELTAup alpha123up
end

deltaAut = [deltaAut;{iter,DELTA}];
alphaAut = [alphaAut;{iter,alpha123}];

%-- Worst case approximation error of ALL triangles for each real Jordan block
deltaM = cell(0);
for ii = 1:K
    deltaM{ii}     = max(DELTA(:,ii))*eye(max(size(JJ{ii})));
    deltaM{ii+K}   = max(DELTA(:,ii+K))*eye(max(size(JJ{ii})));
    deltaM{ii+2*K} = max(DELTA(:,ii+2*K))*eye(max(size(JJ{ii})));
end

%-- Determine Bbd
Bbd = Bb*blkdiag(deltaM{:});

%-- Calculate epsilon
epsilon = zeros(NrNodes,1);
for ii = 1:NrNodes
    epsilon(ii,1) = norm(Bbd,2)*norm(Cb{ii},2);
end
MAX_eps = max(epsilon);
MAX_epsilon = [MAX_epsilon;MAX_eps];

if plotflag == true
    figure(pic)
    subplot(1,2,2)
    semilogx(MAX_epsilon,'bo-')
    xlabel('Number of iterations')
    ylabel('Maximum value of \epsilon')
    pause(1)
end

%-- Determine conversion
NrGridpnt = max(size(gridpnt));
conv = (max(epsilon) < epsilon_u) | (iter == maxiter) | (NrGridpnt == gridpntM);

if conv == 0    % epsilon > epsilon_u

%% NEW GRIDPOINT CALCULATION

%-- Search for largest error
[rd,cd] = find(DELTA==max(max(DELTA)));
rd = rd(1);  % if nr of rd>1, take the first one
cd = cd(1);  % if nr of cd>1, take the first one

%-- Search for matching alpha
alphagrid = alpha123{rd,cd};
alphaZ    = find(alphagrid<=eps);
alphagrid(alphaZ) = 0;  % set to 'real' zero to prevent num. probl.

%- Search for matching triangle and gridpoints
triangleLE   = TRI(rd,:);                    % isolate triangle
gridpntTRILE = [gridpnt(triangleLE(1),:)
                gridpnt(triangleLE(2),:)
                gridpnt(triangleLE(3),:)];

if plotflag == true
    figure(pic)
    subplot(1,2,1)
    patch(gridpntTRILE(:,1),gridpntTRILE(:,2),'r')
    xlabel('Transmission Interval h')
    ylabel('Transmission Delay \tau')
    hold on
    pause(1)
    figure(pic)
    subplot(1,2,1)
    patch(gridpntTRILE(:,1),gridpntTRILE(:,2),'w')
    xlabel('Transmission Interval h')
    ylabel('Transmission Delay \tau')
    hold on
end

%-- Calculate location new gridpoint
%   Round to six decimals to prevent equal grid points due to numerical
%   inaccuracy
newgridpnt = round((alphagrid*gridpntTRILE)*1e6)/1e6;
%-- Subdivide triangle with the largest error into new triangles
[gridpnt_up,TRI_up,NoNewTRI_up] = triangulationGND(TRI,gridpnt,alphagrid,newgridpnt,rd);

else 
    break       % epsilon < epsilon_u  or maximum iterations reached
end

end
if (iter == maxiter) 
    warning('Maximum number of iterations reached \n')
end
if (NrGridpnt == gridpntM) 
    warning('Maximum number of grid points reached \n')
end

%-- Information
N = max(size(gridpnt));
disp(['  Total amount of gridpoints   = ' num2str(N)])
epsilonM = max(epsilon);
fprintf('  The maximum value of epsilon = %1.3e \n',epsilonM)

      
end

%% Function to subdivide triangles into new triangles
function [gridpnt_up,TRI_up,NoNewTRI_up] = triangulationGND(TRI,gridpnt,alphagrid,newgridpnt,rd)
% this function isolates the triangle with the largest approximation error 
% and divides this triangle into new triangles

Triangle = TRI(rd,:);

% Search for equal grid points
eqG = zeros(max(size(gridpnt)),1);
for ii=1:max(size(gridpnt))
    eqG(ii,1) = isequal(gridpnt(ii,:),newgridpnt);
end

eqN = find(eqG==1);

if isempty(eqN)==1
    % no equal gridpoints
    new = length(gridpnt)+1;
else
    % equal gridpoint found
    new = eqN;
end

% subdivide triangles
if alphagrid(1)~=0 && alphagrid(2)~=0 && alphagrid(3)~=0
	q1 = Triangle(1);
    q2 = Triangle(2);
    q3 = Triangle(3);
    TRI_new = [new q1 q2
               new q1 q3
               new q2 q3];
    NoNewTRI_up = 3;
else
    hp = find(alphagrid>0);     % find gridpoints where alpha_j ~= zero
    hz = find(alphagrid==0);    % find gridpoint where alpha_j == zero
    TRI_new = [new Triangle(hz) Triangle(hp(1))
               new Triangle(hz) Triangle(hp(2))];
    NoNewTRI_up = 2;
end
        
TRI(rd,:) = []; 
TRI_up = [TRI;TRI_new];

if isempty(eqN)==1
    % no equal gridpoints
    gridpnt_up = [gridpnt ; newgridpnt];
else
    % equal gridpoint found
    gridpnt_up = [gridpnt];
end

end