function [stable,GNBoutput] = dncs_GNB(NCSobj,gridpntM,epsilon_u,uWired)
% Function that analyses stability for a given NCS
% -------------------------------------------------------------------------
% Author  : S.J.L.M. van Loon
% Version : VII 
% Date    : 03-05-2011
% Project : Stability Analysis of Networked and Quantized Control Systems: 
%           a Switched Linear Systems Approach. Master project.
% -------------------------------------------------------------------------
% Overapproximation based on gridding & norm-bounding
%     x{k+1} = (\sum_{l=1}^{L}\alpha^l * Ab + Bb * \Delta * Cb)x{k}
% -------------------------------------------------------------------------
% Most of the symbols that are used in this file are based on the symbols 
% of the following TAC paper: "Stability Analysis of Networked Control 
% Systems Using a Switched Linear System Approach", by Donkers et al. 2011
%
% The code itself (and the theory behind the code) is explained in greather 
% detail in the manual of the NCS Toolbox and in "Stability Analysis of 
% Networked and Quantized Control Systems: a Switched Linear Systems 
% Approach", MSc Thesis S.J.L.M. van Loon, Eindhoven University of 
% Technology 2011
% -------------------------------------------------------------------------
% The input parameters are:
%  * NCSobj    : NCS object with system information
%  * gridpntM  : maximum number of gridpoints tolerated
%  * epsilon_u : user defined tightness of the overapproximation
%  * uWired    : if uWired==1, controller output is not communicated over a
%                network, uWired==0 it is
% -------------------------------------------------------------------------
% The output parameters are:
%  * stable  : if "stable=0" stability not guaranteed, if "stable=1"
%              stability is guaranteed
%  * Bbd     : matrix Bb * delta
%  * gridpnt : final number of grid points
% -------------------------------------------------------------------------
% The following NCS setups are included
% setup1 := NCS model with continuous-time controller where both y and u 
%           are sent over a network;
% setup2 := NCS model with discrete-time controller where both y and u are 
%           sent over a network;
% setup3 := NCS model if only y is transmitted over a network (assume that 
%           the controller communicates its values continuously and without
%           delays);
%
% Some possible extensions
% setup4 := NCS with state feedback where only y is sent over a network
%           (under construction);
% -------------------------------------------------------------------------

plotflag = false;
quant = 'no';
sigma=[];
%% - Extract Data
NCS_Sys = NCSobj;

% System Matrices
Ap = NCS_Sys.A;
Bp = NCS_Sys.B;
Cp = NCS_Sys.C;

% Nodes u and y
gammaU = NCS_Sys.gammaU;
gammaY = NCS_Sys.gammaY;
protocol = NCS_Sys.protocol;
%-- Select NCS setup and determine Gamma matrix
% setup1 := NCS model with continuous-time controller where both y and u are sent over a network;
% setup2 := NCS model with discrete-time controller where both y and u are sent over a network;
% setup3 := NCS model if only y is transmitted over a network (assume that the controller communicates 
%           its values continuously and without delays);
% setup4 := NCS model with state-feedback when only y is communicated over
%           the network (under construction!!!)

% Determine number of cells
nrCellU = size(gammaU,2);
nrCellY = size(gammaY,2);

GammaMatrix = cell(0);
disp('SYSTEM DATA:')
if NCS_Sys.h(1)-NCS_Sys.h(2) == 0
    disp(['  h          =  ' num2str(NCS_Sys.h(1))])
else
    disp(['  h          = [' num2str(NCS_Sys.h(1)) ',  ' num2str(NCS_Sys.h(2)) ']'])
end
if NCS_Sys.tau(1) - NCS_Sys.tau(2) == 0
    disp(['  tau        =  ' num2str(NCS_Sys.tau(1))]) 
else
    disp(['  tau        = [' num2str(NCS_Sys.tau(1)) ',  ' num2str(NCS_Sys.tau(2)) ']']) 
end


% Check gammaU
if nrCellU == 1 && nrCellY == 1
    disp('  Comm Constraints: None')
    comm = 'no';
    
    GammaMatrix = gammaY;
    
else

    if uWired == 1
        disp('  Comm Constraints: Only outputs')
        GammaMatrix = gammaY;
        commU = 'no';   % 'no' => no communication of u
    elseif uWired == 0
        disp('  Comm Constraints: Both inputs and outputs')
        for jj = 1:nrCellU
            GammaMatrix{jj} = blkdiag(gammaY{jj},gammaU{jj});
        end
        commU = 'yes';  % 'yes' => communication of u
    else
        error('  uWired input parameter should be either 0 or 1')
    end
end


% Total number of nodes
NrNodes = max(size(GammaMatrix));

% Controller
control = NCS_Sys.ctrlType;

switch control
    case{'Static Feedback'}
        
        % UNDER CONSTRUCTION
        %   only applicable with examples Thesis Jos
        
        Kx = -NCS_Sys.K{1};
        disp('  Controller:       Static Feedback ')
        NCSsetup = 'setup4';
        
        % Matrix sizes
        [np mp] = size(Bp);
        lp      = size(Cp,1);
        
        TF = strcmp('no',comm);
        if TF==1
            GammaMatrix = gammaY;
        end
        
    case{'C-LTI Dynamic Feedback'}
        Ac = NCS_Sys.Ac{1};
        Bc = NCS_Sys.Bc{1};
        Cc = NCS_Sys.Cc{1};
        Dc = NCS_Sys.Dc{1};
        disp('  Controller:       Continuous-time Dynamic Feedback')

        % Matrix sizes
        [np mp] = size(Bp);
        lp      = size(Cp,1); 
        [nc mc] = size(Bc);
        lc      = size(Cc,1);
        
        switch commU
            case{'no'}
                NCSsetup = 'setup3';
            case{'yes'}
                NCSsetup = 'setup1';
        end
        
    case{'D-LTI Dynamic Feedback'}
        Ac = NCS_Sys.Ac{1};
        Bc = NCS_Sys.Bc{1};
        Cc = NCS_Sys.Cc{1};
        Dc = NCS_Sys.Dc{1};
        disp('  Controller:       Discrete-time Dynamic Feedback')
        
        % Matrix sizes
        [np mp] = size(Bp);
        lp      = size(Cp,1); 
        [nc mc] = size(Bc);
        lc      = size(Cc,1);
        
        NCSsetup = 'setup2';
end

% Bounds on the transmission delays / intervals
tau = NCS_Sys.tau;
h   = NCS_Sys.h;

% Dropouts (modeled as prolongation of the sampling interval)
dropout = NCS_Sys.delta;
if dropout ~= 0
    hm = h(2)/(dropout+1);
    h  = [h(1) hm];
end


%% - Construct closed-loop matrices depending on setup
switch NCSsetup
    %----------------------------------------------------------------------
    % continuous-time controller and both y and u are communicated
    case {'setup1'}     
        
        % Construct matrix Lambda
        LambdaBar = blkdiag(Ap,Ac);
        
        % Determine the real Jordan form with the Linear Systems Toolbox
        [J T] = rjd(LambdaBar);
        
        % Matrices closed loop system
        B = [zeros(np,mc) Bp;Bc zeros(nc,mp)];
        C = blkdiag(Cp,Cc);
        D = [eye((mc+mp-lc),mc) zeros((mc+mp-lc),(lp+lc-mc));Dc eye(lc,(lp+lc-mc))];
        
        % Matrix Bb of overapproximation
        Bb = [T    T    T
              -C*T -C*T -C*T];
        
        % Matrix Cb of overapproximation
        Cb = cell(0);
        for ii = 1:NrNodes
            Cb{ii} = [inv(T)                 zeros((np+nc),NrNodes); 
                      T\B*D*C                T\B*D
                      zeros((np+nc),(np+nc)) -T\B*GammaMatrix{ii} ];
        end
        
    %----------------------------------------------------------------------
    % discrete-time controller and both y and u are communicated  
    case {'setup2'}     
        
        % Construct matrix Lambda
        LambdaBar = blkdiag(Ap,zeros(size(Ac)));

        % Determine the real Jordan form with the Linear Systems Toolbox
        [J T] = rjd(LambdaBar);        

        % Matrices closed loop system
        B = [zeros(np,mc) Bp;Bc zeros(nc,mp)];
        C = blkdiag(Cp,Cc);
        D = [eye((mc+mp-lc),mc) zeros((mc+mp-lc),(lp+lc-mc));Dc eye(lc,(lp+lc-mc))];

        % Matrix Bb of overapproximation
        Bb = [T    T    T
              -C*T -C*T -C*T];
        
        % Matrix Cb of overapproximation
        Cb = cell(0);
        for ii = 1:NrNodes
            Cb{ii} = [inv(T)                 zeros((np+nc),(mc+mp)); 
                      T\B*D*C                T\B*D
                      zeros((np+nc),(np+nc)) -T\B*GammaMatrix{ii} ];
        end 
    
    %----------------------------------------------------------------------
    % continuous-time controller and only y is communicated
    case {'setup3'}      
        
        % Construct matrix Lambda
        LambdaBar = [Ap           Bp*Cc
                     zeros(nc,np) Ac];
              
        % Determine the real Jordan form with the Linear Systems Toolbox
        [J T] = rjd(LambdaBar);
        
        % Matrices closed loop system
        B = [Bp*Dc;Bc];
        C = [Cp zeros(NrNodes)];
        D = eye(NrNodes);
        
        Bb = [T    T    T
              -C*T -C*T -C*T];
        
        switch quant
            case{'no'}
            % Matrix Cb of overapproximation
            Cb = cell(0);
            nG = size(GammaMatrix{1},1);
            for ii = 1:NrNodes
                Cb{ii} = [inv(T)                 zeros((np+nc),nG); 
                          T\B*D*C                T\B*D
                          zeros((np+nc),(np+nc)) -T\B*GammaMatrix{ii} ];
            end     
            case{'yes'}
            % Matrix Cb and Gb of overapproximation
            Cb = cell(0);
            Gb = cell(0);
            nG = size(GammaMatrix{1},1);
            for ii = 1:NrNodes
                Cb{ii} = [inv(T)                 zeros((np+nc),nG); 
                          T\B*D*C                T\B*D
                          zeros((np+nc),(np+nc)) -T\B*GammaMatrix{ii} ];
                Gb{ii} = [zeros((np+nc),nG)
                          zeros((np+nc),nG)
                          T\B*GammaMatrix{ii}];
            end 
        end
    %----------------------------------------------------------------------
    % state-feedback controller and only y is communicated (under construction!!!!)
    case {'setup4'}      
        
        % UNDER CONSTRUCTION
        %  only checked for the examples of Jos
        
        % Construct matrix Lambda
        LambdaBar = Ap;
              
        % Determine the real Jordan form with the Linear Systems Toolbox
        [J T] = rjd(LambdaBar);
        
        % Matrices closed loop system
        B = Bp;
        C = Cp;
                
        Bb = [T    T    T
              -C*T -C*T -C*T];
                        
        % Matrix Cb of overapproximation
        Cb = cell(0);
        nG = size(GammaMatrix{1},1);
        for ii = 1:NrNodes
            Cb{ii} = [inv(T)    zeros(np,nG); 
                      T\B*Kx*C  T\B*Kx
                      zeros(np) -T\B*Kx*GammaMatrix{ii} ];
        end    
end

%-- Determination of separate Jordan blocks with real eigenvalues JJ
JJ = JordanBlocks(J); % NCS toolbox function written by J.J.C. van Schendel
K  = max(size(JJ));   % total number of Jordan blocks


%% Gridding procedure

%1- Create Gridpoints 
info = false;
[gridpnt0 setG const] = CreateGridpoints(tau,h,info);

%2- Compute overapproximation
% Combine input data
GNBinput.Bb        = Bb;
GNBinput.Cb        = Cb;
GNBinput.JJ        = JJ;
GNBinput.gridpnt0  = gridpnt0;
GNBinput.gridpntM  = gridpntM;
GNBinput.epsilon_u = epsilon_u;
GNBinput.NrNodes   = NrNodes;
GNBinput.setG      = setG;
GNBinput.const     = const;
GNBinput.plotflag  = plotflag;
disp(' ')
disp('OVERAPPROXIMATION DATA:')
[GNBoutput] = GNB(GNBinput);

% Extract output data
Bbd      = GNBoutput.Bbd;
gridpnt  = GNBoutput.gridpnt;

%% -- Determine A, Eh and Eht evaluated at the gridpoints
switch setG
    %----------------------------------------------------------------------
    % No uncertainty
    case {'constht'}
        switch NCSsetup
            case{'setup1'}
               
                tauC = const(1,1);
                hC   = const(1,2);
                
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                
                syms s
                Ah  = blkdiag(Tp*expm(Jp*hC)/Tp,Tc*expm(Jc*hC)/Tc);
                Eh  = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC))/Tp,Tc*double(int(expm(Jc*s),s,0,hC))/Tc);
                Eht = Tp*blkdiag(double(int(expm(Jp*s),s,0,hC-tauC))/Tp,Tc*double(int(expm(Jc*s),s,0,hC-tauC))/Tc);
                
                Ab = cell(0);
                for jj = 1:NrNodes
                     Ab{jj} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht*B*GammaMatrix{jj}
                                 C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht*B*GammaMatrix{jj}-C*Eh*B*D ]);
                end
                
            case{'setup2'}
                            
                tauC = const(1,1);
                hC   = const(1,2);
                
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                   
                syms s
                Ah  = blkdiag(Tp*expm(Jp*hC)/Tp,Ac);
                Eh  = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC))/Tp,Tc*double(int(expm(Jc*s),s,0,hC))/Tc);
                Eht = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC-tauC))/Tp,Tc*double(int(expm(Jc*s),s,0,hC-tauC))/Tc);
                
                for jj = 1:NrNodes
                     Ab{jj} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht*B*GammaMatrix{jj}
                                 C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht*B*GammaMatrix{jj}-C*Eh*B*D ]);
                end
                
            case{'setup3'}
                
                tauC = const(1,1);
                hC   = const(1,2);
                
                Ac3 = [Ap Bp*Cc;zeros(nc,np) Ac];
                [Jc3 Tc3] = rjd(Ac3);

                syms s
                Ah = Tc3*expm(Jc3*hC)/Tc3;
                Eh = Tc3*double(int(expm(Jc3*s),s,0,hC))/Tc3;
                Eht = Tc3*double(int(expm(Jc3*s),s,0,hC-tauC))/Tc3;
                
                Ab = cell(0);
                for jj = 1:NrNodes
                    Ab{jj} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht*B*GammaMatrix{jj}
                                C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht*B*GammaMatrix{jj}-C*Eh*B*D ]);
                end
                
            case{'setup4'}
                
                tauC = const(1,1);
                hC   = const(1,2);
                [Jc4 Tc4] = rjd(Ap);

                syms s
                Ah = Tc4*expm(Jc4*hC)/Tc4;
                Eh = Tc4*double(int(expm(Jc4*s),s,0,hC))/Tc4;
                Eht = Tc4*double(int(expm(Jc4*s),s,0,hC-tauC))/Tc4;
                
                Ab = cell(0);
                for jj = 1:NrNodes
                        Ab{jj} = ([ Ah+Eh*B*Kx*C       Eh*B*Kx-Eht*B*Kx*GammaMatrix{jj}
                                    C-C*Ah-C*Eh*B*Kx*C -C*Eh*B*Kx + C*Eht*B*Kx*GammaMatrix{jj}]);
                end    
        end
    
    %----------------------------------------------------------------------
    % Uncertain transmission delay and constant sampling interval    
    case {'vart'}
        
        switch NCSsetup
            
            case{'setup1'}
                
                hC      = const;
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                
                Ah = blkdiag(Tp*expm(Jp*hC)/Tp,Tc*expm(Jc*hC)/Tc);
                Eh = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC))/Tp,Tc*double(int(expm(Jc*s),s,0,hC))/Tc);
                
                syms s
                Eht = cell(0);
                for ii = 1:length(gridpnt)
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC-gridpnt(ii,1)))/Tp,Tc*double(int(expm(Jc*s),s,0,hC-gridpnt(ii,1)))/Tc);
                end
                Ab = cell(0);
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh*B*D ]);
                    end
                end
                
            case{'setup2'}
                
                hC      = const;
                [Jp Tp] = rjd(Ap);
                                
                Ah = blkdiag(Tp*expm(Jp*hC)/Tp,Ac);
                Eh = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC))/Tp,eye(nc,nc));
                
                syms s
                Eht = cell(0);
                for ii = 1:length(gridpnt)
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,hC-gridpnt(ii,1)))/Tp,eye(nc,nc));
                end
                Ab = cell(0);
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh*B*D ]);
                    end
                end
                
            case{'setup3'}
                
                hC  = const;
                Ac3 = [Ap Bp*Cc;zeros(nc,np) Ac];
                [Jc3 Tc3] = rjd(Ac3);

                syms s
                Ah = Tc3*expm(Jc3*hC)/Tc3;
                Eh = Tc3*double(int(expm(Jc3*s),s,0,hC))/Tc3;
                
                Eht = cell(0);
                for ii = 1:length(gridpnt)
                    Eht{ii} = Tc3*double(int(expm(Jc3*s),s,0,hC-gridpnt(ii,1)))/Tc3;
                end
                Ab = cell(0);
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah+Eh*B*D*C        Eh*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah-C*Eh*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh*B*D ]);
                    end
                end
                
            case{'setup4'}
                
                hC  = const;
                [Jc4 Tc4] = rjd(Ap);

                syms s
                Ah = Tc4*expm(Jc4*hC)/Tc4;
                Eh = Tc4*double(int(expm(Jc4*s),s,0,hC))/Tc4;

                Eht = cell(0);
                for ii = 1:length(gridpnt)
                    Eht{ii} = Tc4*double(int(expm(Jc4*s),s,0,hC-gridpnt(ii,1)))/Tc4;
                end
                Ab = cell(0);
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah+Eh*B*Kx*C       Eh*B*Kx-Eht{ii}*B*Kx*GammaMatrix{jj}
                                       C-C*Ah-C*Eh*B*Kx*C -C*Eh*B*Kx + C*Eht{ii}*B*Kx*GammaMatrix{jj}]);
                    end
                end    
        end % end switch NCS
    
    %----------------------------------------------------------------------
    % Uncertain transmission interval and constant delay    
    case {'varh'}
        
        switch NCSsetup
            
            case{'setup1'}
                
                tauC    = const;
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                
                syms s
                Ah = cell(0); Eh = cell(0); Eht = cell(0); Ab = cell(0);
                for ii = 1:length(gridpnt)
                    Ah{ii}  = blkdiag(Tp*expm(Jp*gridpnt(ii,1))/Tp,Tc*expm(Jc*gridpnt(ii,1))/Tc);
                    Eh{ii}  = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)))/Tp,Tc*double(int(expm(Jc*s),s,0,gridpnt(ii,1)))/Tc);
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)-tauC))/Tp,Tc*double(int(expm(Jc*s),s,0,gridpnt(ii,1)-tauC))/Tc);
                end
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                    end
                end
                
            case{'setup2'}
                
                tauC    = const
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                
                syms s
                Ah = cell(0); Eh = cell(0); Eht = cell(0); Ab = cell(0);
                for ii = 1:length(gridpnt)
                    Ah{ii}  = blkdiag(Tp*expm(Jp*gridpnt(ii,1))/Tp,Ac);
                    Eh{ii}  = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)))/Tp,eye(nc,nc));
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)-tauC))/Tp,eye(nc,nc));
                end
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                    end
                end
                
            case{'setup3'}
                
                tauC      = const;
                Ac3       = [Ap Bp*Cc;zeros(nc,np) Ac];
                [Jc3 Tc3] = rjd(Ac3);
                JJ        = JordanBlocks(Jc3);   % function written by J.J.C. van Schendel
                Kj        = max(size(JJ));       % total number of Jordan blocks
                
                % Search for the maximum size of the Jordan Blocks
                for ii = 1:Kj
                    Msize(ii,1) = max(size(JJ{ii}));
                end

                if max(Msize)== 1
                    JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
                else
                    JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
                end
                
                switch JordanB
                    
                    case {'RD'} % diagonalizable, avoid computing integral of matrix expm
                    
                    Ah = cell(0); Eh = cell(0); Eht = cell(0);
                    for ii = 1:length(gridpnt)
                        Ah{ii}  = Tc3*expm(Jc3*gridpnt(ii,1))/Tc3;
                        for uu = 1:K
                            if abs(Jc3(uu,uu)) > 1e-14
                                Ehint(uu,uu)  = 1/Jc3(uu,uu)*(exp(Jc3(uu,uu)*gridpnt(ii,1))-1);
                                Ehtint(uu,uu) = 1/Jc3(uu,uu)*(exp(Jc3(uu,uu)*(gridpnt(ii,1)-tauC))-1);
                            else
                                Ehint(uu,uu)  = gridpnt(ii,1);
                                Ehtint(uu,uu) = gridpnt(ii,1)-tauC;
                            end
                        end             
                        Eh{ii}  = double(Tc3*Ehint/Tc3);
                        Eht{ii} = double(Tc3*Ehtint/Tc3);
                        clear Ehint Ehtint
                    end 
                    
                    case {'RnDC'} % not diagonalizable
                    
                    syms s
                    Ah = cell(0); Eh = cell(0); Eht = cell(0);
                    for ii = 1:length(gridpnt)
                        Ah{ii}  = Tc3*expm(Jc3*gridpnt(ii,1))/Tc3;
                        Eh{ii}  = Tc3*double(int(expm(Jc3*s),s,0,gridpnt(ii,1)))/Tc3;
                        Eht{ii} = Tc3*double(int(expm(Jc3*s),s,0,gridpnt(ii,1)-tauC))/Tc3;
                    end    
                end % end switch JordanB
                
                switch quant
                    case{'no'}
                    % construct Ab  
                    Ab = cell(0);
                    for jj = 1:NrNodes
                        for ii = 1:length(gridpnt)
                            Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                           C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                        end
                    end    
                    case{'yes'}
                    % construct Ab and Eb 
                    Ab = cell(0); Eb = cell(0);
                    for jj = 1:NrNodes
                        for ii = 1:length(gridpnt)
                            Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                           C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                            Eb{jj,ii} = ([ Eht{ii}*B*GammaMatrix{jj}
                                           D\GammaMatrix{jj}-C*Eht{ii}*B*GammaMatrix{jj} ]);
                        end
                    end
                end
        end
    
    %----------------------------------------------------------------------
    % Uncertain transmission delays and uncertain transmission intervals    
    case {'varht'}
        switch NCSsetup
            case{'setup1'}
                
                [Jp Tp] = rjd(Ap);
                [Jc Tc] = rjd(Ac);
                
                syms s
                Ah = cell(0); Eh = cell(0); Eht = cell(0); Ab = cell(0);
                for ii = 1:length(gridpnt)
                    Ah{ii}  = blkdiag(Tp*expm(Jp*gridpnt(ii,1))/Tp,Tc*expm(Jc*gridpnt(ii,1))/Tc);
                    Eh{ii}  = blkdiag(double(Tp*int(expm(Jp*s),s,0,gridpnt(ii,1))/Tp),Tc*double(int(expm(Jc*s),s,0,gridpnt(ii,1)))/Tc);
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)-gridpnt(ii,2)))/Tp,Tc*double(int(expm(Jc*s),s,0,gridpnt(ii,1)-gridpnt(ii,2)))/Tc);
                end
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                    end
                end
                
            case{'setup2'}
                
                [Jp Tp] = rjd(Ap);
                               
                syms s
                Ah = cell(0); Eh = cell(0); Eht = cell(0); Ab = cell(0);
                for ii = 1:length(gridpnt)
                    Ah{ii}  = blkdiag(Tp*expm(Jp*gridpnt(ii,1))/Tp,Ac);
                    Eh{ii}  = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)))/Tp,eye(nc,nc));
                    Eht{ii} = blkdiag(Tp*double(int(expm(Jp*s),s,0,gridpnt(ii,1)-gridpnt(ii,2)))/Tp,eye(nc,nc));
                end
                for jj = 1:NrNodes
                    for ii = 1:length(gridpnt)
                        Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                       C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                    end
                end
                
            case{'setup3'}
                
                Ac3         = [Ap Bp*Cc;zeros(nc,np) Ac];
                [Jc3 Tc3]   = rjd(Ac3);
                JJ          = JordanBlocks(Jc3);   % function written by J.J.C. van Schendel
                Kj          = max(size(JJ));       % total number of Jordan blocks
                
                % Search for the maximum size of the Jordan Blocks
                Msize = zeros(Kj,1);
                for ii = 1:Kj
                    Msize(ii,1) = max(size(JJ{ii}));
                end

                if max(Msize)== 1
                    JordanB = 'RD';   % Eigenvalues: Real and disctinct (analytical solution)
                else
                    JordanB = 'RnDC'; % Eigenvalues: Real and non-disctinct or complex
                end
                
                switch JordanB
                    case {'RD'} % diagonalizable, avoid computing integral of matrix expm
                    Ah = cell(0); Eh = cell(0); Eht = cell(0);     
                    for ii = 1:length(gridpnt)
                        Ah{ii} = Tc3*expm(Jc3*gridpnt(ii,1))/Tc3;
                        for uu = 1:K
                            if abs(Jc3(uu,uu)) > 1e-14
                                Ehint(uu,uu)  = 1/Jc3(uu,uu)*(exp(Jc3(uu,uu)*gridpnt(ii,1))-1);
                                Ehtint(uu,uu) = 1/Jc3(uu,uu)*(exp(Jc3(uu,uu)*(gridpnt(ii,1)-gridpnt(ii,2)))-1);
                            else
                                Ehint(uu,uu)  = gridpnt(ii,1);
                                Ehtint(uu,uu) = gridpnt(ii,1)-gridpnt(ii,2);
                            end
                        end             
                        Eh{ii}  = double(Tc3*Ehint/Tc3);
                        Eht{ii} = double(Tc3*Ehtint/Tc3);
                        clear Ehint Ehtint
                    end       
                    case {'RnDC'} % not diagonalizable
                    syms s
                    Ah = cell(0); Eh = cell(0); Eht = cell(0); 
                    for ii = 1:length(gridpnt)
                        Ah{ii}  = Tc3*expm(Jc3*gridpnt(ii,1))/Tc3;
                        Eh{ii}  = Tc3*double(int(expm(Jc3*s),s,0,gridpnt(ii,1)))/Tc3;
                        Eht{ii} = Tc3*double(int(expm(Jc3*s),s,0,gridpnt(ii,1)-gridpnt(ii,2)))/Tc3;
                    end    
                end % end switch JordanB
                
                switch quant
                    case{'no'}
                    % construct Ab
                    Ab = cell(0);
                    for jj = 1:NrNodes
                        for ii = 1:length(gridpnt)
                            Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                           C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                        end
                    end    
                    case{'yes'}
                    % construct Ab and Eb
                    Ab = cell(0); Eb = cell(0);
                    for jj = 1:NrNodes
                        for ii = 1:length(gridpnt)
                            Ab{jj,ii} = ([ Ah{ii}+Eh{ii}*B*D*C        Eh{ii}*B*D-Eht{ii}*B*GammaMatrix{jj}
                                           C-C*Ah{ii}-C*Eh{ii}*B*D*C  eye(NrNodes)-D\GammaMatrix{jj}+C*Eht{ii}*B*GammaMatrix{jj}-C*Eh{ii}*B*D ]);
                            Eb{jj,ii} = ([ Eht{ii}*B*GammaMatrix{jj}
                                           D\GammaMatrix{jj}-C*Eht{ii}*B*GammaMatrix{jj} ]);
                        end
                    end
                end % end switch quant
        end % end switch NCSsetup
end % end switch setG



%% -- LMI based stability analysis

switch quant
%--------------------------------------------------------------------------
% Not include quantization    
case{'no'}

switch protocol
    % ---------------------------------------------------------------------
    % RR
    case{'RR'}
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);

    LL = [];
    
    % Construct symmetric P matrices
    P = cell(0);
    for ii = 1:NrNodes
        P{ii} = sdpvar(na);
    end
    
    % Round-Robin protocol (for one node, equal to common quadratic LF)
    P{NrNodes+1} = P{1};
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
    for ii = 1:NrNodes                   % number of nodes
        for k = 1:max(size(gridpnt))     % number of gridpoints

            % create matrix R, where each ri has the size of the corresponding Jordan block
            for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
                r1{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
                r2{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
                r3{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            end
            R{ii,k} = blkdiag(blkdiag(r1{ii,k,:}),blkdiag(r2{ii,k,:}),blkdiag(r3{ii,k,:}));

            LL = LL + set([P{ii}            zeros(na,mb)  Ab{ii,k}'*P{ii+1} Cb{ii}'*R{ii,k}
                           zeros(mb,na)     R{ii,k}       Bbd'*P{ii+1}      zeros(mb,mb) 
                           P{ii+1}*Ab{ii,k} P{ii+1}*Bbd   P{ii+1}           zeros(na,mb)
                           R{ii,k}*Cb{ii}   zeros(mb,mb)  zeros(mb,na)      R{ii,k}         ]>0);
        end
    end  

    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LL,[],sdpsettings('solver','sedumi','verbose',0));
    check     = checklmi(LL);
    Stability = StabInfo.problem;
    
    for ii = 1:NrNodes
        P{ii} = double(P{ii});
    end
    for ii = 1:NrNodes
        for k = 1:max(size(gridpnt))
            R{ii,k} = double(R{ii,k});
        end
    end

    %- Check feasibility
    stable = 1;

    % first check - problem   
    if Stability == 1
        stable = 0;
    end

    % second check - residuals
    PrimalRes = check > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
    end    
    
    % ---------------------------------------------------------------------
    % TOD
    case{'TOD'}
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);

    LL  = []; LLeq = [];
    P   = sdpvar(na);
    pi  = sdpvar(NrNodes,NrNodes,'full');
    LL  = LL + set(pi(:)>0);
    
    V = cell(0);
    for ii = 1:NrNodes
        V{ii} = blkdiag(zeros((na-NrNodes)),-GammaMatrix{ii});
        LLeq  = LLeq+set(sum(pi(:,ii))==1); 
    end
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); piV = cell(0);
    for ii = 1:NrNodes                   % number of nodes
        for k = 1:max(size(gridpnt))     % number of gridpoints
            % create matrix R, where each ri has the size of the corresponding Jordan block
            for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
                r1{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
                r2{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
                r3{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            end
            R{ii,k} = blkdiag(blkdiag(r1{ii,k,:}),blkdiag(r2{ii,k,:}),blkdiag(r3{ii,k,:}));
            
            Pi = P + V{ii};
            
            sumpiV = zeros(max(size(V{1})));
            for qq = 1:NrNodes 
                piV{qq} = pi(qq,ii)*V{qq};
                sumpiV = sumpiV+piV{qq};
            end
            Pj = P+sumpiV;

            LL = LL + set([Pi               zeros(na,mb)  Ab{ii,k}'*Pj      Cb{ii}'*R{ii,k}
                           zeros(mb,na)     R{ii,k}       Bbd'*Pj           zeros(mb,mb) 
                           Pj*Ab{ii,k}      Pj*Bbd        Pj                zeros(na,mb)
                           R{ii,k}*Cb{ii}   zeros(mb,mb)  zeros(mb,na)      R{ii,k}         ]>0);
        end
    end
    
    LMI = LL+LLeq;
    
    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
    
    Stability   = StabInfo.problem;
    [pres,dres] = checkset(LMI);
    
    checkP = pres(1:end-NrNodes);
    checkE = pres(end-NrNodes+1:end);
    
    %- Check feasibility
    stable = 1;

    % first check - infeasible problem   
    if Stability == 1
        stable = 0;
    end

    % second check - P residuals
    PrimalRes = checkP > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
    end    
    
    % third check - E residuals
    EqualRes = abs(checkE) < 1e-8; % deviation of sum(pi) of 1e-8 allowed
    isPosE   = find(EqualRes==0);
    if isempty(isPosE)==0
        stable = 0;
        disp('sum(pi)~=1')
    end 
    
    % ---------------------------------------------------------------------
    % no communication constraints: parameter depended quadratic Lyapunov 
    % function
    case{'pardep'}
        
        % NOT CHECKED!!!!!!
        
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);

    LL = [];
    
    % Construct symmetric P matrix
    P = cell(0);
    for k = 1:max(size(gridpnt))     % number of gridpoints
        P{k} = sdpvar(na);
    end
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
    for k = 1:max(size(gridpnt))     % number of gridpoints
                
        % create matrix R, where each ri has the size of the corresponding Jordan block
        for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
            r1{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
            r2{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            r3{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
        end
        R{k} = blkdiag(blkdiag(r1{k,:}),blkdiag(r2{k,:}),blkdiag(r3{k,:}));
        
        for z=1:max(size(gridpnt))     % number of gridpoints
        LL = LL + set([P{k}         zeros(na,mb) Ab{k}'*P{z}  Cb{1}'*R{k}
                       zeros(mb,na) R{k}         Bbd'*P{z}    zeros(mb,mb) 
                       P{z}*Ab{k}   P{z}*Bbd     P{z}         zeros(na,mb)
                       R{k}*Cb{1}   zeros(mb,mb) zeros(mb,na) R{k}         ]>0);
        end
    end
 

    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LL,[],sdpsettings('solver','sedumi','verbose',0));
    check     = checklmi(LL);
    Stability = StabInfo.problem;
    
    for k = 1:max(size(gridpnt))
        P{k} = double(P{k});
        R{k} = double(R{k});
    end

    %- Check feasibility
    stable = 1;

    % first check - problem   
    if Stability == 1
        stable = 0;
    end

    % second check - residuals
    PrimalRes = check > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
    end
        
    % ---------------------------------------------------------------------
    % no communication constraints: common quadratic Lyapunov function    
    case{'common'}
        
        % NOT CHECKED!!!!!!
        
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);    
     
    LL = [];
    
    % Construct symmetric P matrix
    P = sdpvar(na);
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
    for k = 1:max(size(gridpnt))     % number of gridpoints

        % create matrix R, where each ri has the size of the corresponding Jordan block
        for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
            r1{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
            r2{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            r3{k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
        end
        R{k} = blkdiag(blkdiag(r1{k,:}),blkdiag(r2{k,:}),blkdiag(r3{k,:}));
                
        LL = LL + set([P            zeros(na,mb) Ab{k}'*P     Cb{1}'*R{k}
                       zeros(mb,na) R{k}         Bbd'*P       zeros(mb,mb) 
                       P*Ab{k}      P*Bbd        P            zeros(na,mb)
                       R{k}*Cb{1}   zeros(mb,mb) zeros(mb,na) R{k}         ]>0);
    end
 

    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LL,[],sdpsettings('solver','sedumi','verbose',0));
    check     = checklmi(LL);
    Stability = StabInfo.problem;
    checkset(LL)
    
    for k = 1:max(size(gridpnt))
        R{k} = double(R{k});
    end

    %- Check feasibility
    stable = 1;

    % first check - problem   
    if Stability == 1
        stable = 0;
    end

    % second check - residuals
    PrimalRes = check > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
    end    
        
end % end switch protocol
    
%--------------------------------------------------------------------------
% Include quantization
case{'yes'}

switch protocol
    
    %----------------------------------------------------------------------
    % Quantization included and RR protocol
    case{'RR'}
    
    % UNDER CONSTRUCTION
    % not ready for toolbox use!!!!!    
        
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);
    
    za = 2;         

    alpha3 = 1e-3;
    LL = [];
    P = cell(0);
    for ii = 1:NrNodes
        P{ii} = sdpvar(na);
    end
    
    P{NrNodes+1} = P{1};
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0); 
    for ii = 1:NrNodes                   % number of nodes
        for k = 1:max(size(gridpnt))     % number of gridpoints

            % create matrix R, where each ri has the size of the corresponding Jordan block
            for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
                r1{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
                r2{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
                r3{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            end
            R{ii,k} = blkdiag(blkdiag(r1{ii,k,:}),blkdiag(r2{ii,k,:}),blkdiag(r3{ii,k,:}));

            LL = LL + set([P{ii}             zeros(na,za)     zeros(na,mb) Ab{ii,k}'*P{ii+1} Cb{ii}'*R{ii,k} alpha3*eye(na,na)
                           zeros(za,na)      sigma*eye(za,za) zeros(za,mb) Eb{ii,k}'*P{ii+1} Gb{ii}'*R{ii,k} zeros(za,na)
                           zeros(mb,na)      zeros(mb,za)     R{ii,k}      Bbd'*P{ii+1}      zeros(mb,mb)    zeros(mb,na)
                           P{ii+1}*Ab{ii,k}  P{ii+1}*Eb{ii,k} P{ii+1}*Bbd  P{ii+1}           zeros(na,mb)    zeros(na,na)
                           R{ii,k}*Cb{ii}    R{ii,k}*Gb{ii}   zeros(mb,mb) zeros(mb,na)      R{ii,k}         zeros(mb,na)
                           alpha3*eye(na,na) zeros(na,za)     zeros(na,mb) zeros(na,na)      zeros(na,mb)    alpha3*eye(na,na)   ]> 0);
        end
    end 

    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LL,[],sdpsettings('solver','sedumi','verbose',0));
    check     = checklmi(LL);
    Stability = StabInfo.problem;
    

    %- Check feasibility
    stable = 1;

    % first check - problem   
    if Stability == 1
        stable = 0;
    end

    % second check - residuals
    PrimalRes = check > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
    end
    
    %----------------------------------------------------------------------
    % Quantization included and TOD protocol
    case{'TOD'}
    
    %----------------------------------------------------------------------
    %-- Only Relaxation at k
    %----------------------------------------------------------------------
    %-- Create Matrix Qi --------------------------------------------------
    % Qi = [0 0  | 0        = [Qi11 | Qi12
    %       0 Ti | Ti          -----|----- 
    %       - - -|- -          Qi21 | Qi22]     
    %       0 Ti | Ti]      Where Qi21 = Qi12^T
    % 
    % Sizes:
    % Qi = [(np+nc) x (np+nc) (np+nc) x (np+nc) | (np+nc) x NrNodes        
    %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes           
    %       - - - - - - - - - - - - - - - - - - | - - - - - - - - -               
    %       NrNodes x (np+nc) NrNodes x NrNodes | NrNodes x NrNodes]      
    %----------------------------------------------------------------------
    q11 = cell(0); q12 = cell(0); q22 = cell(0);
    for ii = 1:NrNodes
        q11{ii} = [zeros((np+nc),(np+nc)) zeros((np+nc),NrNodes)
                   zeros(NrNodes,(np+nc)) GammaMatrix{ii}];
        q12{ii} = [zeros((np+nc),NrNodes) 
                   GammaMatrix{ii}];
        q22{ii} = GammaMatrix{ii};
    end

    %-- create \hat{Q}^i and \hat{Q}^j ------------------------------------
    LLeq = [];
    xi = cell(0); qi11 = cell(0); qi12 = cell(0); qi22 = cell(0);
    for ii = 1:NrNodes
        for ff = 1:NrNodes
            if ii == ff
                % relaxation k
                xi{ii,ff} = 0;
                qi11{ii,ff} = xi{ii,ff}*(q11{ii}-q11{ff});
                qi12{ii,ff} = xi{ii,ff}*(q12{ii}-q12{ff});
                qi22{ii,ff} = xi{ii,ff}*(q22{ii}-q22{ff});
            else
                % relaxation k
                xi{ii,ff} = sdpvar(1,1);
                qi11{ii,ff} = xi{ii,ff}*(q11{ii}-q11{ff});
                qi12{ii,ff} = xi{ii,ff}*(q12{ii}-q12{ff});
                qi22{ii,ff} = xi{ii,ff}*(q22{ii}-q22{ff});
                LLeq = LLeq + set(xi{ii,ff}>=0);
            end
        end
    end

    % Add elements together
    Qhi11 = cell(0); Qhi12 = cell(0); Qhi22 = cell(0);
    for ii = 1:NrNodes
        % relaxation k
        tmpi11 = zeros((np+nc+NrNodes),(np+nc+NrNodes));
        tmpi12 = zeros((np+nc+NrNodes),NrNodes);
        tmpi22 = zeros(NrNodes,NrNodes);
        for ff = 1:NrNodes
            % relaxation k
            tmpi11 = tmpi11+qi11{ii,ff};
            tmpi12 = tmpi12+qi12{ii,ff};
            tmpi22 = tmpi22+qi22{ii,ff};
        end
        % relaxation k
        Qhi11{ii,1} = tmpi11;
        Qhi12{ii,1} = tmpi12;
        Qhi22{ii,1} = tmpi22;
    end
    %----------------------------------------------------------------------    
    
    %----------------------------------------------------------------------
    % Actual stability analysis
    % matrix sizes
    na = size(Ab{1},1);
    mb = size(Bbd,2);
    
    za = NrNodes;

    %-- Decay rate
    alpha3 = 1e-3;
    
    LL = [];
    P = cell(0);
    for ii = 1:NrNodes
        P{ii} = sdpvar(na); 
    end
    
    r1 = cell(0); r2 = cell(0); r3 = cell(0); R = cell(0);
    for ii = 1:NrNodes                  % number of nodes
        for k = 1:max(size(gridpnt))    % number of gridpoints
        % create matrix R, where each ri has the size of the corresponding Jordan block
        for m = 1:K    % K = number of Jordan blocks, JJ = size Jordan block
            r1{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));   
            r2{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
            r3{ii,k,m} = sdpvar(1,1)*eye(max(size(JJ{m})));
        end
        R{ii,k} = blkdiag(blkdiag(r1{ii,k,:}),blkdiag(r2{ii,k,:}),blkdiag(r3{ii,k,:}));
            
        for jj = 1:NrNodes   % number of nodes
            LL = LL + ...
                set([P{ii}-alpha3*eye(na,na)-Qhi11{ii} -Qhi12{ii}                  zeros(na,mb) Ab{ii,k}'*P{jj} Cb{ii}'*R{ii,k}
                     -Qhi12{ii}'                       -Qhi22{ii}+sigma*eye(za,za) zeros(za,mb) Eb{ii,k}'*P{jj} Gb{ii}'*R{ii,k}
                     zeros(mb,na)                      zeros(mb,za)                R{ii,k}      Bbd'*P{jj}      zeros(mb,mb)
                     P{jj}*Ab{ii,k}                    P{jj}*Eb{ii,k}              P{jj}*Bbd    P{jj}           zeros(na,mb)
                     R{ii,k}*Cb{ii}                    R{ii,k}*Gb{ii}              zeros(mb,mb) zeros(mb,na)    R{ii,k}      ]>=0);
        end % end for jj
        end % end for gridpnt
    end % end for NrNodes 
    
    neq = size(LLeq,1);
    LMI       = LL + LLeq;
    
    %-- Solve LMIs with SeDuMi
    StabInfo  = solvesdp(LMI,[],sdpsettings('solver','sedumi','verbose',0));
    [pres dres] = checkset(LMI);
    Stability = StabInfo.problem;
           
    checkP = pres(1:end-neq);
    checkE = pres(end-neq+1:end);
    
    checkxi = zeros(NrNodes,NrNodes); xiout = cell(0); 
    for ii = 1:NrNodes
        P{ii} = double(P{ii});
        for jj = 1:NrNodes
            xiout{ii,jj} = double(xi{ii,jj});
            checkxi(ii,jj) = xiout{ii,jj} >= 0;
        end
    end
        
    %- Check feasibility
    stable = 1;

    % first check - infeasible problem   
    if Stability == 1
        stable = 0;
    end

    % second check - Positive definiteness P
    minEigP = zeros(NrNodes,1);
    for ii=1:NrNodes
        minEigP(ii,1)=min(eig(P{ii}));
    end

    checkPosDef = find(minEigP < 1e-14);

    if isempty(checkPosDef)==0
        stable = 0;
        disp('P matrices not positive definite')
    end 
    
    % third check - P residuals
    PrimalRes = checkP > 0;
    isPos     = find(PrimalRes==0);
    if isempty(isPos)==0
        stable = 0;
        disp('Residuals not satisfied')
    end    
    
    % fourth check - Equality residuals
    EqualRes = abs(checkE) < 1e-12; % small enough
    isPosE   = find(EqualRes==0);
    isPosSDxi= find(checkxi==0);
    if isempty(isPosE)==0
        if isempty(isPosSDxi)==1 && isempty(isPos)==1
        stable = 1;
        else
            stable = 0;
            disp('Equality residuals not satisfied')
        end
    end 

end % end switch protocol
        
end % end switch quant
disp(' ')
disp('STABILITY DATA:')
disp(['  Number of LMIs needed to verify stability = ' num2str(size(LL))])
disp(['  ' StabInfo.info])
    if stable ==1
        disp('  Stability: Guaranteed')
    else
        disp('  Stability: Not Guaranteed')
    end
    
fprintf('-------------------------------------------------------------- \n')
end