function [stable,K,totalVerts, expectedNumLMIs]=dncs_JNF_CH(NCS,ovraprx,drpmdl,lyap,gamma)

    A = NCS.A;
    B = NCS.B;
    K = NCS.K{1};

    h = NCS.h;
    tau = NCS.tau;
    deltaBar = NCS.delta;

    h_min = h(1);
    h_max = h(2);
    tau_min = tau(1);
    tau_max = tau(2);

    %% input error checking
    if strcmp(NCS.ctrlType,'Static Feedback') == 0
        error('NCS must be controller type ''Static Feedback''')
    end
    if size(NCS.C,1) ~= size(A,1) || size(NCS.C,2) ~= size(A,2)
        error('NCS must be state feedback (i.e. C = I)')
    end
    if sum(sum(NCS.C - eye(size(A)))) ~= 0
        error('NCS must be state feedback (i.e. C = I)')
    end

    if size(A,2) ~= size(B,1)
        error('number of columns of A must match number of rows of B')
    elseif isempty(K) == 0 && size(B,2) ~= size(K,1)
        error('number of columns of B must match number of rows of K')
    elseif isempty(K) == 0 && size(A,1) ~= size(K,2)
        error('number of columns of K must match number of columns of A')
    elseif strcmp(drpmdl,'lngtrans')==0 && strcmp(drpmdl,'explicit')==0
        error('Invalid dropout model specified, must be "lngtrans" or "explicit"')
    elseif strcmp(ovraprx,'JNF')==0 && strcmp(ovraprx,'CH')==0 && strcmp(ovraprx,'CHe')==0
        error('Invalid overapproximation method specified, must be "JNF", "CH" or "CHe"')    
    elseif strcmp(drpmdl,'explicit') && strcmp(lyap,'common')
        error('Must use "pardep" lyap function with "explicit" dropout modeling')   
    elseif strcmp(lyap,'common')==0 && strcmp(lyap,'pardep')==0
        error('Invalid lyap function type specified, must be "common" or "pardep"')
    elseif isnumeric(gamma) == 0 
        error('Invalid gamma specified. Must be in the range [0,1)')
    elseif gamma < 0 || gamma >= 1
        error('Invalid gamma specified. Must be in the range [0,1)')
    elseif h_min < tau_max
        error('h_min must be smaller than or equal to tau_max when using JNF or CH/2 methods')
    end

    %% Display NCS information

    disp('SYSTEM DATA:')
    disp(['  states     = ' num2str(size(A,2))])
    if h_min-h_max == 0
        disp(['  h          =  ' num2str(h_min)])
    else
        disp(['  h          = [' num2str(h_min) ',  ' num2str(h_max) ']'])
    end
    if tau_min - tau_max == 0
        disp(['  tau        =  ' num2str(tau(1))]) 
    else
        disp(['  tau        = [' num2str(tau(1)) ',  ' num2str(tau(2)) ']']) 
    end

    disp(['  max drops  = ' num2str(deltaBar)])

    %% Determination of Jordan Form of A
    syms hk hktk                       % symbols used in the exponential and integral terms

    [J Q] = rjd(A);                 % determine the real Jordan form with the Linear Systems Toolbox
    JJ = JordanBlocks(J);           % Determination of separate Jordan blocks with real eigenvalues


    %% Compute Exponential and Integral Terms in NCS Model
    S11 = cell(0);
    alfa11  = cell(0);

    q = 1; r = 1;
    for i = 1:size(JJ,2)
        % compute e^(JJ{i}*hk) = sum(S1{i}*alfa1{i})
        [S1 alfa1] = exponent(JJ{i});                    % Determination of varying factors from exponent

        % add the part e^(JJ{i}*hk) = sum(S1{i}*alfa1{i}) into e^(A*hk) = sum(S11{i}*alfa11{i})
        q = q + size(JJ{i},1);
        for j = 1:size(S1,2)
            S11{r} = Q*blkdiag(zeros(q-1-size(JJ{i},1)),S1{j},zeros(size(J,1)-q+1))/Q;
            alfa11{r} = alfa1{j};
            r = r+1;
        end
    end

    % Transform S11,alfa11 to Cayley-Hamilton terms
    if strcmp(ovraprx,'CH') || strcmp(ovraprx,'CHe')
        [S11 alfa11] = Cayley(A,S11,alfa11);                   
    end

    % compute int_hktk^hk(e^{A*s}ds)= sum(S22{i}*alfa22{i}) using S11 and alfa11 computed above
    S22 = cell(1,2*size(S11,2));
    alfa22  = cell(1,2*size(S11,2));
    if rank(A) ~= size(A,2)                     % A not invertible
         for ctr = 1:size(S11,2)
            alfa22{ctr} = int(alfa11{ctr},hk);                                 %need to use int() since Cayley can have terms like f1 + f2*hk^2
            S22{ctr} = S11{ctr};
            alfa22{ctr+size(S11,2)} = subs(alfa22{ctr},hk,hktk,0);    %need to use int() since Cayley can have terms like f1 + f2*hk^2
            S22{ctr+size(S11,2)} = -S11{ctr};
        end   
    else                                        % A invertible,use int_hktk^hk(e^{A*s}ds) = inv(A)*int(e^A*hk) - inv(A)*int(e^A*hktk)
        for ctr = 1:size(S11,2)
            alfa22{ctr} = alfa11{ctr};
            S22{ctr} = A\S11{ctr};
            alfa22{ctr+size(S11,2)} = subs(alfa11{ctr},hk,hktk,0);
            S22{ctr+size(S11,2)} = -A\S11{ctr};
        end
    end

    % compute int_0^hktk(e^{A*s}ds)= sum(S33{i}*beta33{i}) using S22 and
    % alfa22 computed above    
    alfa33 = cell(1,size(alfa22,2));       %preallocate for better preformance
    S33 = cell(1,size(alfa22,2));          %preallocate for better preformance

    temp = subs([alfa22{:}],hktk,0,0);
    temp = subs(temp,hk,hktk,0);
    ctr = 1;
    for j = 1:size(temp,2)
        if temp(j) ~= 0
            alfa33{ctr} = temp(j);
            S33{ctr} = S22{j};
            ctr = ctr+1;
        end
    end

    S33 = S33(1,1:ctr-1);         %resize vector by index ctr
    alfa33 = alfa33(1,1:ctr-1);   %resize vector by index ctr

    %% Combine alfa_i*S_i Exponential and Integral terms to create overapproximation F,G matrices

    [alfa_ht,F_ht,G_ht] = check_alpha(alfa11,alfa22,alfa33,S11,S22,S33,B); 

    %% Build Closed-Loop Overapproximation

    disp(' ')
    disp('OVERAPPROXIMATION DATA:')
    %disp('eigenvalues of A = ')
    %disp(num2str(eig(A)'))
    disp(['unique alfa terms = ' num2str(size(alfa_ht,2))])
    %for ctr = 1:size(alfa_ht,2)
    %    disp(['   alpha' num2str(ctr) ' = ' char(alfa_ht(ctr))]);
    %end
    disp(' ')
    totalVerts = 0;
    
    switch drpmdl
       case 'explicit' 
            if deltaBar > 0
                DropModes = 1;
            else
                DropModes = 0;
            end       
            OvrAprxVert = cell(1,2);
       case 'lngtrans'  
           DropModes = deltaBar;
           OvrAprxVert = cell(1,deltaBar+1);
    end
    %------------------------------------------------------------------
    %------------------------------------------------------------------
    for DropMode = 0:DropModes
        
        %------------------------------------------------------------------
        % find min and max of alfa terms
        switch drpmdl
            case 'explicit'  
                %DropMode = 0 : First build polytopic system that does NOT include dropouts
                %DropMode = 1 : Build polytopic system which data DOES drop (ie open loop)
                
                if DropMode == 0
                    disp('   Mode: Packet Received')
                else
                    disp('   Mode: Packet Dropped')
                end
                
                if DropMode == 1 
                    % in this case, we need sum(alfa22{i}*S22{i}), which equals
                    % int_hktk^hk(e^{JJ{i}*s}ds), to now equal
                    % int_0^hk(e^(A*s)ds. So we will simply take sum(S33{i}*alfa33{i}) which
                    % equals int_0^hktk(e^{A*s}ds) and replace the sym hktk with hk
                    alfa44=cell(0);
                    S44= cell(0);
                    for j = 1:(size(alfa33,2))
                        alfa44{j} =subs(alfa33{j},hktk,hk);             % phi(k+1) = [expm(A*h) int(expm(A*s),s,0,h); zeros(n) 1]*phi(K)
                        S44{j} = S33{j};
                    end
                    [alfa_ht,F_ht] = check_alpha(alfa11,alfa44,[],S11,S44,[],B); %recombine and reduce new alfa_i*S_i multipliers
                end
                
                % find min and max of alfa terms
                [F,F0,G,G0,alfaComboSet,alfaMinMax] = getAlfaMinMax(alfa_ht,F_ht,G_ht,h,tau);
                nu = size(alfaMinMax,2);
                disp(['      Number of varying alfa terms = ' num2str(nu)])

                % add terms in NCS model which are not overapproximated
                if DropMode == 0
                    G0 = G0+[zeros(size(B));eye(size(B,2))];         
                elseif DropMode == 1
                    F0 = F0+blkdiag(zeros(size(A)),eye(size(B,2)));
                end              

            case 'lngtrans'     
                % DropMode = 0:deltaBar : Create a polytopic overapproximation of the NCS for each DropMode

                disp(['   Mode: ' num2str(DropMode) ' Drops'])

                %modify the transmission interval
                hBound = [(DropMode+1)*h_min (DropMode+1)*h_max];
                tBound = [(DropMode+1)*tau_min (DropMode+1)*tau_max];
               
                % find min and max of alfa terms
                [F,F0,G,G0,alfaComboSet,alfaMinMax] = getAlfaMinMax(alfa_ht,F_ht,G_ht,hBound,tBound);
                nu = size(alfaMinMax,2);
                disp(['      Number of varying alfa terms = ' num2str(nu)])

                % add terms in NCS model which are not overapproximated
                G0 = G0+[zeros(size(B));eye(size(B,2))];           
        end  

        %----------------------------------------------------------------
        % Construct open-loop vertices of overapproximation
        switch ovraprx
            case {'JNF' 'CH'}
                % Using alfaComboSet gives 2^(2*nu) vertices of the polytope
                HF = cell(1,size(alfaComboSet,2));
                HG = cell(1,size(alfaComboSet,2));
                if size(alfaComboSet,2)==0
                    HF{1} = F0;
                    HG{1} = G0;        
                else
                    for i = 1:size(alfaComboSet,2)
                            HF{i} = F0;
                            HG{i} = G0;         
                        for j = 1:size(alfaComboSet{i},2)                       %multiply matrices Fi and Gi by all values alfaComboSet{i} to make a polytopic vertex
                            HF{i} = HF{i} + alfaComboSet{i}(j)*F{j};     
                            HG{i} = HG{i} + alfaComboSet{i}(j)*G{j}; 
                        end
                    end
                end
            case 'CHe'
                % Using the alfaMinMax output results in reduced 2*2*nu vertices of the polytope
                HF = cell(1,size(alfaMinMax,1)*size(alfaMinMax,2));
                HG = cell(1,size(alfaMinMax,1)*size(alfaMinMax,2));

                if size(alfaMinMax,2) ==0   %nu = 0
                    HF{1} = F0;
                    HG{1} = G0;
                else
                    p=1;
                    for i = 1:size(alfaMinMax,1)
                        for j = 1:size(alfaMinMax,2)
                            HF{p} = F0+(nu+1)*(alfaMinMax(i,j)*F{j});
                            HG{p} = G0+(nu+1)*(alfaMinMax(i,j)*G{j});
                            p = p+1;
                        end
                    end
                end
        end
        
        %-----------------------------------------------------------------
        % Controller synthesis without considering dropouts
        if size(K,1) == 0
            switch lyap
                case 'pardep'
                    K = PDLF(HF,HG,gamma);
                case 'common'
                    K = CQLF(HF,HG,gamma);  
            end
        end
        Kex = [K zeros(size(B,2))];
        
        %------------------------------------------------------------------
        % Create closed-loop vertices of NCS

        % Total system equation
        for i = 1:size(HF,2)
           if DropMode == 0 || strcmp(drpmdl,'lngtrans')                   % Use K to create closed loop system
                OvrAprxVert{DropMode+1}{i} = HF{i} - HG{i}*Kex;                     
           elseif DropMode == 1 && strcmp(drpmdl,'explicit')               % do not use HG term, only need an HF term when hybrid model is in drop mode
                OvrAprxVert{2}{i} = HF{i};                  
           end
        end

        %count the number of polytopic vertices
        disp(['      Number of Polytopic Vertices = ' num2str(size(OvrAprxVert{DropMode+1},2))])
        totalVerts = totalVerts + size(OvrAprxVert{DropMode+1},2);
                
    end


%% LMI Stability

    disp(' ')
    disp('STABILITY DATA:')

    switch drpmdl
        case 'explicit'
             disp('   Lyapunov function type: Switched Quadratic')
             expectedNumLMIs = (deltaBar+1)*size(OvrAprxVert{1},2)+deltaBar*size(OvrAprxVert{2},2)+(deltaBar+1);
             disp(['      Number of LMIs needed to verify stabiliy = ' num2str(expectedNumLMIs)])
            [stable, LMIs] = LMI(OvrAprxVert,deltaBar,gamma);
        case 'lngtrans'
            switch lyap
                case 'common'
                    disp('   Lyapunov function type: Common Quadratic')
                    expectedNumLMIs = totalVerts + 1;
                    disp(['      Number of LMIs needed to verify stabiliy = ' num2str(expectedNumLMIs)])
                    [stable, LMIs] = LMI_common(OvrAprxVert,gamma);
                case 'pardep'
                    disp('   Lyapunov function type: Switched Quadratic')
                    expectedNumLMIs = (deltaBar+1)*totalVerts + (deltaBar+1);
                    disp(['      Number of LMIs needed to verify stabiliy = ' num2str(expectedNumLMIs)])
                    [stable, LMIs] = LMI_dependent(OvrAprxVert,deltaBar,gamma);
            end
    end

    if size(LMIs) ~= expectedNumLMIs
       error('Number of LMIs solved for is incorrect!') 
    end

    if stable ==1
        disp('      Stability: Guaranteed')
    else
        disp('      Stability: Not Guaranteed')
    end
    fprintf('-------------------------------------------------------------- \n')