function K = CQLF(A,B,gamma)
% CQLF      Controller synthesis for polytopic systems using a common 
%           quadratic Lyapunov function (CQLF)
% 
%   K = CQLF(A,B,N,nk,gamma) determines a controller K that stabilizes the
%   system using a common quadratic Lyapunov function V = x'*P*x. In this 
%   function, the cells A and B are the overapproximated matrices of the
%   system matrices A and B respectively; N is the number of vertices of the
%   overapproximations; nk is the size of the state feedback \bar{K} = [K 0];
%   gamma is the decay rate of the Lyapunov function.


%% Vertex  dimensions 
[n,m] = size(B{1});
N = size(A,2);
nk = size(A{1},1) - size(B{1},2);


%% LMI variables
X1 = sdpvar(nk,nk,'full');
X2 = zeros(nk,m);

Z1 = sdpvar(m,nk);
Z2 = zeros(m,m);


%%  Common Lyapunov Functions
Y = sdpvar(n,n);
I = eye(n,n);
F = set(Y-I > 0); 


%%  LMI conditions SF-CQLF from Marieke [Synthesis, 2008]

X3 =cell(1,N);
X4 =cell(1,N);
X =cell(1,N);

for i = 1:N
    X3{i} = sdpvar(m,nk,'full');
    X4{i} = sdpvar(m,m,'full');
    X{i} = [X1 X2; X3{i} X4{i}];
    
	F = F + set([X{i} + X{i}' - Y           X{i}'*A{i}' - [Z1 Z2]'*B{i}';
                 A{i}*X{i} - B{i}*[Z1 Z2]   (1-gamma)*Y                 ] > 0);
end


%% LMI resolution
solvesdp(F,[],sdpsettings('verbose',0));

for i = 1:N
    X{i} = double(X{i});
end

X1 = double(X1);
Z1 = double(Z1);
Y = double(Y);

K = Z1/X1;    %Z1*inv(X1) = Z1/X1


%% Verification
invalidK = 0;
for i = 1:N
    LMIs =  [X{i} + X{i}' - Y           X{i}'*A{i}' - [Z1 Z2]'*B{i}';  
             A{i}*X{i} - B{i}*[Z1 Z2]   (1-gamma)*Y                 ];
    eigLMI = min(real(eig(LMIs)));
    
    if eigLMI <= 0
        invalidK = 1;
    end
    
    for k = 1:max(size(LMIs))
        minor = det(LMIs(1:k,1:k));
        if minor <= 0
            invalidK = 1;
        end
    end
end

eigenval = min(eig(Y-I));
if eigenval <= 0 || sum(sum(Y - Y')) ~= 0 
    invalidK = 1;
end

for j = 1:max(size(Y))
    minor = det(Y(1:j,1:j)-I(1:j,1:j));
    if minor <= 0
        invalidK = 1;
    end
end
