function K = PDLF(A,B,gamma)
% PDLF      Controller synthesis for polytopic systems using a parameter 
%           dependent quadratic Lyapunov function (PDLF)
% 
%   K = PDLF(A,B,gamma) determines a controller K that stabilizes the
%   system using a parameter dependent quadratic Lyapunov function 
%   V = x'*P(i)*x. In this function: cells A and B are the overapproximated 
%   matrices of the system matrices A and B respectively; gamma is the decay 
%   rate of the Lyapunov function.


%% Vertex  dimensions 
[n,m] = size(B{1});
N = size(A,2);
nk = size(A{1},1) - size(B{1},2);


%% LMI variables
X1 = sdpvar(nk,nk,'full');
X2 = zeros(nk,m);

Z1 = sdpvar(m,nk);
Z2 = zeros(m,m);


%%  Parameter dependent Lyapunov Functions
F = set([]);
I = eye(n,n);
Y =cell(1,N);

for i = 1:N
    Y{i} = sdpvar(n,n); 
    F = F + set(Y{i}-I > 0);
end


%%  LMI conditions SF-CQLF from Marieke [Synthesis, 2008]

X3 =cell(1,N);
X4 =cell(1,N);
X =cell(1,N);

for i = 1:N
    X3{i} = sdpvar(m,nk,'full');
    X4{i} = sdpvar(m,m,'full');
    X{i} = [X1 X2; X3{i} X4{i}];
    
    for j = 1:N
        F = F + set([X{i} + X{i}' - Y{i}           X{i}'*A{i}' - [Z1 Z2]'*B{i}';
                     A{i}*X{i} - B{i}*[Z1 Z2]      (1-gamma)*Y{j}              ] > 0);
    end
end


%% LMI resolution
solvesdp(F,[],sdpsettings('verbose',0));

P =cell(1,N);

for i = 1:N
    X{i} = double(X{i});
    Y{i} = double(Y{i});
    P{i} = inv(Y{i});
end

X1 = double(X1);
Z1 = double(Z1);

K = Z1/X1;    %Z1*inv(X1) = Z1/X1    


%% Verification
invalidK = 0;
for i = 1:N
    for j = 1:N
        LMIs =  [X{i} + X{i}' - Y{i}        X{i}'*A{i}' - [Z1 Z2]'*B{i}';  
                 A{i}*X{i} - B{i}*[Z1 Z2]   (1-gamma)*Y{j}              ];
        
        eigLMI = min(real(eig(LMIs)));

        if eigLMI <= 0
            invalidK = 1;
        end

        for k = 1:max(size(LMIs))
            minor = det(LMIs(1:k,1:k));
            if minor <= 0
                invalidK = 1;
            end
        end
    end
    
    eigenval = min(eig(Y{i}-I));
    if eigenval <= 0 || sum(sum(Y{i} - Y{i}')) ~= 0 
        invalidK = 1;
    end
    
    for k = 1:max(size(Y{i}))
        minor = det(Y{i}(1:k,1:k)-I(1:k,1:k));
        if minor <= 0
            invalidK = 1;
        end
    end
end
