function [Ai f] = Cayley(A,S,alfa)
%CAYLEY     Determines new varying functions using the Cayley-Hamilton
%           theorem
%
%   [f Ji] = Cayley(J,S,alfa) determines the Cayley-Hamilton multipliers for Jordan block J
%   and using e^(J*hk) = sum(S{i}*alfa{i}) according to the Cayley-Hamilton theorem. In 
%   equation form this means:
% 
%       sum(alfa_i*S_i) = sum(f_j*J^(n-1))
% 
%   for i = 1:max(size(alfa)) and j = 0:n-1, where n is the dimension of
%   the system.
%   The inputs of this function are the Jordan form J of the system matrix,
%   constant matrices S and time-varying functions alfa. 

%compute new S terms for Cayley-Hamilton

Ai = cell(0);                             % preallocate for better performance
for i = 1:size(S,2)               
    Ai{i} = A^(i-1);
end

matA = zeros(size(A,1)*size(A,1),size(Ai,2));      % preallocate for better performance
for j = 1:size(Ai,2)
    matA(:,j) = vec(Ai{j});                         %create a matrix whose columns are columns of A^{j-1}
end

% solve for vector beta in problem matJ*beta = vec(S{i})
beta = cell(1,size(Ai,2));                      % preallocate for better performance
for i = 1:size(Ai,2)               
    beta{i} =matA\(vec(S{i}));
end


f = cell(1,size(Ai,2));                         % preallocate for better performance
for i = 1:size(Ai,2)               
    f{i} = 0;
    for j = 1:size(Ai,2)
        f{i} = f{i} + beta{j}(i)*alfa{j};          % f  = sum beta{j}(i)*alfa{j}
    end
end