function [alfa_ht,F_ht,G_ht] = check_alpha(alfa11,alfa22,alfa33,S11,S22,S33,Bc)
%CHECK_ALPHA	Determines one cell of uncertainty terms
%
%   [alfa_ht,F_ht,G_ht] = check_alpha(alfa11,alfa22,alfa33,S11,S22,S33,Bc) 
%   uses all previously determined cells alfa11, alf22 and alfa33 with 
%   uncertainty functions, to determine one cell alfa_ht with all uncertainty 
%   terms. First we define
%
%    L11{i} = [S11{i} 0] 
%             [ 0     0]
%    L22{i} = [0   S22{i}*Bc]
%             [0       0    ]
%    L33{i} = [S33{i}*Bc]           
%             [    0    ]
%
%   so that they have the right form corresponding to the uncertainty terms 
%   in the NCS model. Then we define
%
%     alfa_ht = {alfa11,alfa22,alfa33}
%       F_ht  = {L11,L22,0}
%       G_ht  = {0,0,L33}
%
%   where 0 represent cells of zero matrices corresponding to
%   terms in alfa_ht. Next, duplicate uncertainty functions are eliminated.  
%   Meaning, if alfa_ht{i}=alfa_ht{j}, then F_ht{i} = F_ht{i}+F_ht{j} and 
%   G_ht{i} = G_ht{i}+G_ht{j} and the cells alfa_ht{j},F_ht{j} and G_ht{j} 
%   are removed.  Therefore, each uncertainty term appears only once in the 
%   output alfa_ht.       

syms hk hktk

%% Resize the S11,S22,S33 matrices with zeros.
% After zero padding, the S11 and S22 matrices are the right place for 
% the overapproximated closed loop NCS A matrix (plus constant terms outside
% this function).

for i = 1:size(S11,2)
    S11{i} = blkdiag(S11{i},zeros(size(Bc,2)));
end

for i = 1:size(S22,2)
    S22{i} = [zeros(size(S22{i})) S22{i}*Bc ; zeros(size(Bc,2),size(S22{i},1)) zeros(size(Bc,2))];
end

% use S33 so that it is in the format for the overapproximated closed loop NCS B matrix

for i = 1:size(S33,2)
    S33{i} = [S33{i}*Bc ; zeros(size(Bc,2))];
end   


%% Create alfa_ht, F_ht and G_ht
% Also remove duplicate multipliers

% stack all the alfa terms
alfaStack = [alfa11 alfa22 alfa33];

% create F_stack and G_stack which will be reduced
if isempty(S33) == 0           
    zeroPad = cell(1,size(S33,2));
    for ctr = 1:size(S33,2)
       zeroPad{ctr}  = zeros(size(S22{1})); 
    end

    F_stack = [S11 S22 zeroPad];

    zeroPad = cell(1,size(S11,2)+size(S22,2));
    for ctr = 1:size(S11,2)+size(S22,2) 
      zeroPad{ctr}  = zeros(size(S33{1})); 
    end

    G_stack = [zeroPad S33];
else                                        % for the case for Explicit Drop Modeling, no S33 is given
    F_stack = [S11 S22];
    G_stack = cell(1,size(S11,2)+size(S22,2));
    for ctr = 1:size(S11,2)+size(S22,2) 
      G_stack{ctr}  = 0; 
    end
end

alfa_ht=sym(0);                         %preallocate for better performance
F_ht=cell(1,size(F_stack,2));           %preallocate for better performance
G_ht=cell(1,size(G_stack,2));           %preallocate for better performance

% Eliminate duplicate alfa terms in alfa_stack.  Add F_stack and G_stack elements if alfa's are identical
r=1;
for p = 1:size(alfaStack,2)
    if alfaStack{p} ~= 0    
        alfa_ht(r) = alfaStack{p};
        F_ht{r} = F_stack{p};
        G_ht{r} = G_stack{p};
        for m = p+1:size(alfaStack,2)                     % compare  pth  element to rest of elements
            if alfaStack{m}-alfaStack{p} == 0             % if symbols are equal
                F_ht{r} = F_ht{r} + F_stack{m};           % add matrices of similar multipliers
                G_ht{r} = G_ht{r} + G_stack{m};           % add matrices of similar multipliers
                alfaStack{m} = 0;                         % set dupiclate multiplier to zero
            end
        end
        r = r+1;
    end
end

F_ht=F_ht(1,1:r-1);           %resize to index r
G_ht=G_ht(1,1:r-1);           %resize to index r


