function [F,F0,G,G0,alfaMinMaxCombos,alfaMinMax] = getAlfaMinMax(alfa,Ah,Bh,h,tau)
%GETALFAMINMAX      Determine generators of overapproximation
% 
%   [F,F0,G,G0,alfaMinMaxCombos,alfaMinMax] = getAlfaMinMax(alfa,Ah,Bh,h,tau)
%   calculates the min and max for each function alfa which is not
%   constant. If h or tau or both are not varying, they are substituted into the
%   symbolic expression alfa.
%   F0 = sum alfa(i)*Ah{i} with alfa constant. 
%   G0 = sum alfa(i)*Bh{i} with alfa constant. 
%   F and G are matrices corresponding to nonconstant alfa terms. 
%   alfaMinMaxCombos contains all possible combinations of the lower and upper 
%   bounds on the uncertainty terms in alfa.  
%   alfaMinMax is a list of of the min and max values for each alfa function 
%   and is used for CH2.


syms hk hktk tk

alfa_var = sym(0);                      %preallocate for better performance
F = cell(1:size(alfa,2));               %preallocate for better performance
G = cell(1:size(alfa,2));               %preallocate for better performance

if h(1) - h(2) ==0
    alfa = subs(alfa,hktk,h(1)-tk,0);         %FIRST substitute 'h(1)-tk' into symbol hktk (constant sampling)
    alfa = subs(alfa,hk,h(1),0);              %THEN substitute value of hk into symbol hk (constant sampling)
end

if tau(1) - tau(2)  ==0
    alfa = subs(alfa,hktk,hk-tau(1),0);        %FIRST substitute 'hk-tau(1)' into symbol hktk (constant sampling)
    alfa = subs(alfa,tk,tau(1),0);             %THEN substitute tau(1) into symbol tk (constant delay)
end
    
%% ------------------------------------------------------------------------
%split alfa, Ah, and Bh into constant parts and varying parts
k = 1; 
z = 1;
F0 = 0; 
G0 = 0;
for i = 1:size(alfa,2)
    if diff(alfa(i),hk) == 0 && diff(alfa(i),tk) == 0 && diff(alfa(i),hktk) == 0       %constant terms    
        F0 = F0 + double(alfa(i))*Ah{i};
        G0 = G0 + double(alfa(i))*Bh{i};
        k = k+1;
    else                    %varying alfa terms
        alfa_var(z) = alfa(i);
        F{z} = Ah{i};
        G{z} = Bh{i};
        z = z+1;
    end
end

alfa_var = alfa_var(1,1:z-1);   %resize by index z
F = F(1,1:z-1);                 %resize by index z
G = G(1,1:z-1);                 %resize by index z

%% --------------------------------------------------------------------------
% compute min and max of alfa_i of the varying part and store in alfaMinMax where 
% alfa_i_max = alfaMinMax(2,i) and alfa_i_min = alfaMinMax(1,i)
% this comuptation is done by gridding the function alfa_var(i) and
% choosing the max value 
alfaMinMax = zeros(2,size(alfa_var,2));

gridPoints = 1000;
hBound = h(1):(h(2)-h(1))/(gridPoints-1):h(2);
tBound = tau(1):(tau(2)-tau(1))/(gridPoints-1):tau(2);
htBound = h(1)-tau(2):(h(2)-tau(1) - (h(1)-tau(2)))/(gridPoints-1):h(2)-tau(1);

v=version;  % check matlab version, fmincon changed depending on version
if str2double(v(3:4))>=10    %10 = r2010b
    options=optimset('Display','off','Algorithm','sqp','LargeScale','off');    %'sqp' only in r2010a and later
elseif str2double(v(3:4))>=7  %7 = r2008b
    options=optimset('Display','off','Algorithm','active-set','LargeScale','off');
end
% versions earlier than r2008b cannot use fmincon since 'matlabFunction' is not defined

for k = 1:size(alfa_var,2)
    impl = alfa_var(k);
    % need 'if' structure since if h or tau is constant, then hBound or tBoutnd is not defined
    if diff(alfa_var(k),hktk) ~= 0
        impl = subs(impl,hktk,htBound,0);                                                       % grid function by substututing htBound into sym hktk 
        
        startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
        startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
        if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined
            alfaFunct = matlabFunction(alfa_var(k));                                                % create matlab function from symbolic expression (only for r2008b and newer)
            minPt = fmincon(alfaFunct,startMin,[],[],[],[],htBound(1),htBound(end),[],options);     % search for local min given starting point
            minAlfaFunct = matlabFunction(-alfa_var(k));                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
            maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],htBound(1),htBound(end),[],options);  % search for local max given starting point
            
            alfaMinMax(1,k) = subs(alfaFunct,hktk,minPt);                                           % evaluate value given by fmincon
            alfaMinMax(2,k) = subs(alfaFunct,hktk,maxPt);                                           % evaluate value given by fmincon
        else
            alfaMinMax(1,k) = startMin;
            alfaMinMax(2,k) = startMax;
        end
        
    elseif diff(alfa_var(k),hk) ~= 0
        impl = subs(impl,hk,hBound,0);                                                          % grid function by substututing hBound into sym hk
        
        startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
        startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
        if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined 
            alfaFunct = matlabFunction(alfa_var(k));                                                % create matlab function from symbolic expression (only for r2008b and newer)
            minPt = fmincon(alfaFunct,startMin,[],[],[],[],hBound(1),hBound(end),[],options);       % search for local min given starting point
            minAlfaFunct = matlabFunction(-alfa_var(k));                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
            maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],hBound(1),hBound(end),[],options);    % search for local max given starting point
            
            alfaMinMax(1,k) = subs(alfaFunct,hk,minPt);                                             % evaluate value given by fmincon
            alfaMinMax(2,k) = subs(alfaFunct,hk,maxPt);                                             % evaluate value given by fmincon
        else
            alfaMinMax(1,k) = startMin;
            alfaMinMax(2,k) = startMax;
        end        

    elseif diff(alfa_var(k),tk) ~= 0
        impl = subs(impl,tk,tBound,0);                                                          % grid function by substututing tBound into sym tk
        
        startMin = min(double(impl));                                                           % take min as starting point for local min algorithm
        startMax = max(double(impl));                                                           % take max as starting point for local max algorithm
        if str2double(v(3:4))>=7  %7 = r2008b, cannot use fmincon since 'matlabFunction' is not defined
            alfaFunct = matlabFunction(alfa_var(k));                                                % create matlab function from symbolic expression (only for r2008b and newer)
            minPt = fmincon(alfaFunct,startMin,[],[],[],[],tBound(1),tBound(end),[],options);       % search for local min given starting point
            minAlfaFunct = matlabFunction(-alfa_var(k));                                            % create matlab function from symbolic expression.  Make it negative so fmincon solves for max.
            maxPt = fmincon(minAlfaFunct,startMax,[],[],[],[],tBound(1),tBound(end),[],options);    % search for local max given starting point
            
            alfaMinMax(1,k) = subs(alfaFunct,tk,minPt);                                             % evaluate value given by fmincon
            alfaMinMax(2,k) = subs(alfaFunct,tk,maxPt);                                             % evaluate value given by fmincon
        else
            alfaMinMax(1,k) = startMin;
            alfaMinMax(2,k) = startMax;
        end         
    end
   
end     

%% ------------------------------------------------------------------------
% use the values computed in alfaMinMax to construct alfaMinMaxCombos{i}, which is a cell
% of vectors.  The set of these vectors is all combinations alfa_min and 
% alfa_max 
%
% The size(alfaMinMaxCombos,2) is the number of polytopic vertices, and each vector alfaMinMaxCombos{i}
% corresponds to the mulipliers to create a single vertex

alfaMinMaxCombos = cell(0);     %preallocate for better performance

if size(alfaMinMax,2) == 1          %only one alfa_i
    alfaMinMaxCombos{1} = alfaMinMax(1,:);
    alfaMinMaxCombos{2} = alfaMinMax(2,:);
elseif size(alfaMinMax,2) > 1

    % use upper and lower cells as matrix sets to get all combinations 
    % of the min and max of alfa_i 

    lower = cell(1,size(alfaMinMax,2));                     %preallocate for better performance
    upper = cell(1,size(alfaMinMax,2));                     %preallocate for better performance

    for i = 1:size(alfaMinMax,2)
        lower{i} = nchoosek(1:size(alfaMinMax,2),i);
        for j = 1:size(lower{i},1)
            upper{i}(j,:) = setdiff(1:size(alfaMinMax,2),lower{i}(j,:));
        end
    end

    z = 1;
    for i = 1:size(lower,2)
        for j = 1:size(lower{i},1)
            alfaMinMaxCombos{z}(1,lower{i}(j,:)) = alfaMinMax(1,lower{i}(j,:));
            alfaMinMaxCombos{z}(1,upper{i}(j,:)) = alfaMinMax(2,upper{i}(j,:));
            z = z+1;
        end
    end
    alfaMinMaxCombos{end+1} = alfaMinMax(2,:);
end
