function [stable L P] = LMI(OvrAprxVert,deltaBar,gamma)
%LMI        Stability analysis using linear matrix inequalities (LMIs)
%           that result from a dropout-dependent Lyapunov function
% 
%   [P LMIs stable] = LMI(OvrAprxVert,deltaBar,gamma) checks the stability of the
%   overapproximated system with generators OvrAprxVert. The Lyapunov function that
%   is used is:
% 
%       V(x_k) = x_k^T * P_i * x_k
%       V(x_k) > I
% 
%   To guarantee asymptotic stability, the derivative of the Lyapunov 
%   function should be negative difinite:
% 
%       V(x_k+1)-V(x_k) = x_k^T * (OvrAprxVert{i}{j}^T * P_i * OvrAprxVert{i}{j} - P_j) * x_k
%       V(x_k+1)-V(x_k) < -gamma*V(x_k)
%   
%   for i = 1:deltaBar and j = 1:deltaBar, where not all combinations are
%   possible because of the automaton approach. 
%   These Lyapunov functions are transformed into LMIs (by removing x_k and
%   x_k^T) that are checked if they satisfy their conditions. If they do, 
%   the system is asymptotically stable; if they do not, the system is 
%   unstable.

L = set([]);
I = eye(size(OvrAprxVert{1}{1},1));

Z = cell(1,deltaBar+1);

for i = 1:deltaBar+1
    Z{i} = sdpvar(size(OvrAprxVert{1}{1},1));
    L = L + set(Z{i}-I > 0);
end

for i = 1:size(OvrAprxVert{1},2)
    A0 = OvrAprxVert{1}{i};
    for j = 1:deltaBar+1
        L = L + set(A0'*Z{1}*A0-Z{j}+gamma*Z{j} < 0);
    end
end

if deltaBar > 0
    for i = 1:size(OvrAprxVert{2},2)
        A1 = OvrAprxVert{2}{i};
        for j = 1:deltaBar
            L = L + set(A1'*Z{j+1}*A1-Z{j}+gamma*Z{j} < 0);
        end
    end
end

%solve LMIs
feas = solvesdp(L,[],sdpsettings('verbose',0));
[primalRes,dualRes] = checkset(L);
disp(['      ' feas.info])
%disp(['         primal residuals lie in [' num2str(min(primalRes)) ', ' num2str(max(primalRes)) ']'])
%disp(['         dual residuals lie in [' num2str(min(dualRes)) ', ' num2str(max(dualRes)) ']'])
% Extract P
P = cell(1,size(Z,2));
for i = 1:size(Z,2)
    P{i} = double(Z{i});
end

%% Check LMIs
p = 1;
LMIs = cell(1,size(OvrAprxVert{1},2));

for i = 1:size(OvrAprxVert{1},2)
    A0 = OvrAprxVert{1}{i};
    for j = 1:deltaBar+1
        LMIs{p} = A0'*P{1}*A0-P{j}+gamma*P{j};
        p = p+1;
    end
end

if deltaBar > 0
    for i = 1:size(OvrAprxVert{2},2)
        A1 = OvrAprxVert{2}{i};
        for j = 1:deltaBar
            LMIs{p} = A1'*P{j+1}*A1-P{j}+gamma*P{j};
            p = p+1;
        end
    end
end

stable = 1;
for i = 1:max(size(LMIs))
    eigenval = max(eig(LMIs{i}));
    if eigenval >= 0
        stable = 0;
    end
    
    for j = 1:max(size(LMIs{i}))
        minor = det(-LMIs{i}(1:j,1:j));
        if minor <= 0
            stable = 0;
        end
    end
end

for i = 1:size(P,2)
    eigenval = min(eig(P{i}-I));
    if eigenval <= 0 || sum(sum(P{i} - P{i}')) ~=0
        stable = 0;
    end
    
    for j = 1:max(size(P{i}))
        minor = det(P{i}(1:j,1:j)-I(1:j,1:j));
        if minor <= 0
            stable = 0;
        end
    end
end

