function [stable L P] = LMI_common(OvrAprxVert,gamma)
%LMI_COMMON     Stability analysis using linear matrix inequalities (LMIs)
%               that result from a common quadratic Lyapunov function
% 
%   [P LMIs stable] = LMI_common(OvrAprxVert,gamma) checks the stability of the
%   overapproximated system with generators OvrAprxVert. The Lyapunov function that
%   is used is:
% 
%       V(x_k) = x_k^T * P * x_k
%       V(x_k) > I
% 
%   To guarantee asymptotic stability, the derivative of the Lyapunov 
%   function should be negative difinite:
% 
%       V(x_k+1)-V(x_k) = x_k^T * (OvrAprxVert{i}{j}^T * P * OvrAprxVert{i}{j} - P) * x_k
%       V(x_k+1)-V(x_k) < -gamma*V(x_k)
%                                                                  
%   These Lyapunov functions are transformed into LMIs (by removing x_k and
%   x_k^T) that are checked if they satisfy their conditions. If they do, 
%   the system is asymptotically stable; if they do not, the system is 
%   unstable.

Z = sdpvar(size(OvrAprxVert{1}{1},1));
I = eye(size(OvrAprxVert{1}{1},1));
L = set(Z-I > 0);

for i = 1:size(OvrAprxVert,2)
    for j = 1:size(OvrAprxVert{i},2)
        A = OvrAprxVert{i}{j};
        L = L + set(A'*Z*A-Z+gamma*Z < 0);
    end
end
%solve LMIs
feas = solvesdp(L,[],sdpsettings('verbose',0));
[primalRes,dualRes] = checkset(L);
disp(['      ' feas.info])
%disp(['         primal residuals lie in [' num2str(min(primalRes)) ', ' num2str(max(primalRes)) ']'])
%disp(['         dual residuals lie in [' num2str(min(dualRes)) ', ' num2str(max(dualRes)) ']'])
% Extract P
P = double(Z);

%% Check LMIs
p = 1;
LMIs = cell(1,size(OvrAprxVert{1},2));

for i = 1:size(OvrAprxVert,2)
    for j = 1:size(OvrAprxVert{i},2)
        A = OvrAprxVert{i}{j};
        LMIs{p} = A'*P*A-P+gamma*P;
        p = p+1;
    end
end

stable = 1;
for i = 1:max(size(LMIs))
    eigenval = max(eig(LMIs{i}));
    if eigenval >= 0
        stable = 0;
    end
    
    for j = 1:max(size(LMIs{i}))
        minor = det(-LMIs{i}(1:j,1:j));
        if minor <= 0
            stable = 0;
        end
    end
end

eigenval = min(eig(P-I));
if eigenval <= 0 || sum(sum(P - P')) ~=0
    stable = 0;
end

for j = 1:size(P,2)
    minor = det(P(1:j,1:j)-I(1:j,1:j));
    if minor <= 0
        stable = 0;
    end
end
