function [stable L P]  = LMI_dependent(OvrAprxVert,deltaBar,gamma)
%LMI_DEPENDENT      Stability analysis using linear matrix inequalities 
%                   (LMIs) that result from a dropout-dependent Lyapunov 
%                   function
% 
%   [P LMIs stable] = LMI_dependent(OvrAprxVert,deltaBar,gamma) checks the 
%   stability of the overapproximated system with generators OvrAprxVert. The 
%   Lyapunov function that is used is:
% 
%       V(x_k) = x_k^T * P_i * x_k
%       V(x_k) > I
% 
%   for i = 1:deltaBar+1.
%   To guarantee asymptotic stability, the derivative of the Lyapunov 
%   function should be negative difinite:
% 
%       V(x_k+1)-V(x_k) = x_k^T * (OvrAprxVert{i}{j}^T * P_j * OvrAprxVert{i}{j} - P_i) * x_k
%       V(x_k+1)-V(x_k) < -gamma*V(x_k)
% 
%   for all combinations for i = 1:size(OvrAprxVert,2) and j = 1:deltaBar+1.
%   These Lyapunov functions are transformed into LMIs (by removing x_k and
%   x_k^T) that are checked if they satisfy their conditions. If they do, 
%   the system is asymptotically stable; if they do not, the system is 
%   unstable.

if deltaBar > 1
    
end

L = set([]);
I = eye(size(OvrAprxVert{1}{1},1));

Z = cell(1,deltaBar+1);
for i = 1:deltaBar+1
    Z{i} = sdpvar(size(OvrAprxVert{1}{1},1));
    L = L + set(Z{i}-I > 0);
end

for i = 1:size(OvrAprxVert,2)
    for j = 1:size(OvrAprxVert{i},2)
        A0 = OvrAprxVert{i}{j};
        for k = 1:deltaBar+1
            L = L + set(A0'*Z{k}*A0-Z{i}+gamma*Z{i} < 0);
        end
    end
end
%solve LMIs
feas = solvesdp(L,[],sdpsettings('verbose',0));
[primalRes,dualRes] = checkset(L);
disp(['      ' feas.info])
%disp(['         primal residuals lie in [' num2str(min(primalRes)) ', ' num2str(max(primalRes)) ']'])
%disp(['         dual residuals lie in [' num2str(min(dualRes)) ', ' num2str(max(dualRes)) ']'])
% Extract P
P = cell(1,max(size(Z)));
for i = 1:max(size(Z))
    P{i} = double(Z{i});
end


%% Check LMIs
p = 1;
LMIs = cell(1,size(OvrAprxVert{1},2));

for i = 1:size(OvrAprxVert,2)
    for j = 1:size(OvrAprxVert{i},2)
        A0 = OvrAprxVert{i}{j};
        for k = 1:deltaBar+1
            LMIs{p} = A0'*P{k}*A0-P{i}+gamma*P{i};
            p = p+1;
        end
    end
end

stable = 1;
for i = 1:max(size(LMIs))
    eigenval = max(eig(LMIs{i}));
    if eigenval >= 0
        stable = 0;
    end
    
    for j = 1:max(size(LMIs{i}))
        minor = det(-LMIs{i}(1:j,1:j));
        if minor <= 0
            stable = 0;
        end
    end
end

for i = 1:max(size(P))
    eigenval = min(eig(P{i}-I));
    if eigenval <= 0 || sum(sum(P{i} - P{i}')) ~=0
        stable = 0;
    end
    
    for j = 1:max(size(P{i}))
        minor = det(P{i}(1:j,1:j)-I(1:j,1:j));
        if minor <= 0
            stable = 0;
        end
    end
end

