% ====================
% Authors: P.J.H. Maas, W.P.M.H. Heemels
% Date: March 19th, 2010
% Project: Sum of Squares methods for Stability Analysis of Networked
% % Control Systems
% Master Thesis
%
% Version 0.8
%
% File description:
% This function computes Hmati-Tmad tradeoff curves for maximum allowable
% transmission interval and maximum allowable delay in a Networked Control
% System, following the procedure in [1].
%
% Input:
% hncs class where
% Ap Bp Cp system matrices for plant dynamics in statespace format
% Dc system matrices for controller dynamics in statespace format
% Sy is vector with elements 1 or 0. 1 means connected via network, 0
% means connected directly
% 'RR' or 'TOD' for protocol
% l for number of nodes
%
% Syntax:
% [Hmati,Tmad] = maurice(hncs)
%
% Output:
% Hmati - Tmad tradeoff curve
%
% Required toolboxes
% Yalmip LMI parser (latest version)
% SeDuMi Solver (latest version)
%
% Version history
% V 0.1: basic scripts for calculating constants L_0, L_1, gamma_0, gamma_1
% V 0.2: implementing special structure for RR
% V 0.3: implemented ode solving
% V 0.4: implemented connections via network and directly by vectors Sy, Su
% V 0.4: implemented determination of maximum simulation time based on
% event triggering
% V 0.5: Ac strictly negative, number of nodes l <= number of error signals
% V 0.5.1: debuging test_LMI
% V 0.5.2: debugging test_P
% V 0.5.3: debugging eigenvalues A11, since Ac can be zero controller
%          states are not removed from the model
% V 0.6: adaptive stepsize in iterations in initial condition phi00
% V 0.7: sampled data is possible
% V 0.8: function setup
% V 0.9: static controller addaption
% V 1.0: Final
%
% ====================

function [Hmati,Tmad] = ncs_hyb_statefb(ncs,Su,Sy)


Ap = ncs.A;
Bp = ncs.B;
Cp = ncs.C;
Dc = ncs.K{1};
prot= ncs.protocol;
l = size(ncs.gammaU,2);   % number of nodes

if sum(Su) ~= length(Su)
    error('State feedback must have all actuators wired, i.e. each element in Su = 1')
end

%tic

% global variables (now in function call)
% global L_0 L_1 gamma_0 gamma_1 lambda phi10

% check sizes of system matrices
[mAp,nAp] = size(Ap);
[mBp,nBp] = size(Bp);
[mCp,nCp] = size(Cp);
% [mAc,nAc] = size(Ac);
% [mBc,nBc] = size(Bc);
% [mCc,nCc] = size(Cc);
[mDc,nDc] = size(Dc);
[mSy,nSy] = size(Sy);
% [mSu,nSu] = size(Su);

if (mSy ~= 1)
    Sy = Sy.';
    [mSy,nSy] = size(Sy);
end
% if (mSu ~= 1)
%     Su = Su.';
%     [mSu,nSu] = size(Su);
% end

if mAp ~= nAp
    error('Ap should be square');
elseif mAp ~= mBp
    error('dimensions Ap and Bp do not match');
elseif nAp ~= nCp
    error('dimensions Ap and Cp do not match');
% elseif mAc ~= nAc
%     error('Ac should be square');
% elseif mAc ~= mBc
%     error('dimensions Ac and Bc do not match');
% elseif nAc ~= nCc
%     error('dimensions Ac and Cc do not match');
% elseif mDc ~= mCc
%     error('dimensions Dc and Cc do not match');
% elseif nDc ~= nBc
%     error('dimensions Dc and Bc do not match');
elseif mDc ~= nBp
    error('dimensions Dc and Bp do not match');
elseif nDc ~= mCp
    error('dimensions Dc and Cc do not match');
elseif (mSy ~= 1)
    error('Sy should be a vector');
% elseif (mSu ~= 1)
%     error('Su should be a vector');
elseif nSy ~= mCp
    error('dimensions Sy and Cp do not match');
% elseif nSu ~= nBp
%     error('dimensions Su and Bp do not match');
    % elseif Ac >= 0  EIGENVALUES, not needed since A11 is tested later.
    % error('Ac should be negative definite, use small constant and make Cc
    % 0 for no controller state xc');
elseif l < 1
    error('There should be at least one node (sampled data)')
else
    %disp(strvcat('System matrices have matching dimensions',' '));
end

% ==========
% program calculating constants (L_0, L_1, gamma_0, gamma_1)
% ==========

% Calculate constants depending of protocol according to [1, Lemma V.4]
if strcmp(prot,'RR')
    lambda = sqrt((l-1)/l);
    lambdaW = sqrt(l);
    alpha_low = 1;
    alpha_high = sqrt(l);
    M1 = sqrt(l);
elseif strcmp(prot,'TOD')
    lambda = sqrt((l-1)/l);
    lambdaW = 1;
    alpha_low = 1;
    alpha_high = 1;
    M1 = 1;
else
    error('protocol should be either ''RR''or ''TOD''')
end

% ==========
% program calculating system matrices
% ==========

% Smatrices determining network configuration
Syu = [Sy];
p = length(Syu);
q = length(find(Syu == 1));
if l > q
    error('number of error signals is smaller then the number of nodes')
end
r = 0;
S = zeros(p,q);
for s = 1:p
    if Syu(s) == 1
        r = r+1;
        S(s,r) = 1;
    end
end

% System matrices according to [2], multiplied with Smatrices for  network
% configuration of nodes
A11 = [Ap+Bp*Dc*Cp];
A12x = [Bp*Dc];
A21x = -blkdiag(Cp)*A11;
A22x = -blkdiag(Cp)*A12x;
A12 = A12x*S;
A21 = S.'*A21x;
A22 = S.'*A22x*S;

% test closed loop stability without network
test_A11 = eig(A11);
if sum(test_A11 >= 0) > 0
    error('Closed loop system without network is unstable: reconsider controller; if Ac = 0, use a small negative constant or use the static controller function')
end

% ==========
% program computing L2 gain gamma by solving LMIs [1, (57)]
% ==========

% setup matrices and sdpvars
eps = 0.01;
[m,n] = size(A21);
P = sdpvar(n,n);
gamma_s = sdpvar(1,1);
% LMI definitions
L1 = set([A11.'*P+P*A11+eps*eye(n)+A21.'*A21 P*A12; A12.'*P (eps-gamma_s)*eye(m)]<0);
L2 = set(P>0);
L  = L1+L2;
% Calls SeDuMi solver to solve LMI
%disp(strvcat('Calling SeDuMi solver to solve LMI',' '))
solvesdp(L,gamma_s,sdpsettings('verbose',0));
% Recover solution from SDP variables
P = double(P);
gamma_s = double(gamma_s);
gamma = sqrt(gamma_s);
% Testing the LMI outcome!
test_LMI = round(eig([A11'*P+P*A11+eps*eye(n)+A21'*A21 P*A12; A12'*P (eps-gamma_s)*eye(m)])*10^6)/10^6;
test_P   = round(eig(P)*10^6)/10^6;
if sum(test_P < 0) > 0
    error('error in solving LMI for computing gamma: P [1, (57b)]is not postive definite')
elseif sum(test_LMI > 0) > 0
    error('error in solving LMI for computing gamma: LMI [1 (57a)) is not postive definite')
else
    %disp(strvcat(' ','gamma computed succesfully from LMI by SeDuMi'))
end

% ==========
% program computing constants for differential equations
% split into two parts, depending of sampled data (l==1) and NCS (l>1)
% ==========

% Computing Me
Me = sqrt(max(abs(eig(A22.'*A22))));

% Computing constants for differential equations phi [1, (22)] from [1, Theorem V.3]
if strcmp(prot,'RR')
    % V 0.2: Sharper bound see [1]
    L_0 = (Me)/(alpha_low);
elseif strcmp(prot,'TOD')
    L_0 = (M1*Me)/(alpha_low);
else
    error('protocol should be either ''RR''or ''TOD''')
end

if l>1
    %disp(strvcat(' ','number of nodes, l > 1'))
    L_1 = L_0*(lambdaW/lambda);
    gamma_0 = M1*gamma;
    gamma_1 = (M1*gamma*lambdaW)/lambda;

    % ==========
    % program solving phi functions by ode45 solver for different starting
    % conditions
    % ==========
    %disp(strvcat(' ','Solving differential equations Phi(t) for different initial conditions'))
    % initial conditions on differential equations phi
    % phi10 = phi00, so starting with delay free case.
    % in the loop phi01 is made higher then phi 00, so delayed cases are
    % calculated
    phi00 = 1/lambda;
    phi10 = phi00;
    options = odeset('Maxstep',1e-4,'Events',@(t,phi) events(t,phi,phi10,lambda));

    % initialize variables; tmati arbitratry but tmati>0, tmad = 0, since
    % phi10 = phi01, delay free case [1]
    T = [];
    tmad = .1; tmati = 100;
    a = .0001;
    % iterating for higher delays, finding tmati-tmad combinations in a grid of
    % initial conditions
    while tmad < tmati
        % saving previous tmad
        tmadp = tmad;
        % solving
        [t,phi,TE,YE,IE] = ode45(@(t,phi) Phi(t,phi,L_0,L_1,gamma_0,gamma_1),[0 100],[phi00,phi10],options);
        % finding solution for Hmati according to [1, Theorem IV.2]
        tmati = max((phi(:,1) >= lambda^2*phi10).*t);
        % finding solution for Tmad according to [1, Theorem IV.2]
        tmad = max((phi(:,2) >= phi(:,1)).*t);
        % save new combination of Hmati-Tmad in T
        T = [T; [tmati tmad]];
        % increase or decrease change in init condition, according to
        % relative difference in tmad
        if (tmad - tmadp)/(tmadp+.01) > 0.05
            % iterate more (increase in tmad above 5%)
            a = a/1.2;
        elseif (tmad - tmadp)/(tmadp+.01) < 0.01
            % iterate less (increase in tmad below 1%)
            a = a*1.2;
        else
            a = a;
        end
        % set gridded beginning conditions for next iteration
        phi10 = phi10 + a;
    end
elseif l == 1
        %disp(strvcat(' ','Number of nodes, l, is equal to 1 so sampled data case'))
        %disp('Making iterations for different values for lambda')
    lambda = .1;
    T1 = [];
    while lambda < sqrt((2-1)/2)
        L_1 = L_0*(lambdaW/lambda);
        gamma_0 = M1*gamma;
        gamma_1 = (M1*gamma*lambdaW)/lambda;

        % ==========
        % program solving phi functions by ode45 solver for different starting
        % conditions
        % ==========
        % disp(strvcat(' ','Solving differential equations Phi(t) for different initial conditions'))
        % disp(strvcat('lambda = ',num2str(lambda)))
        % initial conditions on differential equations phi
        % phi10 = phi00, so starting with delay free case.
        % in the loop phi01 is made higher then phi 00, so delayed cases are
        % calculated
        phi00 = 1/lambda;
        phi10 = phi00;
        options = odeset('Maxstep',1e-4,'Events',@(t,phi) events(t,phi,phi10,lambda));

        % initialize variables; tmati arbitratry but tmati>0, tmad = 0, since
        % phi10 = phi01, delay free case [1]

        tmad = .1; tmati = 100;
        a = .0001;
        % iterating for higher delays, finding tmati-tmad combinations in a grid of
        % initial conditions
        while tmad < tmati
            % saving previous tmad
            tmadp = tmad;
            % solving
            [t,phi,TE,YE,IE] = ode45(@(t,phi) Phi(t,phi,L_0,L_1,gamma_0,gamma_1),[0 100],[phi00,phi10],options);
            % finding solution for Hmati according to [1, Theorem IV.2]
            tmati = max((phi(:,1) >= lambda^2*phi10).*t);
            % finding solution for Tmad according to [1, Theorem IV.2]
            tmad = max((phi(:,2) >= phi(:,1)).*t);
            % save new combination of Hmati-Tmad in T
            T1 = [T1; [tmati tmad]];
            % increase or decrease change in init condition, according to
            % relative difference in tmad
            if (tmad - tmadp)/(tmadp+.01) > 0.05
                % iterate more (increase in tmad above 5%)
                a = a/1.2;
            elseif (tmad - tmadp)/(tmadp+.01) < 0.01
                % iterate less (increase in tmad below 1%)
                a = a*1.2;
            else
                a = a;
            end
            % set gridded beginning conditions for next iteration
            phi10 = phi10 + a;
        end
        lambda = lambda*1.25;
    end
    % select least conservative parts of the curves
    T2 = flipdim(sortrows(T1,1),1);
    tmad = T2(1,2);
    T = [];
    for q = 1:length(T2)
        if T2(q,2) > tmad
            T = [T;T2(q,:)];
            tmad = T2(q,2);
        else
            % discard
        end
    end
else
    error('There should be at least one node (sampled data)')
end

% ==========
% output
% ==========

% Hmati-Tmad tradeoff curve
Hmati = T(:,1); Tmad = T(:,2);

%toc

% ====================
% Bibliography
% 
% [1] W.P.M.H. Heemels, A.R. Teel, N. van de Wouw and D. Nesic
%     Networked control systems with communication constraints: Tradeoffs
%     between transmission intervals and delays
%     submitted
%
% [2] Gregory c. Walsh, Hong Ye and Linda G. Bushnell
%     Stability Analysis of Networked Control Systems
%     IEEE Transactions on control systems technology, vol. 10, no. 3, May
%     2002
% ====================



function dphi = Phi(t,phi,L_0,L_1,gamma_0,gamma_1)%,L_0,L_1,gamma_0,gamma_1)

% ====================
% Authors: P.J.H. Maas, W.P.M.H. Heemels
% Date: Februari 22th, 2010
% Project: Sum of Squares methods for Stability Analysis of Networked Control Systems, 
% Master Thesis
% 
% Version 1.0
% 
% File description:
% This function describes differential equations /dot{phi}_{0} and
% /dot{phi}_{1} from [1] for integration by an ode solver.
% BE CAREFUL WITH GLOBAL VARIABLES!
% ====================

% global L_0 L_1 gamma_0 gamma_1 (now in function call)

phi0 = phi(1);
phi1 = phi(2);

dphi = [( -2*L_0*phi0 - gamma_0*(phi0^2 + 1) );
        ( -2*L_1*phi1 - gamma_0*(phi1^2 + gamma_1^2/gamma_0^2) )];
    
% ====================
% [1] W.P.M.H. Heemels, A.R. Teel, N. van de Wouw and D. Nesic
%     Networked control systems with communication constraints: Tradeoffs
%     between transmission intervals and delays
% ====================


function [value,isterminal,direction] = events(t,phi,phi10,lambda)%,phi01)
% Locate the time when height passes through zero in a 
% decreasing direction and stop integration.
% global phi10 lambda (now in function call)
if (phi(:,1) < lambda^2*phi10);  % Detect (phi(:,1) < lambda^2*phi10)
    value = 1;  % do detect and stop
else
    value = 0;  % do not detect and do not stop
end
isterminal = 1;  % Stop the integration
direction = 0;   % Direction does not matter


