classdef ncs
%===================   
%    
%NCS       Networked Control System class
%
% obj = ncs(A,B,C,K,h,tau,delta,gammaU,gammaY,protocol)                    static controller
% obj = ncs(A,B,C,K,L,h,tau,delta,gammaU,gammaY,protocol)                  discrete observer-based controller
% obj = ncs(A,B,C,'C-LTI',Ac,Bc,Cc,Dc,h,tau,delta,gammaU,gammaY,protocol)  continuous dynamic controller
% obj = ncs(A,B,C,'D-LTI',Ac,Bc,Cc,Dc,h,tau,delta,gammaU,gammaY,protocol)  discrete dynamic controller  
%    
%Properties
%-------------------------------------------------------------------------
%SYSTEM DATA:
%   A,B,C         continuous-time plant matrices
%   ctrlType      type of controller 'static feedback',
%                 'D-LTI observer-based feedback', 
%                 'D-LTI dynamic feedback', 'C-LTI dynamic feedback'
%   K             static feedback 
%   K,L           observer-based feedback
%   Ac,Bc,Cc,Dc   dynamic feedback
%
%NETWORK DATA:
%   h             [hmin,hmax] bounds on the sampling time
%   tau           [taumin,taumax] bounds on the delay
%   delta         integer bound on the number of subsequent dropouts
%   gammaU        matrix of 1's and 0's on the diagonal indicating 
%                 network nodes for inputs
%   gammaY        matrix of 1's and 0's on the diagonal indicating 
%                 network nodes for outputs
%   protocol      either 'TOD' or 'RR'
%==============================================================
%
%
%Functions
%-------------------------------------------------------------------------
%     stable        = ncs.isNcsStable(ovraprx,drpmdl,lyap,gamma)
%     stable        = ncs.isNcsStable('GNB',gridpntM,epsilon_u,uWired)
%  [stable, K]      = ncs.stabilizeNcs(ovraprx,drpmdl,lyap,gamma)
% [Hmati,Tmad]      = findNcsStablilityTradeoff(obj,Su,Sy)
%
%Function Parameters:
%   ovraprx   = 'JNF','CH'
%    drpmdl   = 'explicit','lngtrans'   
%     lyap    = 'common','pardep'
%    gamma    = [0,1)  - decay rate of Lyapunov function (0 is slow)
%   gridpntM  = maximum number of gridpoints
%   epsilon_u = user defined tighness of overapproximation
%   uWired    = 1 if u is continuously known by controller
%               0 if u is sampled and sent over the network
%===========================================================
    properties
        A           %plant A matrix
        B           %plant B matrix
        C           %plant C matrix
        
        ctrlType    % type of controller
        K           % state feedback gain
        L           % output injection gain
        Ac          % dynamic feedback A matrix
        Bc          % dynamic feedback B matrix
        Cc          % dynamic feedback C matrix
        Dc          % dynamic feedback D matrix
        
        tau         %[taumin,taumax] bounds on the delay
        h           %[hmin,hmax] bounds on the transmission time
        delta       % integer bound on the number of subsequent dropous
        gammaU      % matrix of 1's and 0's on the diagonal indicating network nodes
        gammaY      % matrix of 1's and 0's on the diagonal indicating network nodes
        protocol    % either TOD or RR
    end
    
    methods
%% Constructor Function        
        function obj = ncs(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,arg11,arg12,arg13,arg14) 
            obj.A = arg1;
            obj.B = arg2;
            obj.C = arg3;
            if nargin == 10
                obj.ctrlType = 'Static Feedback';
                if isnumeric(arg4)
                    obj.K{1} = arg4;
                else
                    obj.K = arg4;
                end
                obj.h = arg5;
                obj.tau = arg6;
                obj.delta = arg7;
                obj.gammaU = arg8;
                obj.gammaY = arg9;
                obj.protocol = arg10; 
            elseif nargin == 11
               obj.ctrlType = 'D-LTI Observer-Based Feedback';
               if isnumeric(arg4)
                    obj.K{1} = arg4;
                else
                    obj.K = arg4;
                end
                if isnumeric(arg5)
                    obj.K{1} = arg5;
                else
                    obj.K = arg5;
                end
               obj.h = arg6;
               obj.tau = arg7;
               obj.delta = arg8;
               obj.gammaU = arg9;
               obj.gammaY = arg10;
               obj.protocol = arg11;  
            elseif nargin == 14
                if strcmp(arg4,'C-LTI')
                    obj.ctrlType = 'C-LTI Dynamic Feedback';
                elseif strcmp(arg4,'D-LTI')
                    obj.ctrlType = 'D-LTI Dynamic Feedback';
                end
                if isnumeric(arg5)
                    obj.Ac{1} = arg5;
                else
                    obj.Ac = arg5;
                end
                if isnumeric(arg6)
                    obj.Bc{1} = arg6;
                else
                    obj.Bc = arg6;
                end
                if isnumeric(arg7)
                    obj.Cc{1} = arg7;
                else
                    obj.Cc = arg7;
                end
                if isnumeric(arg8)
                    obj.Dc{1} = arg8;
                else
                    obj.Dc = arg8;
                end
                obj.h = arg9;
                obj.tau = arg10;
                obj.delta = arg11;
                obj.gammaU = arg12;
                obj.gammaY = arg13;
                obj.protocol = arg14;  
            end
        end
%% Class Functions      
        function stable = isNcsStable(obj,ovraprx,arg1,arg2,arg3)

            if strcmp(ovraprx,'GNB')
                stable = dncs_GNB(obj,arg1,arg2,arg3);
                % arg1 = gridpntM
                % arg2 = epsilon_u
                % arg3 = uWired 
            elseif isempty(obj.K) ~= 0
                error('K matrix not present.  Must use function stabilizeNcs first.')
            elseif strcmp(ovraprx,'JNF') || strcmp(ovraprx,'CH') 
                stable = dncs_JNF_CH(obj,ovraprx,arg1,arg2,arg3);
                % arg1 = drpmdl
                % arg2 = lyap
                % arg3 = gamma                           
            else
                error('Incorrect input parameters.')
            end
        end
        
        function [stable,K] = stabilizeNcs(obj,ovraprx,drpmdl,lyap,gamma)
            if strcmp(obj.ctrlType,'Static Feedback')==0
                error('Controller type must be static Feedback')
            end
            if isempty(obj.K) == 0
                obj.K{1}=[];
            end
            [stable,K] = dncs_JNF_CH(obj,ovraprx,drpmdl,lyap,gamma);
        end
        
        function compareModels(obj,gamma)
            if strcmp(obj.ctrlType,'Static Feedback')==0
                error('Controller type must be static Feedback')
            end
            if isempty(obj.K) ~= 0
                error('K matrix not present.  Must use function synthesizeK first.')
            end
            dncs_compareAll(obj,gamma);
        end
        
        function stabilityBoundry = plotStabilityRegion(obj,res,ovrapx,arg1,arg2,arg3,arg4)
            stabilityBoundry = dncs_plotRobustness(obj,res,ovrapx,arg1,arg2,arg3,arg4);
        end
        
        function [Hmati,Tmad] = findNcsStablilityTradeoff(obj,Su,Sy)
            if strcmp(obj.ctrlType,'C-LTI Dynamic Feedback')
                [Hmati,Tmad] = ncs_hyb_outputfb(obj,Su,Sy);
            elseif strcmp(obj.ctrlType,'Static Feedback')
                [Hmati,Tmad] = ncs_hyb_statefb(obj,Su,Sy); 
            else
                error('Controller must be Static Feedback or C-LTI Dynamic Feedback')
            end
        end        
        
%%  Restrictions on setting the variable type     
      
      function obj = set.A(obj,x)
         if isnumeric(x) == 0
            error('A must be a matrix of type double')
         elseif size(x,1) ~= size(x,2)
            error('A matrix must be square')
         end
        obj.A = x;
      end % set.A
      
      function obj = set.B(obj,x)
         if isnumeric(x) == 0
            error('B must be a matrix of type double') 
         end
        obj.B = x;
      end % set.B

      function obj = set.C(obj,x)
         if isnumeric(x) == 0
            error('K must be a matrix of type double') 
         end
        obj.C = x;
      end % set.C
      
%---------------------------------------------------------------------------      
      function obj = set.h(obj,x)
         if isnumeric(x) == 0
            error('h must be a row vector of type double')
         elseif size(x,2) ~= 2
            error('Must be 2 elements in h = [hmin,hmax]')
         elseif x(1)< 0 || x(2) < 0
            error('Elements in h must be positive')
         elseif x(1) > x(2)
            error('first element in h must be greater than or equal to the second')
         end
         obj.h = x;
      end % set.h
      
      function obj = set.tau(obj,x)
         if isnumeric(x) == 0
            error('tau must be a row vector of type double')
         elseif size(x,2) ~=2
            error('Must be 2 elements in tau = [taumin,taumax]')
         elseif x(1)< 0 || x(2) < 0
            error('Elements in tau must be positive')
         elseif x(1) > x(2)
            error('first element in tau must be greater than or equal to the second')
         end
         obj.tau = x;
      end % set.h      
      
      function obj = set.delta(obj,x)
        if isnumeric(x) == 0 
            error('Invalid delta parameter, must be a positive integer')
        elseif x - round(x) ~= 0 
            error('Invalid delta parameter, must be a positive integer')    
        elseif x < 0
            error('Invalid delta parameter, must be a positive integer')
        end
         obj.delta = x;
      end % set.drpmdl   

    end % methods
      
end