function SKalmanFilter_YOpt(block)
% Level-2 M file S-Function for optimal
%time-varying Kalman Filter - Single output
% Economical implementation of sq-root KF
% [1] Baramov,L. Pachner, D. and Havlena, V.:Kalman Filter for Systems With
% Communication Delay, IFAC Workshop of Networked Estimation and Control
% Systems, NECSys 09, Venice, It
% 
  setup(block);
  
%endfunction

function setup(block)
  
  block.NumDialogPrms  = 10;
  %Parameter list
  %A,B,C            Plant State Space
  %Q, R             Noise covar matrices
  %nd               max network delay in the multiples of output sampling
  
  %figNo            Figure number for the barplot (0 - no plot)
  %pause            pause for vizualization
   %Ts, Tsmin        Controller sampling time/network sampling time
  %-----------------------------------------------------------------------
  
    Ts = block.DialogPrm(9).Data;
    Tsmin = block.DialogPrm(10).Data;
  %% Register number of input port and output port
  block.NumInputPorts  = 3;
  block.NumOutputPorts = 2;

  %% Setup functional port properties
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
% measurements received via wireless network at high sampling rate
  block.InputPort(1).DatatypeID    = 0;
  block.InputPort(1).Complexity    = 0;
  block.InputPort(1).Dimensions    = 1;
% measurement timestamps received via wireless network at high sampling rate
  block.InputPort(2).DatatypeID    = 0;
  block.InputPort(2).Complexity    = 0;
  block.InputPort(2).Dimensions    = 1;
  
% manipulated variables obtained from controller at low sampling rate 
   block.InputPort(3).DatatypeID    = 0;
   block.InputPort(3).Complexity    = 0;
   block.InputPort(3).Dimensions    = 1;

  %filtered CV
  block.OutputPort(1).DatatypeID    = 0;
  block.OutputPort(1).Complexity    = 0;
  block.OutputPort(1).Dimensions    = 1; 
  
 %missed data (CV/MV acknowledgement)
   block.OutputPort(2).DatatypeID    = 8;
   block.OutputPort(2).Complexity    = 0;
   block.OutputPort(2).Dimensions    = 1; 
  
%Define sampling times
   block.InputPort(1).SampleTime  = [Tsmin 0];
   block.InputPort(2).SampleTime  = [Tsmin 0];
   block.InputPort(3).SampleTime  = [Ts   0];
   block.OutputPort(1).SampleTime = [Ts   0];
   block.OutputPort(2).SampleTime = [Ts   0];

  
  block.InputPort(1).DirectFeedthrough = true;
  block.InputPort(2).DirectFeedthrough = true;
  block.InputPort(3).DirectFeedthrough = false;
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';

  block.SetAccelRunOnTLC(true);
  
  %% Reg methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSampleTime',  @SetInpPortST);
  block.RegBlockMethod('SetOutputPortSampleTime', @SetOutPortST);
  block.RegBlockMethod('Update',                  @Update);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
  block.RegBlockMethod('SetInputPortDimensions', @SetInpPortDims);
  block.RegBlockMethod('SetInputPortDataType', @SetInpPortDataType);
  block.RegBlockMethod('SetOutputPortDataType', @SetOutPortDataType);
  block.RegBlockMethod('SetInputPortComplexSignal', @SetInpPortComplexSig);
  block.RegBlockMethod('SetOutputPortComplexSignal', @SetOutPortComplexSig);
  
%endfunction
  % Set input port sample time
function SetInpPortST(block, ~,~)
  ratio=(block.InputPort(1).SampleTime \block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(1).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction

function SetInpPortFrameData(block, idx, fd)
  
  block.InputPort(idx).SamplingMode = fd;
 
    block.OutputPort(1).SamplingMode  = fd;
    block.OutputPort(2).SamplingMode  = fd;
 %endfunction

function SetInpPortDims(block, id, dim)
  
  block.InputPort(id).Dimensions = dim;       
       
%endfunction

function SetInpPortDataType(block, idx, dt)
  
  block.InputPort(idx).DataTypeID = dt;
%endfunction
  
function SetOutPortDataType(block, idx, dt)
  block.OutputPort(idx).DataTypeID  = dt;
  %block.InputPort(1).DataTypeID     = dt;
%endfunction  

function SetInpPortComplexSig(block, idx, c)
  
  block.InputPort(idx).Complexity = c;
 % block.OutputPort(1).Complexity  = c;
%endfunction 
  
function SetOutPortComplexSig(block, idx, c)

  block.OutputPort(idx).Complexity = c;
  %block.InputPort(1).Complexity    = c;
%endfunction 

function SetOutputPortSampleTime(s, i, time)

s.OutputPort(i).SampleTime = time;

function SetInputPortSamplingMode(s, ~, ~)

    for i=1:3
        s.InputPort(i).SamplingMode = 'Sample';
    end
     s.OutputPort(1).SamplingMode = 'Sample';
     s.OutputPort(2).SamplingMode = 'Sample';
%endfunction

% Set output port sample time
function SetOutPortST(block, ~, ~)
  ratio=(block.InputPort(1).SampleTime /block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(6).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction
%------------------------------------------------------------------------
%------------------------------------------------------------------------
function DoPostPropSetup(block)

  %% Setup Dwork
  nd = block.DialogPrm(6).Data;
 
  nx0=size(block.DialogPrm(1).Data,1);
  
  block.NumDworks = 15;
  block.Dwork(1).Name               = 'x';        %state
  block.Dwork(1).Dimensions         = nx0+nd;
  block.Dwork(1).DatatypeID         = 0;
  block.Dwork(1).Complexity         = 'Real';
  block.Dwork(1).UsedAsDiscState    = true;
  
  block.Dwork(2).Name               = 'Itgt';        %baseline delay 
  block.Dwork(2).Dimensions         = 1;
  block.Dwork(2).DatatypeID         = 5;
  block.Dwork(2).Complexity         = 'Real';
  block.Dwork(2).UsedAsDiscState    = true;
  
  block.Dwork(3).Name               = 'Dx';        %Difference factor
  block.Dwork(3).Dimensions         = (nx0+nd)*(nx0+nd+1)/2;
  block.Dwork(3).DatatypeID         = 0;
  block.Dwork(3).Complexity         = 'Real';
  block.Dwork(3).UsedAsDiscState    = true;

  block.Dwork(4).Name               = 'mD';        %Width of theDifference Factor 
  block.Dwork(4).Dimensions         = 1;
  block.Dwork(4).DatatypeID         = 5;
  block.Dwork(4).Complexity         = 'Real';
  block.Dwork(4).UsedAsDiscState    = true;
  
  block.Dwork(5).Name               = 'Dstat';    %matrix of pre-computed gains
  block.Dwork(5).Dimensions         = nd*(nd+1)/2+nd*nx0;
  block.Dwork(5).DatatypeID         = 0;
  block.Dwork(5).Complexity         = 'Real';
  block.Dwork(5).UsedAsDiscState    = false;
  
  block.Dwork(6).Name               = 'K0';    %Optimal KF gain w/out delay
  block.Dwork(6).Dimensions         = nx0;
  block.Dwork(6).DatatypeID         = 0;
  block.Dwork(6).Complexity         = 'Real';
  block.Dwork(6).UsedAsDiscState    = false;
  
  block.Dwork(7).Name               = 'Ws';    %Optimal sqrt output cov  w/out delay
  block.Dwork(7).Dimensions         = 1;
  block.Dwork(7).DatatypeID         = 0;
  block.Dwork(7).Complexity         = 'Real';
  block.Dwork(7).UsedAsDiscState    = false;
  
  %....................................................Y-data Sorter states
  block.Dwork(8).Name            = 'Ybuff';   %input buffer for incomming samples
  block.Dwork(8).Dimensions      = nd;
  block.Dwork(8).DatatypeID      = 0;
  block.Dwork(8).Complexity      = 0;
  block.Dwork(8).UsedAsDiscState = 1;
  
  block.Dwork(9).Name = 'Dbuff';              %input buffer for incomming samples'  time stamps
  block.Dwork(9).Dimensions      = nd;
  block.Dwork(9).DatatypeID      = 2;
  block.Dwork(9).Complexity      = 0;
  block.Dwork(9).UsedAsDiscState = 1;
  
  block.Dwork(10).Name = 'KFTime';              %Time of the last 'slow' sample hit
  block.Dwork(10).Dimensions      = 1;
  block.Dwork(10).DatatypeID      = 0;
  block.Dwork(10).Complexity      = 0;
  block.Dwork(10).UsedAsDiscState = 1;
  
  block.Dwork(11).Name  = 'RefDelStat';        %Delay sample status - without regard for possible limited buffering
  block.Dwork(11).Dimensions         = 1;
  block.Dwork(11).DatatypeID         = 5;
  block.Dwork(11).Complexity         = 'Real';
  block.Dwork(11).UsedAsDiscState    = true;
  
  block.Dwork(12).Name = 'YmissFlag';              %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(12).Dimensions      = 1;
  block.Dwork(12).DatatypeID      = 8;
  block.Dwork(12).Complexity      = 0;
  block.Dwork(12).UsedAsDiscState = 0;
  %--------
  block.Dwork(13).Name = 'barplothandle';       %Graph handles
  block.Dwork(13).Dimensions      = 3;
  block.Dwork(13).DatatypeID      = 0;
  block.Dwork(13).Complexity      = 'Real';
  block.Dwork(13).UsedAsDiscState = true;
  %----------------------------------
  block.Dwork(14).Name               = 'pow_of_2';        %a magic Integer to remember
  block.Dwork(14).Dimensions         = nd+1;
  block.Dwork(14).DatatypeID         = 5;
  block.Dwork(14).Complexity         = 'Real';
  block.Dwork(14).UsedAsDiscState    = false;
  %---------------------------------------------
  block.Dwork(15).Name = 'Statelock';              %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(15).Dimensions      = 1;
  block.Dwork(15).DatatypeID      = 8;
  block.Dwork(15).Complexity      = 0;
  block.Dwork(15).UsedAsDiscState = 1;
%endfunction

function InitConditions(block)

  A= block.DialogPrm(1).Data;
  C= block.DialogPrm(3).Data;
  Q = block.DialogPrm(4).Data;
  R = block.DialogPrm(5).Data;
  nd = block.DialogPrm(6).Data;
  
  figNo=block.DialogPrm(7).Data;
  nx0=size(A,1);
  
  %% Initialize Dwork
  block.Dwork(1).Data = zeros(nx0+nd,1);
  block.Dwork(2).Data = uint16(0);
  block.Dwork(3).Data = zeros((nx0+nd)*(nx0+nd+1)/2,1);
  block.Dwork(4).Data = uint16(0);
  
  [HH, block.Dwork(6).Data, block.Dwork(7).Data]=Get_Fact(A,C,Q,R,nd);
  block.Dwork(5).Data=zeros(nd*(nd+1)/2+nd*nx0,1);
  pnt=1;
  for j=nx0+1:nx0+nd
      for i=1:j
          block.Dwork(5).Data(pnt)=HH(i,j);
          pnt=pnt+1;
      end
  end
  %--------------------
  block.Dwork(11).Data = uint16(2^(nd+1)-1);
  
   
   block.Dwork(8).Data=zeros(nd,1);
   block.Dwork(9).Data=-ones(nd,1,'int8');
   
    block.Dwork(12).Data = false;
   if figNo>0
        InitGraph(block);
   end
   c=zeros(nd+1,1);
   c(nd+1)=1;
   for i=1:nd
       c(nd+1-i)=2*c(nd+1-i+1);
   end
   block.Dwork(14).Data=uint16(c);
   block.Dwork(15).Data=true;
%    
   
   
%endfunction

function Output(block)
% Kalman data (filtering step implemented in the output Callback)
% Receives process measurements possibly delayed with time stamps as inputs
% Updates internal Kalman filter state (conditional means) based on these
% data). Sents filtered process output to the 

%%
 if block.OutputPort(1).IsSampleHit 
     nd = block.DialogPrm(6).Data;
        X=block.Dwork(1).Data;
        nx0=length(X)-nd;
        C=block.DialogPrm(3).Data;
        DlstX=double(block.Dwork(11).Data);
     if  block.Dwork(15).Data  %make sure that filtering and predicton steps 
                                %are run only once per sampling interval
        pow2=block.Dwork(14).Data;
        [block, yy, Dell]=SOutput(block);
        ny=length(Dell);
        
        aX = Dec2Bin(DlstX-1,pow2);
        y=zeros(ny,1); Del=zeros(ny,1);
        cnt=0;
    %check if some of the received samples have not been used before
        for i=1:ny
            di=Dell(i);
            idel=nd+1-di;
            if ~aX(idel)
                cnt=cnt+1;
                Del(cnt)=di;
                y(cnt)=yy(i);
                DlstX=DlstX+pow2(idel);
                aX(idel)=1;
            end
        end
        ny=cnt;
        Del=Del(1:cnt);
        y=y(1:cnt);
    %end of pre-sorting

        A=block.DialogPrm(1).Data;
    
        figNo=block.DialogPrm(7).Data;
        Itgt=double(block.Dwork(2).Data); %old 'target delay'
    %get static gains
        HH=zeros(nx0+nd,nd);
        pnt=1;
        for j=1:nd;
            for i=1:nx0+j
                HH(i,j)=block.Dwork(5).Data(pnt);
                pnt=pnt+1;
            end
        end
        K00=block.Dwork(6).Data;
        Ws=block.Dwork(7).Data;
        W0=Ws^2;
        mD=double(block.Dwork(4).Data);
        Dx=zeros(nx0+nd,mD);
        cnt=1;
        for j=1:mD
            for i=j:nd+nx0
                Dx(i,j)=block.Dwork(3).Data(cnt);
                cnt=cnt+1;
            end
        end
 
%---------------------------------------------------------------
        if ny
            if Del(1)<=Itgt
        %upon decrease of the target ind, expand Dx;
                haux=HH(:,Del(1)+1:Itgt);
                Dx=[Dx,haux];
                mD=mD+Itgt-Del(1);
                Itgt=Del(1);
            else
                Itgt=min(Itgt+1,nd);
            end

            K0=zeros(nx0+nd,1);
            if Del(1)
                K0(1:nx0+Itgt)=HH(1:nx0+Itgt,Itgt)/Ws;
            else
                K0(1:nx0)=K00;
            end

            if ny>mD &&mD
                mmd=ny-mD;
                Dx=[Dx,zeros(nx0+nd,mmd)];
                mD=mD+mmd;
            end
        dxy=zeros(ny,mD);
        Yp=zeros(ny,1);
        for i=1:ny
            if ~Del(i)
                dxy(i,:)=C*Dx(1:nx0,:);
                Yp(i)=C*X(1:nx0);
            else
                dxy(i,:)=Dx(nx0+Del(i),:);
                Yp(i)=X(nx0+Del(i));
            end
%         DlstX=DlstX+2^Del(i);
        end
        if mD
            TTx=dxy';
            [qz0,rz0]=qr(TTx(:,ny:-1:1));
            rz0=rz0(mD:-1:1,ny:-1:1);
            qz0=qz0(:,mD:-1:1);
            V1=qz0(:,1:mD-ny);
            if Itgt==Del(1);
                ia=1;
            else
                ia=0;
            end
            V2=qz0(:,mD-ny+1:mD-ny+ia); 
            V3=qz0(:,mD-ny+1+ia:mD); 
            M11=rz0(mD-ny+1:mD-ny+ia,1:ia); 
            M21=rz0(mD-ny+ia+1:mD,1:ia); 
            M22=rz0(mD-ny+ia+1:mD,ia+1:ny);
            if ia
                M22i=inv(M22);
                wk=M11^2+W0(1:ia,1:ia); 
                gamma=M21'*M22i';
                alpha=wk\W0; beta=1-alpha;
                Kk=K0*[1,-gamma]*alpha+Dx*[V2,V3]*[M11/wk,-M11/wk*gamma;...
                    zeros(ny-1,1),M22i'];
                Phi=[V1,V2*sqrt(alpha)];
            else
                Kk=Dx*V3/(M22');
                Phi=V1;
            end
            X=X+Kk*(y-Yp);
            Dx=Dx*Phi;
            mD=size(Dx,2);
            if mD
                if ~Del(1)
                    axm=C*Dx(1:nx0,:);
                    Dx(1:nx0,:)=Dx(1:nx0,:)-K00*axm;
                else
                    if ia
                        Dx(:,mD)= Dx(:,mD)-...
                            HH(:,Itgt)*sqrt(beta)*sign(Dx(nx0+Itgt,mD));
                    end
                end
            end
        else
            X=X+K0*(y-Yp);
        
        end
        else
            Itgt=min(Itgt+1,nd);
        end
%----------------------------------------------------------------
    %checking for missing data
        if DlstX<=2^nd
            missY=true;
        else
            missY=false;
        end
        block.Dwork(11).Data=uint16(DlstX);
    %update graphics
        if figNo>0
            UpdateGraph(block)
        end
    
    %Prediction step
        Yf=C*X(1:nx0);

        Dx(nx0+2:nx0+nd,:)=Dx(nx0+1:nx0+nd-1,:);
        X(nx0+2:nx0+nd,:)=X(nx0+1:nx0+nd-1,:);
        if Itgt
            Dx(nx0+1,:)=C*Dx(1:nx0,:);
            X(nx0+1,:)=Yf;
        else
            X(nx0+1,:)=y(1);
            Dx(1+nx0,:)=zeros(1,mD);
        end
        Dx(1:nx0,:)=A*Dx(1:nx0,:);
    %  X(1:nx0,:)=A*X(1:nx0,:)+B*u;
        X(1:nx0,:)=A*X(1:nx0,:);
        Dx=housered_rnk(Dx',1e-12)';
        mD=size(Dx,2);
 
%Updating the state
        block.Dwork(1).Data=X;
        block.Dwork(2).Data=uint16(Itgt);
        block.Dwork(4).Data=uint16(mD);
        cnt=1;
        for j=1:mD
            for i=j:nd+nx0
                block.Dwork(3).Data(cnt)=Dx(i,j);
                cnt=cnt+1;
            end
        end
    
        block.Dwork(11).data=uint16(mod(2*DlstX-2,2^(nd+1))+1);
        block.Dwork(12).data=logical(missY);
     else
         Yf=C*X(1:nx0);
         if DlstX<=2^nd
            missY=true;
        else
            missY=false;
        end
     end
    block.Dwork(15).Data=false;
% Updating output ports
    block.OutputPort(1).Data=Yf;
    block.OutputPort(2).Data=missY;
%    
 end
%endfunction

function Update(block)
%State update - Kalman filter prediction step

nd = block.DialogPrm(6).Data;
if ~block.OutputPort(1).IsSampleHit
    
    %sort out incomming data from WSN
    ts=block.InputPort(2).Data;
    if ts>-1
        dts=(block.Dwork(10).Data-ts)/block.OutputPort(1).SampleTime(1);
        aux=int8(dts)+1;
        if aux<=nd
            y=block.InputPort(1).Data;
            block.Dwork(8).Data(2:nd)=block.Dwork(8).Data(1:nd-1);
            block.Dwork(8).Data(1)=y;
            block.Dwork(9).Data(2:nd)=block.Dwork(9).Data(1:nd-1);
            block.Dwork(9).Data(1)=aux;
        end
        
    end
    
else
     X=block.Dwork(1).Data;
    nx0=length(X)-nd;
    B=block.DialogPrm(2).Data;
    u=block.InputPort(3).Data;
    X(1:nx0,:)=X(1:nx0,:)+B*u;
    block.Dwork(1).Data=X;
    block.Dwork(15).Data=true; %unlock state 
end

%endfunction
%>>>>>>>>>>>>>>>>>Local helper functions serving the callbacks
%Y-Sorter output function
function [block, Ybuff, Dbuff]=SOutput(block)

        nd=block.DialogPrm(6).Data; %max delay
        TimeS=block.CurrentTime;
        block.Dwork(10).Data=TimeS;
        y=block.InputPort(1).Data;
        ts=block.InputPort(2).Data;
        if ts>-1
            dts=(TimeS-ts)/block.OutputPort(1).SampleTime(1);
            if dts<=nd
                Ybuff=[y;block.Dwork(8).Data];
                Dbuff=[int8(dts);block.Dwork(9).Data];
            else
                Ybuff=block.Dwork(8).Data;
                Dbuff=block.Dwork(9).Data;
            end
        else
            Ybuff=block.Dwork(8).Data;
            Dbuff=block.Dwork(9).Data;
        end
       
        block.Dwork(8).Data=zeros(nd,1);
        block.Dwork(9).Data=-ones(nd,1,'int8');
        iy=find(Dbuff>-1);
        [Dbuff,ind]=sort(Dbuff(iy),1,'ascend');
        Ybuff=Ybuff(iy(ind));
        
%endfunction

%>>>Graph-related functions
function InitGraph(block)

  %% Initialize Dwork
  nd = block.DialogPrm(6).Data;
 
  fig= block.DialogPrm(7).Data;
 figure(fig);
%set(fg,'color',[0.,0.,0])

hh=bar(nd:-1:0,zeros(1,nd+1)); hold off
ylim([0,1]);xlim([-1,nd+1]);
% title('Received CV Data Status','fontsize',14,'fontweight','demi')
set(gca,'visible','off')

set(hh,'facecolor','cyan')
set(hh,'edgecolor',[0.75,0.75,1])
set(hh,'showbaseline','off')
set(hh,'LineWidth',3)

for i=0:nd
    aux=text(i,-.25,int2str(i),'fontsize',14,...
        'horizontalalignment','center');
        set(aux,'color',[.8,.8,.8],'fontweight','normal');
end

set(gca,'color',[0.5,0.5,0.5])
set(gcf,'windowstyle','normal')
set(gcf,'position',[150,725,550,125])
set(gca,'position',[.075,.25,.775,.4])
set(gcf,'Color',[0.5,0.5,0.5])
ttx=text(-.5,1.4,'Received CV Data Status');
set(ttx,'fontsize',14);
set(ttx,'color',[0.9,0.9,0.9]);

tt=text(nd+.5,1.6,'00:00:00');
set(tt,'fontsize',14);
set(tt,'color',[1,.25,.25]);

tt2=text(nd+.5,-.25,'smpl loss');
set(tt2,'fontsize',14);
set(tt2,'fontweight','demi');
 set(tt2,'color',[0,1,0]);

set(tt2,'backgroundcolor','none');
  block.Dwork(13).Data=[hh;tt;tt2];
%endfunction

function UpdateGraph(block)
% nd = block.DialogPrm(6).Data;
pau = block.DialogPrm(8).Data;
s= block.Dwork(11).Data-1;

a=Dec2Bin(s,block.Dwork(14).Data);%-----------------------------------


%figure(fig)
set(block.Dwork(13).Data(1),'ydata',a);

stim=block.CurrentTime;
hr=floor(stim/3600);
stim=stim-hr*3600;
mn=floor(stim/60);
sc=floor(stim-60*mn);
hr=int2str(hr); if length(hr)<2, hr=['0',hr];  end
mn=int2str(mn); if length(mn)<2, mn=['0',mn];  end
sc=int2str(sc); if length(sc)<2, sc=['0',sc];  end
set(block.Dwork(13).Data(2),'String',[hr,':',mn,':',sc])
if block.Dwork(12).Data
    set(block.Dwork(13).Data(3),'BackgroundColor',[.66,0,0])
    set(block.Dwork(13).Data(3),'color',[0.8 0.8 0]);
else
   set(block.Dwork(13).Data(3),'BackgroundColor','none')
   set(block.Dwork(13).Data(3),'color',[0,.5,0]);
end
pause(max(1e-3,pau))

function [HH,K,Ws]=Get_Fact(A0,C0,Q0,R0,nd)

nx0=size(A0,1);
HH=zeros(nx0+nd);
P0=dare(A0',C0',Q0,R0);

W0=C0*P0*C0'+R0;
K=P0*C0'/W0;
%AL=A-(A*K)*C;
Ws=sqrt(W0);
[u,s,~]=svd(P0);
HH(1:nx0,nx0:-1:1)=housered_rnk(diag(sqrt(diag(s)))*u',1e-16)';
HH(1:nx0,nx0+1)=A0*P0*C0'/Ws;HH(nx0+1,nx0+1)=Ws;
for i=2:nd
    HH(1:nx0+1,nx0+i)=[A0;C0]*HH(1:nx0,nx0+i-1);
    HH(nx0+2:nx0+i,nx0+i)=HH(nx0+1:nx0+i-1,nx0+i-1);
end
    
function U = housered_rnk(M,tol)

% housered(M) -  reduce matrix M to upper triangular matrix U
%                by a sequence of Householder reflections

[m,n]=size(M);
i=1; j=1;
tol2=tol^2;
while i<=m &&j<=n
    if i<m
        h=max(abs(M(i+1:m,j)));
        if h>tol
            [v,b]=gethouse(M(i:m,j),tol2);
        else
            b=0;
        end
    else
        b=0;
    end
  if b>0
%     w = b*M(i:m,j:n)'*v;
%     M(i:m,j:n)=M(i:m,j:n)-v*w';
    w = (b*v')*M(i:m,j:n);
    M(i:m,j:n)=M(i:m,j:n)-v*w;
    j=j+1;
    i=i+1;
  else
      if abs(M(i,j))<tol
          j=j+1;
      else
          i=i+1;j=j+1;
      end
  end
end
U=M(1:i-1,:);

function [v,b]=gethouse(x,tol)

% gethouse - get Householder vector and b coeff such that
%            P = I - b*v*v' is a Hoseholder reflection matrix

n = length(x);
x1=x(1);
x2=x(2:n);
v=[1;x2];
nx2s=x2'*x2;
%if nx2s==0,
if nx2s<tol,
  b=0;
else
  nx1=sqrt(x1*x1+nx2s);
  if x1 <= 0,
    v1=x1-nx1;
  else
    v1=-nx2s/(x1+nx1);
  end
  v(1)=v1;
  b = 2*v1*v1/(nx2s+v1*v1);
  v=v/v1;
end

function a=Dec2Bin(d,c)
%Decimal numbwer to binary representation
n=length(c);
a=zeros(1,n);

if d>=c(1)*2 
    error('overflow')
end
for i=1:n
    if d>=c(i)
        a(i)=1;
        d=d-c(i);
    end
    
end
