function msfcn_multi_hop2(block)
% Multi-hop network Simulink block sfunction
% Note:
%   Buffers are FIFO with first data vector on the first row.

setup(block);
  
%endfunction


%% Block Setup
function setup(block)
  
  block.NumDialogPrms  = 11;
  
  % --- Block parameters from dialog ----------------------
  Ts        = block.DialogPrm(1).Data;
  Ts_netw   = block.DialogPrm(2).Data;
  path_len  = block.DialogPrm(3).Data;
  path_prob = block.DialogPrm(4).Data;
  p_switch  = block.DialogPrm(5).Data;  
  
  % Register number of input and output ports
  block.NumInputPorts  = 2;
  block.NumOutputPorts = 2;

  % Setup functional port properties to dynamically inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
  
  block.InputPort(1).Complexity   = 'Real';
  block.InputPort(1).DataTypeId   = 0;
  block.InputPort(1).SamplingMode = 'Sample';
  
  block.InputPort(2).Complexity   = 'Real';
  block.InputPort(2).DataTypeId   = 0;
  block.InputPort(2).SamplingMode = 'Sample';  
  block.InputPort(2).Dimensions   = 1;
  
  block.OutputPort(1).Complexity   = 'Real';
  block.OutputPort(1).DataTypeId   = 0;
  block.OutputPort(1).SamplingMode = 'Sample';    
   
  block.OutputPort(2).Complexity   = 'Real';
  block.OutputPort(2).DataTypeId   = 0;
  block.OutputPort(2).SamplingMode = 'Sample';
  block.OutputPort(2).Dimensions   = 1;
  
  %    Direct feedthroughs
  block.InputPort(1).DirectFeedthrough = false;
  block.InputPort(2).DirectFeedthrough = false;
  % Set block sample time to inherited
  block.SampleTimes = [Ts_netw 0];
  
  % Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';

  % Register methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Update',                  @Update);  
  
%endfunction


%% Block Post Setup
function DoPostPropSetup(block)

  % --- Block parameters from dialog ----------------------
  Ts        = block.DialogPrm(1).Data;
  Ts_netw   = block.DialogPrm(2).Data;
  path_len  = block.DialogPrm(3).Data;
  path_prob = block.DialogPrm(4).Data;
  p_switch  = block.DialogPrm(5).Data;

  max_buf_size = 20;                    % maximum buffer size  
  n = block.InputPort(1).Dimensions+1;  % data dimension (+1 for time mark)
  nPaths       = length( path_len);     % number of alternative paths
  
  rs = RandStream('mt19937ar');

  % --- Setup Dwork ---------------------------------------
  block.NumDworks = 6;
  block.Dwork(1).Name = 'buf'; 
  block.Dwork(1).Dimensions      = n*max_buf_size * sum(path_len);
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = false;
  
  block.Dwork(2).Name = 'in_buf'; 
  block.Dwork(2).Dimensions      = n*max_buf_size;
  block.Dwork(2).DatatypeID      = 0;
  block.Dwork(2).Complexity      = 'Real';
  block.Dwork(2).UsedAsDiscState = false;

  block.Dwork(3).Name = 'out_buf'; 
  block.Dwork(3).Dimensions      = n*max_buf_size;
  block.Dwork(3).DatatypeID      = 0;
  block.Dwork(3).Complexity      = 'Real';
  block.Dwork(3).UsedAsDiscState = false;

  block.Dwork(4).Name = 'path_sel'; 
  block.Dwork(4).Dimensions      = 1;
  block.Dwork(4).DatatypeID      = 0;
  block.Dwork(4).Complexity      = 'Real';
  block.Dwork(4).UsedAsDiscState = false;  
  
  block.Dwork(5).Name = 'last_output'; 
  block.Dwork(5).Dimensions      = n-1;
  block.Dwork(5).DatatypeID      = 0;
  block.Dwork(5).Complexity      = 'Real';
  block.Dwork(5).UsedAsDiscState = false;  
  
  block.Dwork(6).Name = 'random_state'; 
  block.Dwork(6).Dimensions      = length(rs.State);
  block.Dwork(6).DatatypeID      = 0;
  block.Dwork(6).Complexity      = 'Real';
  block.Dwork(6).UsedAsDiscState = false;    
  
  delete(rs);

%endfunction


%% Block Init Conditions 
function InitConditions(block)

  % --- Block parameters from dialog ----------------------
  Ts        = block.DialogPrm(1).Data;
  Ts_netw   = block.DialogPrm(2).Data;
  path_len  = block.DialogPrm(3).Data;
  path_prob = block.DialogPrm(4).Data;
  p_switch  = block.DialogPrm(5).Data;
  synchronize = block.DialogPrm(11).Data;  

  max_buf_size = 20;                    % maximum buffer size  
  n = block.InputPort(1).Dimensions+1;  % data dimension (+1 for time mark)
  nPaths       = length( path_len);     % number of alternative paths
  
  if synchronize
    rs = RandStream('mt19937ar');
  else
    rs = RandStream.getDefaultStream;
  end


  % --- Initialize Dwork ----------------------------------
  block.Dwork(1).Data = nan( n*max_buf_size * sum(path_len) , 1);
  block.Dwork(2).Data = nan( n*max_buf_size , 1);
  block.Dwork(3).Data = nan( n*max_buf_size , 1);  
  block.Dwork(4).Data = randi(rs,nPaths);
  block.Dwork(5).Data = zeros(n-1,1);
  block.Dwork(6).Data = double(rs.State);  
  
  if synchronize,
    delete(rs);
  end  
  
%endfunction


%% Block Output Function
function Output(block)
  max_buf_size = 20;                    % maximum buffer size  
  n = block.InputPort(1).Dimensions+1;  % data dimension (+1 for time mark)
  hold_output = block.DialogPrm(7).Data;
  
  % --- Load data from DWork ------------------------------
  out_buf = block.Dwork(3).Data;
  out_buf = reshape(out_buf,max_buf_size,[]);
  
  % --- Write to output -----------------------------------
  if isnan(out_buf(1,1)),
    if hold_output,
      block.OutputPort(1).Data = block.Dwork(5).Data;
    else
      block.OutputPort(1).Data = zeros(n-1,1);    
    end
    block.OutputPort(2).Data = -1;
  else
    block.OutputPort(1).Data = out_buf(1,2:end)';    
    block.Dwork(5).Data = out_buf(1,2:end)';      % save for hold output functionality
    block.OutputPort(2).Data = out_buf(1,1);        
    out_buf = [ out_buf(2:end,:) ; nan(1,n) ];
  end
  
  % --- Save data to DWork --------------------------------
  block.Dwork(3).Data = reshape(out_buf,[],1);
  
%endfunction


%% Block Update
function Update(block)

  % --- Block parameters from dialog ----------------------
  Ts          = block.DialogPrm(1).Data;
  Ts_netw     = block.DialogPrm(2).Data;
  path_len    = block.DialogPrm(3).Data;
  path_prob   = block.DialogPrm(4).Data;
  p_switch    = block.DialogPrm(5).Data;
  raise_error = block.DialogPrm(6).Data;
  p_lost      = block.DialogPrm(8).Data;  
  burst_deg   = block.DialogPrm(9).Data;  
  buffer_type = block.DialogPrm(10).Data;  
  synchronize = block.DialogPrm(11).Data;  
  
  if strcmpi(buffer_type,'FIFO'),
    FIFO = 1;
  else
    FIFO = 0;    
  end
  
  if synchronize
    rs = RandStream('mt19937ar');
    set(rs,'State',uint32(block.Dwork(6).Data));
  else
    rs = RandStream.getDefaultStream;
  end  
  
  max_buf_size = 20;                    % maximum buffer size  
  n = block.InputPort(1).Dimensions+1;  % data dimension (+1 for time mark)
  nPaths       = length( path_len);     % number of alternative paths
  
  % --- Load data from DWork ------------------------------
  tmp = reshape(block.Dwork(1).Data,n*max_buf_size,[]);
  buf = {};
  cnt = 1;
  for iPath = 1:nPaths,
    for iBuffer = 1:path_len(iPath),
      buf{iPath,iBuffer} = reshape(tmp(:,cnt),max_buf_size,[]);
      cnt = cnt + 1;
    end
  end  
  in_buf   = block.Dwork(2).Data;  
  in_buf   = reshape(in_buf,max_buf_size,[]);  
  out_buf  = block.Dwork(3).Data;
  out_buf  = reshape(out_buf,max_buf_size,[]);
  path_sel = block.Dwork(4).Data;  

  
  % --- Path switching --------------------------------------
  if nPaths>1,
    if rand(rs)<=p_switch,
      i = randi(rs,nPaths-1);
      if i==path_sel, i=i+1; end
      if i>nPaths, i=1; end
      path_sel = i;      
    end
  end  

  
  % --- Input sampling ------------------------------------
  if abs( rem(block.CurrentTime,Ts) ) < 0.1*Ts_netw,
    first_empty = find(isnan(in_buf(:,1)),1);
    if isempty(first_empty),
      if raise_error,
        error('Input Buffer Overflow.');
      end
      if FIFO
        in_buf(1:end-1,:) = in_buf(2:end,:);
        first_empty = max_buf_size;        
      end
    end    
    if FIFO,
      in_buf(first_empty,:) = [ block.CurrentTime , block.InputPort(1).Data'];
    else % LIFO
      in_buf = [ block.CurrentTime , block.InputPort(1).Data'
                 in_buf(1:end-1,:) ];
    end
  end
  

  % --- Burst probability degradation factor --------------
  if block.InputPort(2).Data
    deg_factor = burst_deg;
  else
    deg_factor = 1;
  end  

  
  % --- Transmitting to destination node buffer -----------
  for iPath = 1:nPaths,
    tmp = buf{iPath,path_len(iPath)};
    if rand(rs)<=deg_factor*path_prob(iPath) && ~isnan(tmp(1,1)),
      first_empty = find(isnan(out_buf(:,1)),1);
      if isempty(first_empty),
        if raise_error,
          error('Output Buffer Overflow.');
        end
        if FIFO
          out_buf(1:end-1,:) = out_buf(2:end,:);
          first_empty = max_buf_size;        
        end
      end          
      if rand(rs)>=p_lost,
        if FIFO,
          out_buf(first_empty,:) = tmp(1,:);
        else % LIFO
          out_buf = [ tmp(1,:)
                      out_buf(1:end-1,:) ];
        end
      end
      buf{iPath,path_len(iPath)} = [ tmp(2:end,:) ; nan(1,n) ];
    end
  end
  
  
  % --- Transmission between intermediate node buffers ----
  for iPath = 1:nPaths,
    for iBuffer = path_len(iPath)-1:-1:1,
      buf1 = buf{iPath,iBuffer};      
      if ~isnan(buf1(1,1)),
        if rand(rs)<=deg_factor*path_prob(iPath),
          buf2 = buf{iPath,iBuffer+1};
          % ---
          first_empty = find(isnan(buf2(:,1)),1);          
          if isempty(first_empty),
            if raise_error,
              error('Buffer Overflow.');
            end
            if FIFO
              buf2(1:end-1,:) = buf2(2:end,:);
              first_empty = max_buf_size;        
            end
          end                        
          if rand(rs)>=p_lost,
            if FIFO,
              buf2(first_empty,:) = buf1(1,:);
            else % LIFO
              buf2 = [ buf1(1,:)
                       buf2(1:end-1,:) ];
            end
          end
          buf1 = [ buf1(2:end,:) ; nan(1,n) ];
          % ---
          buf{iPath,iBuffer}   = buf1;
          buf{iPath,iBuffer+1} = buf2;
        end
      end
    end
  end

  
  % --- Transmitting from source node buffer ---------------
  if rand(rs)<=deg_factor*path_prob(path_sel) && ~isnan(in_buf(1,1)),
    tmp        = buf{path_sel,1};
    first_empty = find(isnan(tmp(:,1)),1);    
    if isempty(first_empty),
      if raise_error,
        error('Buffer Overflow.');
      end
      if FIFO
        tmp(1:end-1,:) = tmp(2:end,:);
        first_empty = max_buf_size;        
      end
    end          
    if rand(rs)>=p_lost,
      if FIFO,
        tmp(first_empty,:) = in_buf(1,:);
      else % LIFO
        tmp = [ in_buf(1,:)
                tmp(1:end-1,:) ];
      end
    end
    buf{path_sel,1} = tmp;
    in_buf = [ in_buf(2:end,:) ; nan(1,n) ];
  end    

  
  % --- Save data to DWork --------------------------------
  tmp = [];
  for iPath = 1:nPaths,
    for iBuffer = 1:path_len(iPath),
      tmp = [ tmp ; reshape(buf{iPath,iBuffer},[],1) ];
    end
  end  
  block.Dwork(1).Data = tmp;
  block.Dwork(2).Data = reshape(in_buf,[],1);
  block.Dwork(3).Data = reshape(out_buf,[],1);
  block.Dwork(4).Data = reshape(path_sel,[],1);
  block.Dwork(6).Data = double(rs.State);  
  
  if synchronize,
    delete(rs);
  end    
    
%endfunction
