%% Example of decLMI decentralized controller synthesis
%
% This demo file shows how to use the DECLMI class to synthesize three LTI 
% proportional controllers that guarantees stability while
% fulfilling constraints for the netwotk depicted below:
%
% <html>
% <IMG SRC="network.jpg", height=400>
% </html>
%
% <latex>
% \includegraphics[width=.7\hsize]{network.jpg}
% </latex>
%
% Controllers are:
%
% * centralized ideal: each actuator can always exploit the whole state
% measurements to compute the control action;
%
% * decentralized ideal: each actuator can exploit a subset of state
% measurements to compute the control action, and that set is
% time invariant;
%
% * decentralized lossy: each controller can exploit a subset of state
% measurements, and measruements sent through some network links can be lost.
% No stochastic model of the dropouts is exploited here. The resulting 
% controller guarantees robust convergence to the origin for any occurrence 
% of packet dropouts.
%
% Moreover, a stochastic controller is also computed, which guarantees 
% closed-loop stability in the mean-square sense. It exploits a model of 
% the packet dropouts as given by a two-state Markov chain. In this
% case the constraints are not guaranteed to hold at every time step.
%
clear variables
clear globals
close all
clc
%% Plant model
%
% Net describes how the network is connected:
% * 1  = wired (reliable);
% * -1 = wireless (unreliable, i.e., seubject to dropouts);
% * 0  = no link.
      % e  f  g  h  j  k  l  m
Net = [ 1  1  1  0  0 -1  0  0   % a 
       -1  1  0  1  1  0  0  0   % b
        0  0  1  0  0  1  1 -1   % c
        0  0  0 -1  1  0  1  1]; % d
[m,n]=size(Net); 
%%
% State space matrices of the LTI discrete-time model
% $x(k+1) = A x(k) + B u(k)
A =[0.2843   -0.2895    0.1268   -0.0624   -0.2098  0.1882 -0.0061  0.2227;
   -0.2729    0.5052    0.0129   -0.1057   -0.1361  0.2154  0.2799 -0.0183;
    0.1610   -0.1229    0.1829   -0.4267    0.0589 -0.0905 -0.0417 -0.1709;
   -0.1084   -0.0138   -0.4611   -0.0735   -0.2013  0.1463  0.0854 -0.1567;
   -0.0300   -0.1167   -0.0012   -0.1847    0.3547  0.2081  0.0567  0.1566;
    0.2852    0.3036   -0.0994    0.1240    0.1099  0.1262  0.0229 -0.1599
   -0.0032    0.1558   -0.1285    0.1898   -0.0040  0.0017  0.7065 -0.1697;
    0.1961   -0.0173   -0.1023   -0.1888    0.0327 -0.2866 -0.1202  0.3198];
B =[     0   -2.2374   -0.4531         0;
   -0.1794    1.0976    1.3995    0.4287;
   -1.4671         0   -0.4620   -0.7370;
    1.3953   -1.6146    0.0327    0.5649;
    0.4408   -1.2287    0.7988   -1.3842;
    0.5654    0.2074    0.8968    0.4603;
         0         0    0.1379         0;
         0   -1.0061         0    0.3798];
%%
% Sample time
Ts = 1;

%%
% Constraints on input and state
% sqrt(x'x) <= xmax, sqrt(u'u)<=umax
xmax = 150;
umax = 150;

%%
%Weights
Qx = eye(n);
Qu = 1e-2*eye(m);

%%
% Vertices of the initial state uncertainty polytope
X0 = [
    7.7618    7.7618    8.7618    8.7618
    6.0485    7.0485    6.0485    7.0485
    6.2866    6.2866    6.2866    6.2866
    6.6018    6.6018    6.6018    6.6018
    5.1547    5.1547    5.1547    5.1547
    7.8859    7.8859    7.8859    7.8859
    5.1507    5.1507    5.1507    5.1507
    5.0470    5.0470    5.0470    5.0470];


 
%% Markov chain description
%
% The two-state Markov chain is described by:
%
% * d : array, where d(i) is the probability of losing a packet
% being in the i-th state of the Markov chain;
% * q : array, where q(i) is the probability of of remaining in
% the i-th state of the Markov chain.
%
Mc.d = [.1 .5];
Mc.q = [.8 .5];

%%
% Computation
% instance creation
obj = decLMI(Net,A,B,Qx,Qu,X0,xmax,umax,Mc);
%%
% Now all the controller synthesis problems are formulated and solved. The 
% time needed for each computation is stored for comparison purposes.

%% Centralized ideal
tic
obj=obj.solve_centralized_lmi();
CPUtime.centralized = toc;

%% Decentralized ideal
tic
obj = obj.solve_dec_ideal_lmi();
CPUtime.dec_ideal = toc;

%% Decentralized lossy (robust)
tic
obj = obj.solve_dec_lossy_lmi();
CPUtime.dec_lossy = toc;

%% Decentralized stochastic
tic
obj = obj.solve_dec_stoch_lmi();
CPUtime.dec_stoch = toc;

%% Simulation
% Random initial state in X0
x0 =[
    8.3272
    5.7953
    7.2282
    8.6640
    5.8317
    6.6488
    8.1784
    6.5376];
%%
% Simulation length
Tsim = 50;
%%
% Number of simulations to be carried out
Nsim = 50;
%%
% For a proper comparison, a sequence of states and emissions of the Markov
% chain model is stored.
load Statechange_paper
load Randvals_paper

Sim = [];
for iter=1:Nsim
    %%
    % Generate realizations of Markov chain
    statechange = Statechange(1,:,iter);
    randvals = Randvals(1,:,iter);
    %%
    % Initial Markov chain state
    initialstate = 1; 
    %%
    % Compute sequence of Markov chain states
    trc = cumsum(obj.Mc.T,2);
    ec = cumsum(obj.Mc.E,2);
    currentstate = initialstate;
    numStates = size(obj.Mc.T,1);
    numEmissions = size(obj.Mc.E,2);
    for i = 1:Tsim
        %%
        % Calculate state transition
        stateVal = statechange(i);
        state = 1;
        for innerState = numStates-1:-1:1
            if stateVal > trc(currentstate,innerState)
                state = innerState + 1;
                break
            end
        end
        %%
        % Calculate emission
        val = randvals(i);
        emit = 1;
        for inner = numEmissions-1:-1:1
            if val > ec(state,inner)
                emit = inner + 1;
                break
            end
        end
        %%
        % Add values and states to output
        seq(i) = emit;
        states(i) = state;
        currentstate = state;
    end

    %%
    % Get controllers gains from decLMI obj
    Kc=obj.K.ci;
    Kd=obj.K.di;
    Kl=obj.K.dl;
    Ks=obj.K.ds;
    
    %%
    % Init states, inputs, state norms and input norms
    Xc = x0;
    Xd = x0;
    Xn = x0;
    Xl = x0;
    Xs = x0;
    
    Uc = [];
    Ud = [];
    Un = [];
    Ul = [];
    Us = [];
    
    Xcnorm = [];
    Xdnorm = [];
    Xnnorm = [];
    Xlnorm = [];
    Xsnorm = [];
    
    Ucnorm = [];
    Udnorm = [];
    Unnorm = [];
    Ulnorm = [];
    Usnorm = [];
    
    %%
    % Perform current simulation
    for i=1:Tsim
        %%
        % Centralized
        Uc(:,i) = Kc*Xc(:,i);
        Xc(:,i+1) = A*Xc(:,i) + B*Uc(:,i);
        Xcnorm(i) = norm(Xc(:,i));
        Ucnorm(i) = norm(Uc(:,i));
        %%
        % Decentralized with ideal network
        Ud(:,i) = Kd*Xd(:,i);
        Xd(:,i+1) = A*Xd(:,i) + B*Ud(:,i);
        Xdnorm(i) = norm(Xd(:,i));
        Udnorm(i) = norm(Ud(:,i));
        %%
        % Decentralized with lossy network and robust stability
        Ul(:,i) = Kl{seq(i)}*Xl(:,i);
        Xl(:,i+1) = A*Xl(:,i) + B*Ul(:,i);
        Xlnorm(i) = norm(Xl(:,i));
        Ulnorm(i) = norm(Ul(:,i));
        %%
        % Decentralized with lossy network and stoch. stability
        Us(:,i) = Ks{seq(i),states(i)}*Xs(:,i);
        Xs(:,i+1) = A*Xs(:,i) + B*Us(:,i);
        Xsnorm(i) = norm(Xs(:,i));
        Usnorm(i) = norm(Us(:,i));

    end
    %%
    % Performance indices initialization
    Jc = [];
    Jd = [];
    Jl = [];
    Js = [];
    %%
    % The used performance index is a sum over the entire simulation
    % horizon of the state and input norms weighted by the Riccati equation
    % weights.
    for i=1:Tsim
        Jc(i) = sqrt(Xc(:,i)'*Qx*Xc(:,i) + Uc(:,i)'*Qu*Uc(:,i));
        Jd(i) = sqrt(Xd(:,i)'*Qx*Xd(:,i) + Ud(:,i)'*Qu*Ud(:,i));
        Jl(i) = sqrt(Xl(:,i)'*Qx*Xl(:,i) + Ul(:,i)'*Qu*Ul(:,i));
        Js(i) = sqrt(Xs(:,i)'*Qx*Xs(:,i) + Us(:,i)'*Qu*Us(:,i));
    end
    %%
    % store the current simulation results
    Sim.Jc(iter) = sum(Jc);
    Sim.Jd(iter) = sum(Jd);
    Sim.Jl(iter) = sum(Jl);
    Sim.Js(iter) = sum(Js);
    
end

disp('  Global results over %d simulations:   ',Nsim);
disp('Central. ideal performance: %.2f ',sum(Sim.Jc));
disp('Decentr. ideal performance: %.2f ',sum(Sim.Jd));
disp('Decentr. lossy performance: %.2f ',sum(Sim.Jl));
disp('Decentr. stoch performance: %.2f ',sum(Sim.Js));

disp(' Central. ideal performance AVG: %.2f ',mean(Sim.Jc));
disp('Decentr. ideal performance AVG: %.2f ',mean(Sim.Jd));
disp('Decentr. lossy performance AVG: %.2f ',mean(Sim.Jl));
disp('Decentr. stoch performance AVG: %.2f ',mean(Sim.Js));

disp(' Central. ideal performance STD: %.2f ',std(Sim.Jc));
disp('Decentr. ideal performance STD: %.2f ',std(Sim.Jd));
disp('Decentr. lossy performance STD: %.2f ',std(Sim.Jl));
disp('Decentr. stoch performance STD: %.2f ',std(Sim.Js));

CPUtime

%% Conclusions
%
% *Performance evaluation*
%
% As expected, the best performance is achieved by the centralized ideal
% controller, as it can use all the state measurements for all actuators at
% all time steps.
%
% However, the decentralized ideal controller achieves a performance which 
% is only 10% higher than the ideal centralized one, while using much less
% communications between sensors and actuators.
%
% A robustly stabilizing controller has been found also for the case where 
% some of the links are subject to possible packet dropouts. In this case
% constraints are fulfilled at every time step, regardless of the
% occurrence of the dropouts in the network.
%
% Exploiting a stochastic model of the packet dropouts, a stochastic
% controller has been computed which obtaines an improvement on the 
% performance index of around 6% with respect to the robust control scheme.