%% Class definition file for Hierarchical and Decentralized Model Predictive
%% Control (HiMPC)
%
% This class implements the hierarchical MPC control paradigm presented in 
% |Barcelli, Bemporad, Ripaccioli, CDC10| and decentralized extension.

%% Class description
%
% In this work we propose a decentralized hierarchical multi-rate
% control design approach to linear systems subject to linear
% constraints on input and output variables. At the lower level,
% a set of linear controllers stabilize the open-loop process without
% considering the constraints. A higher-layer, composed of a set of 
% independent controllers, commands reference signals at a lower sampling 
% frequency so as to enforce linear constraints on the variables of the 
% process. By optimally constraining the magnitude and the rate of 
% variation of the reference signals applied to the lower control layer, we
% provide quantitative criteria for selecting the ratio between the 
% sampling rates of the upper and lower layers to preserve closed-loop
% stability without violating the prescribed constraints.
%
% The HiMPC class has been developed to automatically generate the proposed
% multi-layer control architecture, which structure is shown in the 
% following figure.
%
% <html>
% <IMG SRC="dhimpc.jpg", height=400>
% </html>
%
% <latex>
% \includegraphics[width=.7\hsize]{DHiScheme.pdf}
% </latex>
%
% The layout of the controller is composed by two levels in a hierarchical 
% structure which, in turn, are composed by a set of independent and 
% decentralized sub-controllers.
% We assume that the stability of the process is guaranteed by the action
% of the lower control layer, which runs at the same sample time of the 
% plant. Therefore is required that the LTI model of the plant in 
% closed-loop with the set of lower layer controllers, given by
% the user, is stable. Future update to the toolbox will allow the user to
% automatically generate the stabilizing lower layer given the plant model 
% and the desired decentralization, for example though LMI (See 
% corresponding section of the toolbox). 
% The decentralization of the inner loop controller, i.e. the sets of input,
% states and outputs assigned to each sub-controllers is one of the 
% strenghts of the proposed approach. 
% The decentralized design has been exploited together with the LTI model 
% of the plant and the constraints on inputs and states to compute the 
% polytopic sets of each submodel which is the bound the mutual influence 
% of the different subsystems on each other. This result is fundamental 
% because allows to treat each subsystem independently, since for
% all of them a single Maximal Output Admissible Set (MOAS) is computed [1]. 
% The independece of the MOASs is guaranteed by taking into account 
% the uncertainty polytope, which bounds the interaction with the others, 
% in the invariant set computation.
%
% Moreover, the MOAS determination is related to the restriction of the
% admissible output set, which is a polytope tightening. The contraction
% factor, $\Delta y$, is the main tuning knob of the approach:
% the smaller the components of $\Delta y$, the larger is the set
% of admissible set points, but the smaller will be the
% admissible reference increments to maintain tracking
% errors within the admissible error set.
%
% Then for each subsystem, the maximum element-wise 
% reference variation such that for any admissible state and interaction,
% the system state will be in a MOAS at the next execution of higher level
% controller. 
% It is evident that such variation is a function of the ratio between the 
% two layers sample time.
% The constraint computed in that way bounds the reference
% variation that is possible to give to the sub-controllers,
% while preserving the closed-loop stability and enforcing the constraints.
% Such task is performed by the higher level controllers.
%
% Finally the maximum element-wise reference variation for each subsystem
% is computed.
% The maximum element-wise reference variation is defined as the smallest
% change of reference vector that can be applied to the closed-loop system 
% of the lower layer controllers with the plant such that, starting from
% an invariant set, the state vector lands outside a new invariant set 
% a given number of steps. Or, in other words, for
% all reference changes the closed-loop system is such that, starting from an
% invariant set, the state vector always arrives into a new
% invariant set after given number of steps.
%
% [1] Kolmanovsky, I. and Gilbert, E.G., _Maximal output admissible sets 
% for discrete-time systems with disturbance inputs_, 14th American Control
% Conference 1995, Seattle, WA
%
%
classdef HiMPC_old
    properties (Access=private)
        %% Properties
        %
        % |DeltaY| : Array of the same size of the output constraints that
        %            determines their tightening
        %
        % |Ycon|   : Structure of output constraints with fields
        %
        % * min;
        % * max;
        %
        %
        % |dec|    : Decentralization structure, is a cell array of
        %  structures each of which with fileds:
        % 
        % * x        (states of the subsystem);
        % * y        (outputs of the subsystem);
        % * u        (inputs of the subsystem);
        % * applied  (inputs effectively applied, no overlap is allowed).
        % i-th structure contains the indices of the respective elements
        % that are currently included in the i-th subsystem
        %
        % |model|  : ss object with the plant LTI model
        %
        % |Xcon|   : State bounds structure containing fields 
        %
        % * min;
        %
        % * max.
        % The constraints are expressed in the state space of the
        % centralized system.
        %
        DeltaY;
        DeltaY2;
        Ycon;
        dec;
        model;      
        Xcon;
        %%
        % Cell array of the state indices that the do not belong to the i-th
        % subsystem
        comp;
        %%
        % Cell array of non element-wise state constraints with fields H and
        % K to be appended to the constraint polytope of the i-th subsystem
        coupledCons;
        %%
        % nKo(i)=length(Ko{i})
        nKo;
    end
    
    properties (SetAccess = private)
        %%
        % Cell array of polytope of i-th subsystem output bounds
        Py;
        %%
        % Py{i} = {Hy{i} * y <= Ky{i}}
        Hy;
        Ky;
        %%
        % Cell array of state space matrix of each subsystem
        Ai;
        Bi;
        %%
        % Cell array of state constraint of the i-th subsystem
        P;
        %%
        % Cell array of MOAS
        Oinf;
        %%
        % Cell array of MOAS computed without the other subsystem influence
        OinfNoNoise;
        %%
        % Oinf{i} = {Ho{i} * x <= Ko{i}}
        Ho;
        Ko;
        %%
        % Cell array of maximum allowable reference variation, computed for
        % each sample time ratio between the higher and lower control
        % levels
        Dr;
        %%
        % Dr{i} is divided by the corresponding N (sample times ratio) in
        % order to determine the maximum reference variation per unit of
        % time
        DrN;
        %%
        % Output constraints expressed in state coordinates
        Hyx;
        Kyx;
        %%
        % Cell array of polytopes of the influence of all the other
        % subsystems with respect to to the i-th one.
        Pnoise;
    end
    
    methods
        %% Constructor
        %
        % obj  = HiMPC(model,dec,Ycon,DeltaY,Xcon,coupledCons)
        % 
        % |model|  : ss object with the plant LTI model
        %
        % |dec|    : Decentralization structure: is supposed to an array of
        %  structure containing fileds:
        % 
        % * x         (states of the subsystem);
        % * y         (outputs of the subsystem);
        % * u         (inputs of the subsystem);
        % * applied  (inputs effectively applied, no overlap is allowed).
        % each containing the array of indices to be included in the
        % subsystem.
        %
        % |Ycon|   : structure of output constraints with fields min and max
        %
        % |DeltaY| : Array of the same size of the output constraints that
        %            determines their tightening
        %
        % |Xcon|   : state bounds structure eight fields min and max
        %
        % |coupledCons|: Cell array of non element-wise state constraints 
        % with fields H and K to be appended to the constraint polytope of
        % the corresponding subsystem
        %
        function obj = HiMPC(varargin)
            error(nargchk(5,7,nargin));
            % model
            if isequal(class(varargin{1}),'ss')
                obj.model=varargin{1};
            else
                error('First parameter must be a valid ss object');
            end
            
            % dec
            if 1%add a check for decentralization
                obj.dec=varargin{2};
            else
                error('Second argument must be  a valid decentralization');
            end            
            
            % Ycon
            obj.Ycon=varargin{3};
            if ~isfield(obj.Ycon,'min') || ~isfield(obj.Ycon,'max')
                error(['Third argument must be a structure with fields ' ...
                    '''min'' and ''max''']);
            end
            
            % DeltaY
            if isnumeric(varargin{4})
                ny=size(obj.model.c,1);
                obj.DeltaY.max=ones(ny,1)*varargin{4};
                obj.DeltaY.min=-obj.DeltaY.max;
            else
                if ~isfield(varargin{4},'min') || ~isfield(varargin{4},'max')
                    error(['Fourth argument must be either numeric or a '... 
                        'structure as Third argument'])
                else
                    obj.DeltaY=varargin{4};
                end
            end
           
            % Xcon
            obj.Xcon=varargin{5};
            if ~isfield(obj.Xcon,'min') || ~isfield(obj.Xcon,'max')
                error(['Fifth argument must be a structure with fields '...
                    '''min'' and ''max''']);
            end

            % coupledCons
            if nargin>=6
                obj.coupledCons=varargin{6};
            else
                for i=1:length(obj.dec)
                    obj.coupledCons(i).H=[];
                    obj.coupledCons(i).K=[];
                end
            end
            
            if nargin==7
                obj.DeltaY2=varargin{7};
            end
            
            obj=obj.createSubSys();
        end

        %% Submodel-wise MOAS determination
        %
        % Compute the MOAS of all the submodels, no parameter is
        % required
        %
        function obj=computeMOAS(obj)
            obj=obj.computeCompIndices();
            obj=obj.computePnoise();
            
            dec=obj.dec;
            nDec=length(dec);
            for i=1:nDec
                %% 
                % Output constraint of the current state (in state dimension space,
                % which in the current example is exactly twice as the output)
                [H_i,K_i,kk]= obj.computeOutputInSpace(i);
                H_i=[H_i;obj.coupledCons(i).H];
                K_i=[K_i;obj.coupledCons(i).K*obj.DeltaY2];
                obj.Hyx{i}=H_i;
                obj.Kyx{i}=K_i;%[kk;obj.coupledCons(i).K];

                P=polytope(H_i,K_i);
                obj.P{i}=P;
                %%
                % Computes the MOAS
                disp('computing the MOAS...')
                [Oinf,tstar,fd,isemptypoly] = mpt_infset(obj.Ai{i},...
                    obj.P{i},1e2,obj.Pnoise{i});
                obj.Oinf{i}=Oinf;
                disp(Oinf);
                %%
                % Inv set inequatily description
                [obj.Ho{i},obj.Ko{i}]=double(Oinf);
                obj.nKo(i)=length(obj.Ko{i});
                
                if length(obj.comp)
                    [Oinf,tstar,fd,isemptypoly] = mpt_infset(obj.Ai{i},...
                        obj.P{i},1e2);
                    obj.OinfNoNoise{i}=Oinf;
                else
                    obj.OinfNoNoise{i}=obj.Oinf{i};
                end
                    
            end
        end
        
        %% Plot of all MOAS
        %
        % Plot a figure with as many subfigures as the submodels each
        % showing in red the MOAS with no disturbance and in blue the
        % real MOAS.
        %        
        function plotMOAS(obj)
            figure;
            nDec=length(obj.dec);
            for i=1:nDec
                subplot(ceil(nDec/2),ceil(nDec/2),i), ...
                    plot([obj.OinfNoNoise{i} obj.Oinf{i}])
                title(['Plot of the MOAS of the ' num2str(i) '-th controller'])
                axis equal
            end
        end
        
        %% Compute DeltaR and DeltaR/N for each sub-model
        %
        % Computes the maximum reference variation of each sub-model as
        % a function of the sample time ratio, also with sample time
        % normalization. No parameter is required.
        %
        function obj = computeDeltaR(obj,DrMax)
            nDec = length(obj.dec);
            if isnumeric(DrMax)
                DrMax=ones(nDec,1)*DrMax;
            end
            %%
            % For each sub-model
            for j = 1:nDec
                obj.Dr{j}=zeros(DrMax(j),1);
                Nh=0;
                sol.problem=0;
                while (Nh<=DrMax(j)) & (sol.problem==0)
                    Nh=Nh+1;
                    %%
                    % Variable declaration
                    [nXi(j),nYi(j)]=size(obj.Bi{j});                    
                    %%
                    % Continuos
                    epsilon=sdpvar(1);
                    x0=sdpvar(nXi(j),1);
                    r1=sdpvar(nYi(j),1);
                    r2=sdpvar(nYi(j),1);
                    %%
                    % Binary
                    delta = binvar(obj.nKo(j),1);
                    %%
                    % DC gain
                    Gx= (eye(nXi(j))-obj.Ai{j})\obj.Bi{j};
                    %%
                    % Constraints
                    F=[];
                    %%
                    % Epsilon = max(r1-r2)
                    F=set(epsilon>=r1(1)-r2(1));
                    F=F + set(epsilon>=-r1(1)+r2(1));
                    for i=2:nYi(j)
                        F = F + set(epsilon>=r1(i)-r2(i));
                        F = F + set(epsilon>=-r1(i)+r2(i));
                    end
                    %%
                    % xN is the state at next sample time of the 
                    % hierarchical, that is after Nh
                    % samples of the fast model
                    RN=zeros(size(Gx));
                    AN=eye(size(obj.Ai{j}));
                    for i = 0: Nh-1
                        RN = RN +AN*obj.Bi{j};
                        AN=AN*obj.Ai{j};
                    end
                    xN = AN*x0+RN*r2;
                    %%
                    % x(0) in omega(r1)
                    xr1 = Gx*r1;
                    F = F + set(obj.Ho{j}*(x0-xr1)<=obj.Ko{j});
                    %%
                    % xr2
                    xr2=Gx*r2;
                    %%
                    % Big M
                    [M,m] = findBigM(obj,j,AN,Gx,RN);
                    M=M+1;
                    m=m-1;
                    %%
                    % xN NOT in omega(r2)
                    HoRow=obj.Ho{j}*(xN-xr2); %support variable
                    for i=1:obj.nKo(j)
                        %%
                        % big M
                        F=F+set(HoRow(i)-obj.Ko{j}(i)<=M*(1-delta(i)));
                        F=F+set(-HoRow(i)+obj.Ko{j}(i)<=-m*delta(i)-1e-4);
                    end
                    %%
                    % One constraint must be violated in xN
                    F = F + set(sum(delta)<=obj.nKo(j)-1);
                    %%
                    % r1,r2 in (Ho,Ko) polytope
                    sup=obj.model.C(obj.dec(j).y,obj.dec(j).x)*obj.dec(j).x';
                    if size(obj.coupledCons(j).K,1)>0
                        HHy=[obj.Hy{j};obj.coupledCons(j).H(sup)];                        
                    else
                        HHy=obj.Hy{j};
                    end
                    KKy=[obj.Ky{j};obj.coupledCons(j).K*obj.DeltaY2];
                    DDy=[obj.DeltaY.max(obj.dec(j).y);...
                        obj.DeltaY.max(obj.dec(j).y);...
                        zeros(size(obj.coupledCons(j).K,1))];
                    F = F + set(HHy*r1<=KKy-DDy);
                    F = F + set(HHy*r2<=KKy-DDy);
                    
                    %%
                    % Solver setup
                    sdpopt = sdpsettings('solver','cplex','verbose',1,...
                        'showprogress',1,'debug',0);
                    disp(['Time needed for the ' num2str(Nh) ...
                        '-th problem of the ' num2str(j) ' subsystem']);
                    tic
                    sol = solvesdp(F,epsilon,sdpopt);
                    toc
                    xx0{j,Nh}=double(x0);
                    rr1{j,Nh}=double(r1);
                    rr2{j,Nh}=double(r2);
                    if sol.problem==0
                        obj.Dr{j}(Nh) = max(abs(rr1{j,Nh}-rr2{j,Nh}));
                        disp(['DeltaR=' num2str(obj.Dr{j}(Nh))]);
                    else
                        disp(['...DeltaR=' num2str(max(abs(rr1{j,Nh}-...
                            rr2{j,Nh})))]);
                    end
                    

                end
                for i=Nh-1:-1:2
                    obj.Dr{j}(i-1)=min(obj.Dr{j}(i),obj.Dr{j}(i-1));
                end
                nNot0 = max(find(obj.Dr{j}));
                obj.DrN{j}=obj.Dr{j}./[1:nNot0 ones(1,DrMax(j)-nNot0)]';
                %clc
            end
        end

        %% Plot DeltaR and DeltaR/N for each sub-model
        % Plot both $\Delta r(N)$ and the ratio $\Delta r(N)/N$ over the
        % sample times ratio N.
        %
        function plotDeltaR(obj)
            nDec=length(obj.dec);
            figure;
            for i=1:nDec,
                subplot(2,2,i),
                plot(obj.Dr{i});
                title(['DeltaR of ' num2str(i) '-th subsystem']);
            end
            figure
            for i=1:nDec,
                subplot(2,2,i),
                plot(obj.DrN{i});
                title(['DeltaR/N of ' num2str(i) '-th subsystem']);
            end
        end

    end
    
    methods (Access=private)
        
        function obj = createSubSys(obj)
            dec=obj.dec;
            nDec=length(dec);
            A=obj.model.a;
            B=obj.model.b;
            Ymax=obj.Ycon.max;
            Ymin=obj.Ycon.min;
            for i=1:nDec
                nYi = length(dec(i).y);
                %subsystem closed loop matrix
                obj.Ai{i}=A(dec(i).x,dec(i).x);
                obj.Bi{i}=B(dec(i).x,dec(i).u);
                PP = polytope([-eye(nYi);eye(nYi)],[-Ymin(dec(i).y);...
                    Ymax(dec(i).y)]);
                obj.Py=PP;
                [HH,KK]=double(PP);
                obj.Hy{i}=HH;
                obj.Ky{i}=KK;
            end
        end
        
        function obj = computeCompIndices(obj)
            dec=obj.dec;
            nDec=length(dec);
            for i=1:nDec
                obj.comp{i}.x=[];
                obj.comp{i}.u=[];
                for j=1:nDec %for each sub-model
                    if j~=i %if is not the current
                        % for each component in the sub-model
                        for k=1:length(dec(j).x) 
                            if ~length(find(dec(i).x==dec(j).x(k)))
                                if ~length(find(obj.comp{i}.x==dec(j).x(k)))
                                    obj.comp{i}.x=[obj.comp{i}.x dec(j).x(k)];
                                end
                            end
                        end
                        for k=1:length(dec(j).u) 
                            if ~length(find(dec(i).u==dec(j).u(k)))
                                if ~length(find(obj.comp{i}.u==dec(j).u(k)))
                                    obj.comp{i}.u=[obj.comp{i}.u dec(j).u(k)];
                                end
                            end
                        end
                    end
                end
            end
        end
        
        function obj = computePnoise(obj)
            A=obj.model.a;
            B=obj.model.b;
            Xmin=obj.Xcon.min;
            Xmax=obj.Xcon.max;
            nDec=length(obj.dec);
            for i=1:nDec
                comp=obj.comp{i}.x;
                compU=obj.comp{i}.u;
                nXn = length(comp);
                if nXn>0
                    % tildeA
                    H_i_x=zeros(2*nXn,nXn);
                    K_i_x=[];
                    for j=1:length(comp)
                        H_i_x(2*(j-1)+1,find(comp==comp(j))) =  1;
                        K_i_x(end+1,1)=Xmax(comp(j));
                        H_i_x(2*j,find(comp==comp(j)))= -1;
                        K_i_x(end+1,1)=-Xmin(comp(j));
                    end
                    P_i_x{i}=polytope(H_i_x,K_i_x);
                    v_i_x=extreme(P_i_x{i});

                    %influence to the current state
                    nAi=A(obj.dec(i).x,comp);
                    vv=nAi*v_i_x';
                    %MPT toolbox is incapable of handling 0-volume polytopes
                    vv = vv' + 1e-5*(rand(size(vv'))-.5);

                    %polytope containing the disturbance
                    Pnoise = hull(vv);
                    
                    % Add influence of tildeB (bounds on u required)
                    if 1
                        % the set of inputs is the set of references, which
                        % implies that we have to cut R
                        str='HH=blkdiag(';
                        KK=[];
                        for ii=1:nDec
                            if ii~=i
                                str=[str 'obj.Hy{' num2str(ii) '},'];
                                DY=[-obj.DeltaY.min(obj.dec(ii).y);...
                                    obj.DeltaY.max(obj.dec(ii).y)];
                                KK=[KK;obj.Ky{ii}-DY];
                            end
                        end
                        str(end:end+1)=');';
                        eval(str);
                        P_i_u{i} = polytope(HH,KK);
                        v_i_u = extreme(P_i_u{i});

                        %influence to the current state
                        nBi=B(obj.dec(i).x,compU);
                        vv=nBi*v_i_u';
                        %MPT toolbox is incapable of handling 0-volume polytopes
                        s=size(vv');
                        vv = vv';
                        nn=length(obj.dec(i).x);
                        v2=extreme(polytope([eye(nn);-eye(nn)],1e-6*ones(2*nn,1)));
                        vv=[(vv(1,:)'*ones(1,nn^2))'+v2;vv(2:end,:)];
                        pp=hull(vv);
                        Pnoise = Pnoise + pp;
                    end
                    
                else
                    Pnoise=[];
                end
                obj.Pnoise{i}=Pnoise;
            end
        end
       
        function [H_i,K_i,kk]= computeOutputInSpace(obj,i)
            dec=obj.dec;
            DeltaYmax=obj.DeltaY.max;
            DeltaYmin=obj.DeltaY.min;
            nYi = length(dec(i).y);
            H_i=zeros(2*nYi,nYi*2);
            K_i=[];
            kk=[];
            for j=1:nYi
                %% Vecchio
%                 H_i(2*(j-1)+1,2*find(dec(i).y==dec(i).y(j))-1) =  1;
%                 K_i(end+1,1)=...obj.Ycon.max(dec(i).y(j))-
%                     DeltaYmax(dec(i).y(j));
%                 kk(end+1,1)=obj.Ycon.max(dec(i).y(j))-DeltaYmax(dec(i).y(j));
%                 H_i(2*j,2*find(dec(i).y==dec(i).y(j))-1)= -1;
%                 K_i(end+1,1)=...obj.Ycon.min(dec(i).y(j))
%                     -DeltaYmin(dec(i).y(j));
%                 kk(end+1,1)=obj.Ycon.min(dec(i).y(j))-DeltaYmin(dec(i).y(j));
                
                %% Nuovo
                H_i(j+nYi,2*find(dec(i).y==dec(i).y(j))-1) =  1;
                K_i(j+nYi,1)=...obj.Ycon.max(dec(i).y(j))-
                    DeltaYmax(dec(i).y(j));
                kk(end+1,1)=obj.Ycon.max(dec(i).y(j))-DeltaYmax(dec(i).y(j));
                H_i(j,2*find(dec(i).y==dec(i).y(j))-1)= -1;
                K_i(j,1)=...obj.Ycon.min(dec(i).y(j))
                    -DeltaYmin(dec(i).y(j));
                kk(end+1,1)=obj.Ycon.min(dec(i).y(j))-DeltaYmin(dec(i).y(j));
            end
        end
        
        function [M,m] = findBigM(obj,j,AN,Gx,RN)
            ind=obj.model.C(obj.dec(j).y,obj.dec(j).x)*obj.dec(j).x';
            if size(obj.coupledCons(j).H,1)>0
                H=[obj.Hy{j}; obj.coupledCons(j).H(ind)];
            else
                H=obj.Hy{j};
            end
            clear ind
            K=[obj.Ky{j}; obj.coupledCons(j).K];
            Ho=obj.Ho{j};
            Ko=obj.Ko{j};
            DeltaY=obj.DeltaY.max(1);

            % big M setup
            % define XX and barA
            sH=size(H);
            sHo=size(Ho);
            barA = [Ho*AN zeros(sHo(1),sH(2)) Ho*(RN-Gx)];
            XX=[];
            XX.H=[Ho, -Ho*Gx ,zeros(sHo(1),sH(2));
                zeros(sH(1),sHo(2)), H, zeros(sH(1),sH(2));
                zeros(sH(1),sHo(2)),zeros(sH(1),sH(2)), H];

            XX.K=[Ko;K-DeltaY;K-DeltaY];
            [M,m]=bigM(barA,Ko,XX);
            M=M+1e-2;
            m=m-1e-2;
        end
        
        function [M,m] = bigM(obj,A,b,XX)
            % Calculates the solution of following LP problem
            %
            % element-wise max/min A*x-b
            %            x  st. x in XX
            %
            %
            % for XX being a structure containing fields .H .K
            
            [n,m]=size(A);
            
            x = sdpvar(m,1);

            
            M=zeros(n,1);
            m=zeros(n,1);
            
            for i=1:n
                x=lpsol(-A(i,:),XX.H,XX.K);
                M(i) = A(i,:)*x-b(i);
                if nargout>1,
                    x=lpsol(A(i,:),XX.H,XX.K);
                    m(i) = A(i,:)*x-b(i);
                end
            end
        end
        
    end
end